/****************************************************************************
** Meta object code from reading C++ file 'UiPluginModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/UiPluginModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UiPluginModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13UiPluginModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::UiPluginModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13UiPluginModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::UiPluginModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireShowRequest",
        "",
        "UiModule",
        "pModule",
        "fireHideRequest",
        "fireDominatorChanged",
        "fireHighContrastEnabledChanged",
        "fireDarkModeEnabledChanged",
        "fireProxyAuthenticationRequired",
        "ProxyCredentials*",
        "pProxyCredentials",
        "fireShowFocusIndicator",
        "fireScaleFactorChanged",
        "fireFontScaleFactorChanged",
        "fireFontWeightAdjustmentChanged",
        "fireIsUpdatePendingChanged",
        "fireA11yButtonShapeActiveChanged",
        "fireA11yOnOffSwitchLabelActiveChanged",
        "setUpdatePending",
        "pNewIsUpdatePending",
        "hideFromTaskbar",
        "doRefresh",
        "showUpdateInformationIfPending",
        "qtVersion",
        "debugBuild",
        "developerVersion",
        "dominator",
        "dominated",
        "highContrastEnabled",
        "darkModeEnabled",
        "fixedFontFamily",
        "initialWindowSize",
        "QSize",
        "showFocusIndicator",
        "scaleFactor",
        "fontScaleFactor",
        "fontWeightAdjustment",
        "isChromeOS",
        "isUpdatePending",
        "a11yButtonShapeActive",
        "a11yOnOffSwitchLabelActive"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireShowRequest'
        QtMocHelpers::SignalData<void(UiModule)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'fireHideRequest'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDominatorChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireHighContrastEnabledChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDarkModeEnabledChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireProxyAuthenticationRequired'
        QtMocHelpers::SignalData<void(ProxyCredentials *)>(13, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'fireShowFocusIndicator'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireScaleFactorChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireFontScaleFactorChanged'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireFontWeightAdjustmentChanged'
        QtMocHelpers::SignalData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireIsUpdatePendingChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireA11yButtonShapeActiveChanged'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireA11yOnOffSwitchLabelActiveChanged'
        QtMocHelpers::SignalData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setUpdatePending'
        QtMocHelpers::SlotData<void(bool)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Method 'hideFromTaskbar'
        QtMocHelpers::MethodData<void() const>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'doRefresh'
        QtMocHelpers::MethodData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'showUpdateInformationIfPending'
        QtMocHelpers::MethodData<bool()>(27, 6, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'qtVersion'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'debugBuild'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'developerVersion'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dominator'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'dominated'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'highContrastEnabled'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'darkModeEnabled'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'fixedFontFamily'
        QtMocHelpers::PropertyData<QString>(35, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'initialWindowSize'
        QtMocHelpers::PropertyData<QSize>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'showFocusIndicator'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'scaleFactor'
        QtMocHelpers::PropertyData<qreal>(39, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'fontScaleFactor'
        QtMocHelpers::PropertyData<qreal>(40, QMetaType::QReal, QMC::DefaultPropertyFlags, 8),
        // property 'fontWeightAdjustment'
        QtMocHelpers::PropertyData<int>(41, QMetaType::Int, QMC::DefaultPropertyFlags, 9),
        // property 'isChromeOS'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'isUpdatePending'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'a11yButtonShapeActive'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'a11yOnOffSwitchLabelActive'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<UiPluginModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::UiPluginModel::staticMetaObject = { {
    QMetaObject::SuperData::link<UiPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13UiPluginModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13UiPluginModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13UiPluginModelE_t>.metaTypes,
    nullptr
} };

void governikus::UiPluginModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UiPluginModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireShowRequest((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 1: _t->fireHideRequest(); break;
        case 2: _t->fireDominatorChanged(); break;
        case 3: _t->fireHighContrastEnabledChanged(); break;
        case 4: _t->fireDarkModeEnabledChanged(); break;
        case 5: _t->fireProxyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<ProxyCredentials*>>(_a[1]))); break;
        case 6: _t->fireShowFocusIndicator(); break;
        case 7: _t->fireScaleFactorChanged(); break;
        case 8: _t->fireFontScaleFactorChanged(); break;
        case 9: _t->fireFontWeightAdjustmentChanged(); break;
        case 10: _t->fireIsUpdatePendingChanged(); break;
        case 11: _t->fireA11yButtonShapeActiveChanged(); break;
        case 12: _t->fireA11yOnOffSwitchLabelActiveChanged(); break;
        case 13: _t->setUpdatePending((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->hideFromTaskbar(); break;
        case 15: _t->doRefresh(); break;
        case 16: { bool _r = _t->showUpdateInformationIfPending();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ProxyCredentials* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)(UiModule )>(_a, &UiPluginModel::fireShowRequest, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireHideRequest, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireDominatorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireHighContrastEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireDarkModeEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)(ProxyCredentials * )>(_a, &UiPluginModel::fireProxyAuthenticationRequired, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireShowFocusIndicator, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireScaleFactorChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireFontScaleFactorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireFontWeightAdjustmentChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireIsUpdatePendingChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireA11yButtonShapeActiveChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginModel::*)()>(_a, &UiPluginModel::fireA11yOnOffSwitchLabelActiveChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getQtVersion(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isDebugBuild(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isDeveloperVersion(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getDominator(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isDominated(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isHighContrastEnabled(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isDarkModeEnabled(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getFixedFontFamily(); break;
        case 8: *reinterpret_cast<QSize*>(_v) = _t->getInitialWindowSize(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->getShowFocusIndicator(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->getScaleFactor(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->getFontScaleFactor(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->getFontWeightAdjustment(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isChromeOS(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isUpdatePending(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isA11yButtonShapeActive(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->isA11yOnOffSwitchLabelActive(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 10: _t->setScaleFactor(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::UiPluginModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::UiPluginModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13UiPluginModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "governikus.UiPlugin"))
        return static_cast< governikus::UiPlugin*>(this);
    return UiPlugin::qt_metacast(_clname);
}

int governikus::UiPluginModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UiPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void governikus::UiPluginModel::fireShowRequest(UiModule _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::UiPluginModel::fireHideRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::UiPluginModel::fireDominatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::UiPluginModel::fireHighContrastEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::UiPluginModel::fireDarkModeEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::UiPluginModel::fireProxyAuthenticationRequired(ProxyCredentials * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void governikus::UiPluginModel::fireShowFocusIndicator()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::UiPluginModel::fireScaleFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::UiPluginModel::fireFontScaleFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::UiPluginModel::fireFontWeightAdjustmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::UiPluginModel::fireIsUpdatePendingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void governikus::UiPluginModel::fireA11yButtonShapeActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::UiPluginModel::fireA11yOnOffSwitchLabelActiveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
