// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>



QT_DECLARE_EXTERN_RESOURCE(qmake_Governikus_View)
QT_DECLARE_EXTERN_RESOURCE(AusweisAppUiQmlModuleLinuxView_raw_qml_0)


QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_Governikus_View)

class Governikus_ViewPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    Governikus_ViewPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_Governikus_View)

QT_KEEP_RESOURCE(qmake_Governikus_View)
QT_KEEP_RESOURCE(AusweisAppUiQmlModuleLinuxView_raw_qml_0)
    }
};



#include "AusweisAppUiQmlModuleLinuxViewplugin_Governikus_ViewPlugin.moc"
