/****************************************************************************
** Meta object code from reading C++ file 'WorkflowContext.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/workflows/base/context/WorkflowContext.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'WorkflowContext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus15WorkflowContextE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::WorkflowContext::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15WorkflowContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::WorkflowContext",
        "fireStateApprovedChanged",
        "",
        "pApproved",
        "fireStateChanged",
        "pNewState",
        "fireReaderPluginTypesChanged",
        "pExplicitStart",
        "fireReaderInfoChanged",
        "fireReaderNameChanged",
        "fireCardConnectionChanged",
        "fireAcceptedEidTypesChanged",
        "fireEidTypeMismatchChanged",
        "fireCanChanged",
        "firePinChanged",
        "firePukChanged",
        "firePaceResultUpdated",
        "fireResultChanged",
        "fireCanAllowedModeChanged",
        "firePasswordTypeChanged",
        "fireProgressChanged",
        "fireRemoveCardFeedbackChanged",
        "fireInitialInputErrorShownChanged",
        "fireCancelWorkflow",
        "fireNextWorkflowPending",
        "onWorkflowCancelled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireStateApprovedChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'fireStateChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'fireReaderPluginTypesChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'fireReaderPluginTypesChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'fireReaderInfoChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireReaderNameChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCardConnectionChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAcceptedEidTypesChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireEidTypeMismatchChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCanChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePinChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePukChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePaceResultUpdated'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireResultChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCanAllowedModeChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePasswordTypeChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireProgressChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRemoveCardFeedbackChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireInitialInputErrorShownChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCancelWorkflow'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireNextWorkflowPending'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onWorkflowCancelled'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkflowContext, qt_meta_tag_ZN10governikus15WorkflowContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::WorkflowContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15WorkflowContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15WorkflowContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus15WorkflowContextE_t>.metaTypes,
    nullptr
} };

void governikus::WorkflowContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkflowContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireStateApprovedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->fireStateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->fireReaderPluginTypesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->fireReaderPluginTypesChanged(); break;
        case 4: _t->fireReaderInfoChanged(); break;
        case 5: _t->fireReaderNameChanged(); break;
        case 6: _t->fireCardConnectionChanged(); break;
        case 7: _t->fireAcceptedEidTypesChanged(); break;
        case 8: _t->fireEidTypeMismatchChanged(); break;
        case 9: _t->fireCanChanged(); break;
        case 10: _t->firePinChanged(); break;
        case 11: _t->firePukChanged(); break;
        case 12: _t->firePaceResultUpdated(); break;
        case 13: _t->fireResultChanged(); break;
        case 14: _t->fireCanAllowedModeChanged(); break;
        case 15: _t->firePasswordTypeChanged(); break;
        case 16: _t->fireProgressChanged(); break;
        case 17: _t->fireRemoveCardFeedbackChanged(); break;
        case 18: _t->fireInitialInputErrorShownChanged(); break;
        case 19: _t->fireCancelWorkflow(); break;
        case 20: _t->fireNextWorkflowPending(); break;
        case 21: _t->onWorkflowCancelled(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)(bool )>(_a, &WorkflowContext::fireStateApprovedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)(const QString & )>(_a, &WorkflowContext::fireStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)(bool )>(_a, &WorkflowContext::fireReaderPluginTypesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireReaderInfoChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireReaderNameChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireCardConnectionChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireAcceptedEidTypesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireEidTypeMismatchChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireCanChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::firePinChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::firePukChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::firePaceResultUpdated, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireResultChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireCanAllowedModeChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::firePasswordTypeChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireProgressChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireRemoveCardFeedbackChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireInitialInputErrorShownChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireCancelWorkflow, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowContext::*)()>(_a, &WorkflowContext::fireNextWorkflowPending, 20))
            return;
    }
}

const QMetaObject *governikus::WorkflowContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::WorkflowContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15WorkflowContextE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::WorkflowContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void governikus::WorkflowContext::fireStateApprovedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::WorkflowContext::fireStateChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::WorkflowContext::fireReaderPluginTypesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 4
void governikus::WorkflowContext::fireReaderInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::WorkflowContext::fireReaderNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::WorkflowContext::fireCardConnectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::WorkflowContext::fireAcceptedEidTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::WorkflowContext::fireEidTypeMismatchChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::WorkflowContext::fireCanChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::WorkflowContext::firePinChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void governikus::WorkflowContext::firePukChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::WorkflowContext::firePaceResultUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void governikus::WorkflowContext::fireResultChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void governikus::WorkflowContext::fireCanAllowedModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void governikus::WorkflowContext::firePasswordTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void governikus::WorkflowContext::fireProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void governikus::WorkflowContext::fireRemoveCardFeedbackChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void governikus::WorkflowContext::fireInitialInputErrorShownChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void governikus::WorkflowContext::fireCancelWorkflow()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void governikus::WorkflowContext::fireNextWorkflowPending()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}
namespace {
struct qt_meta_tag_ZN10governikus10EnumActionE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumAction::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus10EnumActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumAction",
        "Action",
        "AUTH",
        "SELF_AUTH",
        "CHANGE_PIN",
        "REMOTE_SERVICE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Action'
        QtMocHelpers::EnumData<Action>(1, 1, QMC::EnumIsScoped).add({
            {    2, Action::AUTH },
            {    3, Action::SELF_AUTH },
            {    4, Action::CHANGE_PIN },
            {    5, Action::REMOTE_SERVICE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus10EnumActionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus10EnumActionE =
    governikus::EnumAction::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus10EnumActionE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus10EnumActionE =
    qt_staticMetaObjectContent_ZN10governikus10EnumActionE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus10EnumActionE =
    qt_staticMetaObjectContent_ZN10governikus10EnumActionE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumAction::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus10EnumActionE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus10EnumActionE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus10EnumActionE.metaTypes,
    nullptr
} };

QT_WARNING_POP
