/****************************************************************************
** Meta object code from reading C++ file 'AuthModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/AuthModel.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AuthModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus9AuthModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::AuthModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus9AuthModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::AuthModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireChangeTransportPinChanged",
        "",
        "fireTransactionInfoChanged",
        "fireProgressChanged",
        "fireRefreshUrlChanged",
        "onDidAuthenticateEac1Changed",
        "cancelWorkflowToChangeTransportPin",
        "fireResultChanged",
        "transactionInfo",
        "progressValue",
        "progressMessage",
        "changeTransportPin",
        "resultHeader",
        "errorHeader",
        "errorText",
        "resultViewButtonIcon",
        "resultViewButtonText",
        "resultViewButtonLink",
        "QUrl",
        "refreshUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireChangeTransportPinChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireTransactionInfoChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireProgressChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRefreshUrlChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDidAuthenticateEac1Changed'
        QtMocHelpers::SlotData<void()>(10, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'cancelWorkflowToChangeTransportPin'
        QtMocHelpers::MethodData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'transactionInfo'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'progressValue'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags, 2),
        // property 'progressMessage'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'changeTransportPin'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'resultHeader'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 0x70000000 | 12),
        // property 'errorHeader'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags, 0x70000000 | 12),
        // property 'errorText'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 0x70000000 | 12),
        // property 'resultViewButtonIcon'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 0x70000000 | 12),
        // property 'resultViewButtonText'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 0x70000000 | 12),
        // property 'resultViewButtonLink'
        QtMocHelpers::PropertyData<QUrl>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0x70000000 | 12),
        // property 'refreshUrl'
        QtMocHelpers::PropertyData<QUrl>(24, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<AuthModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::AuthModel::staticMetaObject = { {
    QMetaObject::SuperData::link<WorkflowModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9AuthModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9AuthModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus9AuthModelE_t>.metaTypes,
    nullptr
} };

void governikus::AuthModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AuthModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireChangeTransportPinChanged(); break;
        case 1: _t->fireTransactionInfoChanged(); break;
        case 2: _t->fireProgressChanged(); break;
        case 3: _t->fireRefreshUrlChanged(); break;
        case 4: _t->onDidAuthenticateEac1Changed(); break;
        case 5: _t->cancelWorkflowToChangeTransportPin(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AuthModel::*)()>(_a, &AuthModel::fireChangeTransportPinChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthModel::*)()>(_a, &AuthModel::fireTransactionInfoChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthModel::*)()>(_a, &AuthModel::fireProgressChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AuthModel::*)()>(_a, &AuthModel::fireRefreshUrlChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getTransactionInfo(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->getProgressValue(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getProgressMessage(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->getChangeTransportPin(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getResultHeader(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getErrorHeader(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getErrorText(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getResultViewButtonIcon(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getResultViewButtonText(); break;
        case 9: *reinterpret_cast<QUrl*>(_v) = _t->getResultViewButtonLink(); break;
        case 10: *reinterpret_cast<QUrl*>(_v) = _t->getRefreshUrl(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::AuthModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::AuthModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9AuthModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<AuthModel>"))
        return static_cast< SingletonCreator<AuthModel>*>(this);
    return WorkflowModel::qt_metacast(_clname);
}

int governikus::AuthModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WorkflowModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void governikus::AuthModel::fireChangeTransportPinChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::AuthModel::fireTransactionInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::AuthModel::fireProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::AuthModel::fireRefreshUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace CheckNotifySignalValidity_ZN10governikus9AuthModelE {
template<typename T> using has_nullary_fireResultChanged = decltype(std::declval<T>().fireResultChanged());
template<typename T> using has_unary_fireResultChanged = decltype(std::declval<T>().fireResultChanged(std::declval<QString>()));
static_assert(qxp::is_detected_v<has_nullary_fireResultChanged, governikus::AuthModel> || qxp::is_detected_v<has_unary_fireResultChanged, governikus::AuthModel>,
              "NOTIFY signal fireResultChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
