/****************************************************************************
** Meta object code from reading C++ file 'PinResetInformationModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/PinResetInformationModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PinResetInformationModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus24PinResetInformationModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::PinResetInformationModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus24PinResetInformationModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::PinResetInformationModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireUpdated",
        "",
        "onTranslationChanged",
        "getResetPinWithPRSHint",
        "pHasPRS",
        "hasPinResetService",
        "pinResetUrl",
        "QUrl",
        "pinResetActivationUrl",
        "administrativeSearchUrl",
        "activateOnlineFunctionForPRSHint",
        "activateOnlineFunctionAtAuthorityHint",
        "activateOnlineFunctionDescription",
        "resetPinWithPRSActionText",
        "resetPinAtAuthorityActionText",
        "resetPinWithPRSHintTitle",
        "resetPinAtAuthorityHintTitle",
        "resetPinWithPRSHint",
        "resetPinAtAuthorityHint"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireUpdated'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'getResetPinWithPRSHint'
        QtMocHelpers::MethodData<QString(bool) const>(8, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Bool, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasPinResetService'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'pinResetUrl'
        QtMocHelpers::PropertyData<QUrl>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'pinResetActivationUrl'
        QtMocHelpers::PropertyData<QUrl>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'administrativeSearchUrl'
        QtMocHelpers::PropertyData<QUrl>(14, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'activateOnlineFunctionForPRSHint'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'activateOnlineFunctionAtAuthorityHint'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'activateOnlineFunctionDescription'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinWithPRSActionText'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinAtAuthorityActionText'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinWithPRSHintTitle'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinAtAuthorityHintTitle'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinWithPRSHint'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'resetPinAtAuthorityHint'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<PinResetInformationModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::PinResetInformationModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus24PinResetInformationModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus24PinResetInformationModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus24PinResetInformationModelE_t>.metaTypes,
    nullptr
} };

void governikus::PinResetInformationModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PinResetInformationModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireUpdated(); break;
        case 1: _t->onTranslationChanged(); break;
        case 2: { QString _r = _t->getResetPinWithPRSHint((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PinResetInformationModel::*)()>(_a, &PinResetInformationModel::fireUpdated, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasPinResetService(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->getPinResetUrl(); break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->getPinResetActivationUrl(); break;
        case 3: *reinterpret_cast<QUrl*>(_v) = _t->getAdministrativeSearchUrl(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getActivateOnlineFunctionForPRSHint(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getActivateOnlineFunctionAtAuthorityHint(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getActivateOnlineFunctionDescription(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getResetPinWithPRSActionText(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getResetPinAtAuthorityActionText(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getResetPinWithPRSHintTitle(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->getResetPinAtAuthorityHintTitle(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->getResetPinWithPRSHint(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->getResetPinAtAuthorityHint(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::PinResetInformationModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::PinResetInformationModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus24PinResetInformationModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<PinResetInformationModel>"))
        return static_cast< SingletonCreator<PinResetInformationModel>*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::PinResetInformationModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void governikus::PinResetInformationModel::fireUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
