/****************************************************************************
** Meta object code from reading C++ file 'ReaderManager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/ReaderManager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ReaderManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13ReaderManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::ReaderManager::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13ReaderManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::ReaderManager",
        "firePluginAdded",
        "",
        "ReaderManagerPluginInfo",
        "pInfo",
        "fireStatusChanged",
        "fireReaderAdded",
        "ReaderInfo",
        "fireReaderRemoved",
        "fireReaderPropertiesUpdated",
        "fireCardInserted",
        "fireCardRemoved",
        "fireCardInfoChanged",
        "fireInitialized",
        "fireInitialScanFinished",
        "doUpdateCacheEntry",
        "doRemoveCacheEntry",
        "doUpdatePluginCache",
        "shutdown"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'firePluginAdded'
        QtMocHelpers::SignalData<void(const ReaderManagerPluginInfo &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireStatusChanged'
        QtMocHelpers::SignalData<void(const ReaderManagerPluginInfo &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireReaderAdded'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireReaderRemoved'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireReaderPropertiesUpdated'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireCardInserted'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireCardRemoved'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireCardInfoChanged'
        QtMocHelpers::SignalData<void(const ReaderInfo &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Signal 'fireInitialized'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireInitialScanFinished'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doUpdateCacheEntry'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Slot 'doRemoveCacheEntry'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Slot 'doUpdatePluginCache'
        QtMocHelpers::SlotData<void(const ReaderManagerPluginInfo &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReaderManager, qt_meta_tag_ZN10governikus13ReaderManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::ReaderManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13ReaderManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13ReaderManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13ReaderManagerE_t>.metaTypes,
    nullptr
} };

void governikus::ReaderManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ReaderManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->firePluginAdded((*reinterpret_cast<std::add_pointer_t<ReaderManagerPluginInfo>>(_a[1]))); break;
        case 1: _t->fireStatusChanged((*reinterpret_cast<std::add_pointer_t<ReaderManagerPluginInfo>>(_a[1]))); break;
        case 2: _t->fireReaderAdded((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 3: _t->fireReaderRemoved((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 4: _t->fireReaderPropertiesUpdated((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 5: _t->fireCardInserted((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 6: _t->fireCardRemoved((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 7: _t->fireCardInfoChanged((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 8: _t->fireInitialized(); break;
        case 9: _t->fireInitialScanFinished(); break;
        case 10: _t->doUpdateCacheEntry((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 11: _t->doRemoveCacheEntry((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 12: _t->doUpdatePluginCache((*reinterpret_cast<std::add_pointer_t<ReaderManagerPluginInfo>>(_a[1]))); break;
        case 13: _t->shutdown(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderManagerPluginInfo & )>(_a, &ReaderManager::firePluginAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderManagerPluginInfo & )>(_a, &ReaderManager::fireStatusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireReaderAdded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireReaderRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireReaderPropertiesUpdated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireCardInserted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireCardRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)(const ReaderInfo & )>(_a, &ReaderManager::fireCardInfoChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)()>(_a, &ReaderManager::fireInitialized, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderManager::*)()>(_a, &ReaderManager::fireInitialScanFinished, 9))
            return;
    }
}

const QMetaObject *governikus::ReaderManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::ReaderManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13ReaderManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::ReaderManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void governikus::ReaderManager::firePluginAdded(const ReaderManagerPluginInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::ReaderManager::fireStatusChanged(const ReaderManagerPluginInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::ReaderManager::fireReaderAdded(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void governikus::ReaderManager::fireReaderRemoved(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void governikus::ReaderManager::fireReaderPropertiesUpdated(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void governikus::ReaderManager::fireCardInserted(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void governikus::ReaderManager::fireCardRemoved(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void governikus::ReaderManager::fireCardInfoChanged(const ReaderInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void governikus::ReaderManager::fireInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::ReaderManager::fireInitialScanFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
