/****************************************************************************
** Meta object code from reading C++ file 'AppController.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/core/AppController.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AppController.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13AppControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::AppController::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13AppControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::AppController",
        "fireInitialized",
        "",
        "fireStarted",
        "fireShutdown",
        "fireWorkflowStarted",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "fireWorkflowFinished",
        "fireWorkflowUnhandled",
        "fireShowUi",
        "UiModule",
        "pModule",
        "fireHideUi",
        "fireShowUserInformation",
        "pInformationMessage",
        "fireTranslationChanged",
        "fireProxyAuthenticationRequired",
        "QNetworkProxy",
        "pProxy",
        "QAuthenticator*",
        "pAuthenticator",
        "fireApplicationActivated",
        "fireSystemSettingsChanged",
        "fireUiDomination",
        "const UiPlugin*",
        "pUi",
        "pInformation",
        "pAccepted",
        "fireUiDominationReleased",
        "doShutdown",
        "pExitCode",
        "onUiPlugin",
        "pPlugin",
        "onWorkflowFinished",
        "onWorkflowRequested",
        "onCloseReminderFinished",
        "pDontRemindAgain",
        "onLanguageChanged",
        "onUiDominationRequested",
        "onUiDominationRelease",
        "onRestartApplicationRequested"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireInitialized'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireStarted'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireShutdown'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireWorkflowStarted'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'fireWorkflowFinished'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'fireWorkflowUnhandled'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'fireShowUi'
        QtMocHelpers::SignalData<void(UiModule)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'fireHideUi'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireShowUserInformation'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'fireTranslationChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireProxyAuthenticationRequired'
        QtMocHelpers::SignalData<void(const QNetworkProxy &, QAuthenticator *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Signal 'fireApplicationActivated'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSystemSettingsChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUiDomination'
        QtMocHelpers::SignalData<void(const UiPlugin *, const QString &, bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { QMetaType::QString, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Signal 'fireUiDominationReleased'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onUiPlugin'
        QtMocHelpers::SlotData<void(const UiPlugin *) const>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 33 },
        }}),
        // Slot 'onWorkflowFinished'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWorkflowRequested'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'onCloseReminderFinished'
        QtMocHelpers::SlotData<void(bool) const>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 37 },
        }}),
        // Slot 'onLanguageChanged'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onUiDominationRequested'
        QtMocHelpers::SlotData<void(const UiPlugin *, const QString &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 }, { QMetaType::QString, 27 },
        }}),
        // Slot 'onUiDominationRelease'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRestartApplicationRequested'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AppController, qt_meta_tag_ZN10governikus13AppControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::AppController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13AppControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13AppControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13AppControllerE_t>.metaTypes,
    nullptr
} };

void governikus::AppController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireInitialized(); break;
        case 1: _t->fireStarted(); break;
        case 2: _t->fireShutdown(); break;
        case 3: _t->fireWorkflowStarted((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 4: _t->fireWorkflowFinished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 5: _t->fireWorkflowUnhandled((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 6: _t->fireShowUi((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 7: _t->fireHideUi(); break;
        case 8: _t->fireShowUserInformation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->fireTranslationChanged(); break;
        case 10: _t->fireProxyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        case 11: _t->fireApplicationActivated(); break;
        case 12: _t->fireSystemSettingsChanged(); break;
        case 13: _t->fireUiDomination((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 14: _t->fireUiDominationReleased(); break;
        case 15: _t->doShutdown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->doShutdown(); break;
        case 17: _t->onUiPlugin((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1]))); break;
        case 18: _t->onWorkflowFinished(); break;
        case 19: _t->onWorkflowRequested((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 20: _t->onCloseReminderFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->onLanguageChanged(); break;
        case 22: _t->onUiDominationRequested((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 23: _t->onUiDominationRelease(); break;
        case 24: _t->onRestartApplicationRequested(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkProxy >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireInitialized, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireStarted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireShutdown, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &AppController::fireWorkflowStarted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &AppController::fireWorkflowFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &AppController::fireWorkflowUnhandled, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(UiModule )>(_a, &AppController::fireShowUi, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireHideUi, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const QString & )>(_a, &AppController::fireShowUserInformation, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireTranslationChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const QNetworkProxy & , QAuthenticator * )>(_a, &AppController::fireProxyAuthenticationRequired, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireApplicationActivated, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireSystemSettingsChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)(const UiPlugin * , const QString & , bool )>(_a, &AppController::fireUiDomination, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppController::*)()>(_a, &AppController::fireUiDominationReleased, 14))
            return;
    }
}

const QMetaObject *governikus::AppController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::AppController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13AppControllerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QAbstractNativeEventFilter"))
        return static_cast< QAbstractNativeEventFilter*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::AppController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void governikus::AppController::fireInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::AppController::fireStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::AppController::fireShutdown()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::AppController::fireWorkflowStarted(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void governikus::AppController::fireWorkflowFinished(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void governikus::AppController::fireWorkflowUnhandled(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void governikus::AppController::fireShowUi(UiModule _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void governikus::AppController::fireHideUi()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::AppController::fireShowUserInformation(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void governikus::AppController::fireTranslationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::AppController::fireProxyAuthenticationRequired(const QNetworkProxy & _t1, QAuthenticator * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void governikus::AppController::fireApplicationActivated()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::AppController::fireSystemSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void governikus::AppController::fireUiDomination(const UiPlugin * _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3);
}

// SIGNAL 14
void governikus::AppController::fireUiDominationReleased()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
