/****************************************************************************
** Meta object code from reading C++ file 'GlobalStatus.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/global/GlobalStatus.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GlobalStatus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus12GlobalStatusE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::GlobalStatus::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12GlobalStatusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::GlobalStatus",
        "Origin",
        "Server",
        "Client",
        "ExternalInformation",
        "ECARDAPI_ERROR",
        "ECARDAPI_SERVERMESSAGE",
        "LAST_URL",
        "HTTP_STATUS_CODE",
        "REDIRECT_URL",
        "CERTIFICATE_ISSUER_NAME",
        "URL_SCHEME",
        "ACTIVATION_ERROR"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Origin'
        QtMocHelpers::EnumData<enum Origin>(1, 1, QMC::EnumIsScoped).add({
            {    2, Origin::Server },
            {    3, Origin::Client },
        }),
        // enum 'ExternalInformation'
        QtMocHelpers::EnumData<enum ExternalInformation>(4, 4, QMC::EnumIsScoped).add({
            {    5, ExternalInformation::ECARDAPI_ERROR },
            {    6, ExternalInformation::ECARDAPI_SERVERMESSAGE },
            {    7, ExternalInformation::LAST_URL },
            {    8, ExternalInformation::HTTP_STATUS_CODE },
            {    9, ExternalInformation::REDIRECT_URL },
            {   10, ExternalInformation::CERTIFICATE_ISSUER_NAME },
            {   11, ExternalInformation::URL_SCHEME },
            {   12, ExternalInformation::ACTIVATION_ERROR },
        }),
    };
    return QtMocHelpers::metaObjectData<GlobalStatus, qt_meta_tag_ZN10governikus12GlobalStatusE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::GlobalStatus::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus12GlobalStatusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus12GlobalStatusE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus12GlobalStatusE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus20EnumGlobalStatusCodeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumGlobalStatusCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumGlobalStatusCodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumGlobalStatusCode",
        "QML.Element",
        "GlobalStatusCode",
        "Unknown_Error",
        "No_Error",
        "Network_ServiceUnavailable",
        "Network_ServerError",
        "Network_ClientError",
        "Network_Ssl_Establishment_Error",
        "Network_TimeOut",
        "Network_Proxy_Error",
        "Network_Other_Error",
        "Downloader_File_Not_Found",
        "Downloader_Cannot_Save_File",
        "Downloader_Data_Corrupted",
        "Downloader_Missing_Platform",
        "Downloader_Aborted",
        "Update_Execution_Failed",
        "Workflow_AlreadyInProgress_Error",
        "Workflow_Communication_Missing_Redirect_Url",
        "Workflow_InternalError_BeforeTcToken",
        "Workflow_Cancellation_By_User",
        "Workflow_Card_Removed",
        "Workflow_Cannot_Confirm_IdCard_Authenticity",
        "Workflow_Unknown_Paos_From_EidServer",
        "Workflow_Start_Paos_Response_Missing",
        "Workflow_Unexpected_Message_From_EidServer",
        "Workflow_Preverification_Developermode_Error",
        "Workflow_Preverification_Error",
        "Workflow_No_Unique_AtCvc",
        "Workflow_No_Unique_DvCvc",
        "Workflow_No_Permission_Error",
        "Workflow_Wrong_Length_Error",
        "Workflow_Certificate_No_Description",
        "Workflow_Certificate_No_Url_In_Description",
        "Workflow_Certificate_Hash_Error",
        "Workflow_Certificate_Sop_Error",
        "Workflow_Browser_Transmission_Error",
        "Workflow_TrustedChannel_Establishment_Error",
        "Workflow_TrustedChannel_Server_Error",
        "Workflow_TrustedChannel_Client_Error",
        "Workflow_TrustedChannel_Hash_Not_In_Description",
        "Workflow_TrustedChannel_No_Data_Received",
        "Workflow_TrustedChannel_Ssl_Connection_Unsupported_Algorithm_Or_Le"
        "ngth",
        "Workflow_TrustedChannel_Ssl_Certificate_Unsupported_Algorithm_Or_L"
        "ength",
        "Workflow_TrustedChannel_ServiceUnavailable",
        "Workflow_TrustedChannel_TimeOut",
        "Workflow_TrustedChannel_Proxy_Error",
        "Workflow_TrustedChannel_Server_Format_Error",
        "Workflow_TrustedChannel_Other_Network_Error",
        "Workflow_Reader_Became_Inaccessible",
        "Workflow_Server_Incomplete_Information_Provided",
        "Workflow_Network_Ssl_Connection_Unsupported_Algorithm_Or_Length",
        "Workflow_Network_Ssl_Certificate_Unsupported_Algorithm_Or_Length",
        "Workflow_Network_Ssl_Hash_Not_In_Certificate_Description",
        "Workflow_Network_Empty_Redirect_Url",
        "Workflow_Network_Expected_Redirect",
        "Workflow_Network_Invalid_Scheme",
        "Workflow_Network_Malformed_Redirect_Url",
        "Workflow_Wrong_Parameter_Invocation",
        "Paos_Unexpected_Warning",
        "Paos_Generic_Server_Error",
        "Paos_Error_AL_Unknown_Error",
        "Paos_Error_AL_Internal_Error",
        "Paos_Error_AL_Communication_Error",
        "Paos_Error_DP_Trusted_Channel_Establishment_Failed",
        "Paos_Error_SAL_Cancellation_by_User",
        "Paos_Error_SAL_Invalid_Key",
        "Card_Not_Found",
        "Card_Communication_Error",
        "Card_Protocol_Error",
        "Card_Unexpected_Transmit_Status",
        "Card_Cancellation_By_User",
        "Card_Input_TimeOut",
        "Card_Pin_Deactivated",
        "Card_Invalid_Pin",
        "Card_Invalid_Can",
        "Card_Invalid_Puk",
        "Card_Pin_Not_Blocked",
        "Card_Puk_Blocked",
        "Card_NewPin_Mismatch",
        "Card_NewPin_Invalid_Length",
        "Card_ValidityVerificationFailed",
        "RemoteReader_CloseCode_AbnormalClose",
        "IfdConnector_InvalidRequest",
        "IfdConnector_NoSupportedApiLevel",
        "IfdConnector_ConnectionError",
        "IfdConnector_RemoteHostRefusedConnection"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GlobalStatusCode'
        QtMocHelpers::EnumData<GlobalStatusCode>(2, 2, QMC::EnumIsScoped).add({
            {    3, GlobalStatusCode::Unknown_Error },
            {    4, GlobalStatusCode::No_Error },
            {    5, GlobalStatusCode::Network_ServiceUnavailable },
            {    6, GlobalStatusCode::Network_ServerError },
            {    7, GlobalStatusCode::Network_ClientError },
            {    8, GlobalStatusCode::Network_Ssl_Establishment_Error },
            {    9, GlobalStatusCode::Network_TimeOut },
            {   10, GlobalStatusCode::Network_Proxy_Error },
            {   11, GlobalStatusCode::Network_Other_Error },
            {   12, GlobalStatusCode::Downloader_File_Not_Found },
            {   13, GlobalStatusCode::Downloader_Cannot_Save_File },
            {   14, GlobalStatusCode::Downloader_Data_Corrupted },
            {   15, GlobalStatusCode::Downloader_Missing_Platform },
            {   16, GlobalStatusCode::Downloader_Aborted },
            {   17, GlobalStatusCode::Update_Execution_Failed },
            {   18, GlobalStatusCode::Workflow_AlreadyInProgress_Error },
            {   19, GlobalStatusCode::Workflow_Communication_Missing_Redirect_Url },
            {   20, GlobalStatusCode::Workflow_InternalError_BeforeTcToken },
            {   21, GlobalStatusCode::Workflow_Cancellation_By_User },
            {   22, GlobalStatusCode::Workflow_Card_Removed },
            {   23, GlobalStatusCode::Workflow_Cannot_Confirm_IdCard_Authenticity },
            {   24, GlobalStatusCode::Workflow_Unknown_Paos_From_EidServer },
            {   25, GlobalStatusCode::Workflow_Start_Paos_Response_Missing },
            {   26, GlobalStatusCode::Workflow_Unexpected_Message_From_EidServer },
            {   27, GlobalStatusCode::Workflow_Preverification_Developermode_Error },
            {   28, GlobalStatusCode::Workflow_Preverification_Error },
            {   29, GlobalStatusCode::Workflow_No_Unique_AtCvc },
            {   30, GlobalStatusCode::Workflow_No_Unique_DvCvc },
            {   31, GlobalStatusCode::Workflow_No_Permission_Error },
            {   32, GlobalStatusCode::Workflow_Wrong_Length_Error },
            {   33, GlobalStatusCode::Workflow_Certificate_No_Description },
            {   34, GlobalStatusCode::Workflow_Certificate_No_Url_In_Description },
            {   35, GlobalStatusCode::Workflow_Certificate_Hash_Error },
            {   36, GlobalStatusCode::Workflow_Certificate_Sop_Error },
            {   37, GlobalStatusCode::Workflow_Browser_Transmission_Error },
            {   38, GlobalStatusCode::Workflow_TrustedChannel_Establishment_Error },
            {   39, GlobalStatusCode::Workflow_TrustedChannel_Server_Error },
            {   40, GlobalStatusCode::Workflow_TrustedChannel_Client_Error },
            {   41, GlobalStatusCode::Workflow_TrustedChannel_Hash_Not_In_Description },
            {   42, GlobalStatusCode::Workflow_TrustedChannel_No_Data_Received },
            {   43, GlobalStatusCode::Workflow_TrustedChannel_Ssl_Connection_Unsupported_Algorithm_Or_Length },
            {   44, GlobalStatusCode::Workflow_TrustedChannel_Ssl_Certificate_Unsupported_Algorithm_Or_Length },
            {   45, GlobalStatusCode::Workflow_TrustedChannel_ServiceUnavailable },
            {   46, GlobalStatusCode::Workflow_TrustedChannel_TimeOut },
            {   47, GlobalStatusCode::Workflow_TrustedChannel_Proxy_Error },
            {   48, GlobalStatusCode::Workflow_TrustedChannel_Server_Format_Error },
            {   49, GlobalStatusCode::Workflow_TrustedChannel_Other_Network_Error },
            {   50, GlobalStatusCode::Workflow_Reader_Became_Inaccessible },
            {   51, GlobalStatusCode::Workflow_Server_Incomplete_Information_Provided },
            {   52, GlobalStatusCode::Workflow_Network_Ssl_Connection_Unsupported_Algorithm_Or_Length },
            {   53, GlobalStatusCode::Workflow_Network_Ssl_Certificate_Unsupported_Algorithm_Or_Length },
            {   54, GlobalStatusCode::Workflow_Network_Ssl_Hash_Not_In_Certificate_Description },
            {   55, GlobalStatusCode::Workflow_Network_Empty_Redirect_Url },
            {   56, GlobalStatusCode::Workflow_Network_Expected_Redirect },
            {   57, GlobalStatusCode::Workflow_Network_Invalid_Scheme },
            {   58, GlobalStatusCode::Workflow_Network_Malformed_Redirect_Url },
            {   59, GlobalStatusCode::Workflow_Wrong_Parameter_Invocation },
            {   60, GlobalStatusCode::Paos_Unexpected_Warning },
            {   61, GlobalStatusCode::Paos_Generic_Server_Error },
            {   62, GlobalStatusCode::Paos_Error_AL_Unknown_Error },
            {   63, GlobalStatusCode::Paos_Error_AL_Internal_Error },
            {   64, GlobalStatusCode::Paos_Error_AL_Communication_Error },
            {   65, GlobalStatusCode::Paos_Error_DP_Trusted_Channel_Establishment_Failed },
            {   66, GlobalStatusCode::Paos_Error_SAL_Cancellation_by_User },
            {   67, GlobalStatusCode::Paos_Error_SAL_Invalid_Key },
            {   68, GlobalStatusCode::Card_Not_Found },
            {   69, GlobalStatusCode::Card_Communication_Error },
            {   70, GlobalStatusCode::Card_Protocol_Error },
            {   71, GlobalStatusCode::Card_Unexpected_Transmit_Status },
            {   72, GlobalStatusCode::Card_Cancellation_By_User },
            {   73, GlobalStatusCode::Card_Input_TimeOut },
            {   74, GlobalStatusCode::Card_Pin_Deactivated },
            {   75, GlobalStatusCode::Card_Invalid_Pin },
            {   76, GlobalStatusCode::Card_Invalid_Can },
            {   77, GlobalStatusCode::Card_Invalid_Puk },
            {   78, GlobalStatusCode::Card_Pin_Not_Blocked },
            {   79, GlobalStatusCode::Card_Puk_Blocked },
            {   80, GlobalStatusCode::Card_NewPin_Mismatch },
            {   81, GlobalStatusCode::Card_NewPin_Invalid_Length },
            {   82, GlobalStatusCode::Card_ValidityVerificationFailed },
            {   83, GlobalStatusCode::RemoteReader_CloseCode_AbnormalClose },
            {   84, GlobalStatusCode::IfdConnector_InvalidRequest },
            {   85, GlobalStatusCode::IfdConnector_NoSupportedApiLevel },
            {   86, GlobalStatusCode::IfdConnector_ConnectionError },
            {   87, GlobalStatusCode::IfdConnector_RemoteHostRefusedConnection },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus20EnumGlobalStatusCodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus20EnumGlobalStatusCodeE =
    governikus::EnumGlobalStatusCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumGlobalStatusCodeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus20EnumGlobalStatusCodeE =
    qt_staticMetaObjectContent_ZN10governikus20EnumGlobalStatusCodeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumGlobalStatusCodeE =
    qt_staticMetaObjectContent_ZN10governikus20EnumGlobalStatusCodeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumGlobalStatusCode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumGlobalStatusCodeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumGlobalStatusCodeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumGlobalStatusCodeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
