/****************************************************************************
** Meta object code from reading C++ file 'SettingsModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/SettingsModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SettingsModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13SettingsModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::SettingsModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13SettingsModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::SettingsModel",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireLanguageChanged",
        "",
        "fireAdvancedSettingsChanged",
        "fireDeveloperOptionsChanged",
        "fireSimulatorChanged",
        "fireDeviceNameChanged",
        "firePinPadModeChanged",
        "fireShowAccessRightsChanged",
        "fireScreenKeyboardChanged",
        "fireCanAllowedChanged",
        "fireStartupModuleChanged",
        "fireAutoStartChanged",
        "fireAutoCloseWindowAfterAuthenticationChanged",
        "fireAutoRedirectAfterAuthenticationChanged",
        "fireAutoUpdateCheckChanged",
        "fireRemindUserToCloseChanged",
        "fireRemindUserOfAutoRedirectChanged",
        "fireTransportPinReminderChanged",
        "fireAppUpdateDataChanged",
        "pAfterManualRequest",
        "fireShowInAppNotificationsChanged",
        "fireUseCustomProxyChanged",
        "fireUseSystemFontChanged",
        "fireUseAnimationsChanged",
        "fireDarkModeChanged",
        "fireTrayIconEnabledChanged",
        "firePreferredTechnologyChanged",
        "fireScreenPrivacyChanged",
        "onAppUpdateDataChanged",
        "onNfcStateChanged",
        "ApplicationModel::NfcState",
        "pNfcState",
        "onTranslationChanged",
        "requestStoreFeedback",
        "hideFutureStoreFeedbackDialogs",
        "updateAppcast",
        "resetHideableDialogs",
        "language",
        "advancedSettings",
        "developerOptions",
        "developerMode",
        "showBetaTesting",
        "appendTransportPin",
        "useSelfauthenticationTestUri",
        "pinPadMode",
        "showAccessRights",
        "deviceName",
        "visualPrivacy",
        "shuffleScreenKeyboard",
        "enableCanAllowed",
        "skipRightsOnCanAllowed",
        "enableSimulator",
        "startupModule",
        "governikus::EnumUiModule::UiModule",
        "showOnboarding",
        "onboardingShown",
        "autoStartAvailable",
        "autoStartApp",
        "autoStartSetByAdmin",
        "trayIconEnabled",
        "autoUpdateAvailable",
        "autoCloseWindowAfterAuthentication",
        "autoRedirectAfterAuthentication",
        "autoUpdateCheck",
        "autoUpdateCheckSetByAdmin",
        "remindUserToClose",
        "remindUserOfAutoRedirect",
        "transportPinReminder",
        "showInAppNotifications",
        "appUpdateData",
        "governikus::AppUpdateDataModel*",
        "customProxyUrl",
        "QUrl",
        "customProxyAttributesPresent",
        "useCustomProxy",
        "useSystemFont",
        "useAnimations",
        "screenPrivacy",
        "userDarkMode",
        "ModeOption",
        "preferredTechnology",
        "governikus::EnumReaderManagerPluginType::ReaderManagerPluginType",
        "ON",
        "OFF",
        "AUTO"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireLanguageChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAdvancedSettingsChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDeveloperOptionsChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSimulatorChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDeviceNameChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePinPadModeChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireShowAccessRightsChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireScreenKeyboardChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCanAllowedChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireStartupModuleChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAutoStartChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAutoCloseWindowAfterAuthenticationChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAutoRedirectAfterAuthenticationChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAutoUpdateCheckChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRemindUserToCloseChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRemindUserOfAutoRedirectChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireTransportPinReminderChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppUpdateDataChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'fireShowInAppNotificationsChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUseCustomProxyChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUseSystemFontChanged'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUseAnimationsChanged'
        QtMocHelpers::SignalData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDarkModeChanged'
        QtMocHelpers::SignalData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireTrayIconEnabledChanged'
        QtMocHelpers::SignalData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePreferredTechnologyChanged'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireScreenPrivacyChanged'
        QtMocHelpers::SignalData<void()>(34, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAppUpdateDataChanged'
        QtMocHelpers::SlotData<void()>(35, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onNfcStateChanged'
        QtMocHelpers::SlotData<void(ApplicationModel::NfcState)>(36, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(39, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'requestStoreFeedback'
        QtMocHelpers::MethodData<bool() const>(40, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'hideFutureStoreFeedbackDialogs'
        QtMocHelpers::MethodData<void() const>(41, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'updateAppcast'
        QtMocHelpers::MethodData<void()>(42, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'resetHideableDialogs'
        QtMocHelpers::MethodData<void() const>(43, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'language'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'advancedSettings'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'developerOptions'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'developerMode'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'showBetaTesting'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'appendTransportPin'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'useSelfauthenticationTestUri'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'pinPadMode'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'showAccessRights'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'deviceName'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'visualPrivacy'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'shuffleScreenKeyboard'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'enableCanAllowed'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'skipRightsOnCanAllowed'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'enableSimulator'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 3),
        // property 'startupModule'
        QtMocHelpers::PropertyData<governikus::EnumUiModule::UiModule>(59, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'showOnboarding'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'onboardingShown'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'autoStartAvailable'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'autoStartApp'
        QtMocHelpers::PropertyData<bool>(64, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 10),
        // property 'autoStartSetByAdmin'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'trayIconEnabled'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'autoUpdateAvailable'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'autoCloseWindowAfterAuthentication'
        QtMocHelpers::PropertyData<bool>(68, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'autoRedirectAfterAuthentication'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'autoUpdateCheck'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'autoUpdateCheckSetByAdmin'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'remindUserToClose'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'remindUserOfAutoRedirect'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'transportPinReminder'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'showInAppNotifications'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'appUpdateData'
        QtMocHelpers::PropertyData<governikus::AppUpdateDataModel*>(76, 0x80000000 | 77, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 17),
        // property 'customProxyUrl'
        QtMocHelpers::PropertyData<QUrl>(78, 0x80000000 | 79, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'customProxyAttributesPresent'
        QtMocHelpers::PropertyData<bool>(80, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'useCustomProxy'
        QtMocHelpers::PropertyData<bool>(81, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'useSystemFont'
        QtMocHelpers::PropertyData<bool>(82, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'useAnimations'
        QtMocHelpers::PropertyData<bool>(83, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'screenPrivacy'
        QtMocHelpers::PropertyData<bool>(84, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'userDarkMode'
        QtMocHelpers::PropertyData<enum ModeOption>(85, 0x80000000 | 86, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag, 22),
        // property 'preferredTechnology'
        QtMocHelpers::PropertyData<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType>(87, 0x80000000 | 88, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 24),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ModeOption'
        QtMocHelpers::EnumData<enum ModeOption>(86, 86, QMC::EnumIsScoped).add({
            {   89, ModeOption::ON },
            {   90, ModeOption::OFF },
            {   91, ModeOption::AUTO },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<SettingsModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10governikus13SettingsModelE[] = {
    QMetaObject::SuperData::link<governikus::EnumUiModule::staticMetaObject>(),
    QMetaObject::SuperData::link<governikus::EnumReaderManagerPluginType::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject governikus::SettingsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13SettingsModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13SettingsModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10governikus13SettingsModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13SettingsModelE_t>.metaTypes,
    nullptr
} };

void governikus::SettingsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SettingsModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireLanguageChanged(); break;
        case 1: _t->fireAdvancedSettingsChanged(); break;
        case 2: _t->fireDeveloperOptionsChanged(); break;
        case 3: _t->fireSimulatorChanged(); break;
        case 4: _t->fireDeviceNameChanged(); break;
        case 5: _t->firePinPadModeChanged(); break;
        case 6: _t->fireShowAccessRightsChanged(); break;
        case 7: _t->fireScreenKeyboardChanged(); break;
        case 8: _t->fireCanAllowedChanged(); break;
        case 9: _t->fireStartupModuleChanged(); break;
        case 10: _t->fireAutoStartChanged(); break;
        case 11: _t->fireAutoCloseWindowAfterAuthenticationChanged(); break;
        case 12: _t->fireAutoRedirectAfterAuthenticationChanged(); break;
        case 13: _t->fireAutoUpdateCheckChanged(); break;
        case 14: _t->fireRemindUserToCloseChanged(); break;
        case 15: _t->fireRemindUserOfAutoRedirectChanged(); break;
        case 16: _t->fireTransportPinReminderChanged(); break;
        case 17: _t->fireAppUpdateDataChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->fireShowInAppNotificationsChanged(); break;
        case 19: _t->fireUseCustomProxyChanged(); break;
        case 20: _t->fireUseSystemFontChanged(); break;
        case 21: _t->fireUseAnimationsChanged(); break;
        case 22: _t->fireDarkModeChanged(); break;
        case 23: _t->fireTrayIconEnabledChanged(); break;
        case 24: _t->firePreferredTechnologyChanged(); break;
        case 25: _t->fireScreenPrivacyChanged(); break;
        case 26: _t->onAppUpdateDataChanged(); break;
        case 27: _t->onNfcStateChanged((*reinterpret_cast<std::add_pointer_t<ApplicationModel::NfcState>>(_a[1]))); break;
        case 28: _t->onTranslationChanged(); break;
        case 29: { bool _r = _t->requestStoreFeedback();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->hideFutureStoreFeedbackDialogs(); break;
        case 31: _t->updateAppcast(); break;
        case 32: _t->resetHideableDialogs(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireLanguageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireAdvancedSettingsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireDeveloperOptionsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireSimulatorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireDeviceNameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::firePinPadModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireShowAccessRightsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireScreenKeyboardChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireCanAllowedChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireStartupModuleChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireAutoStartChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireAutoCloseWindowAfterAuthenticationChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireAutoRedirectAfterAuthenticationChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireAutoUpdateCheckChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireRemindUserToCloseChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireRemindUserOfAutoRedirectChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireTransportPinReminderChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)(bool )>(_a, &SettingsModel::fireAppUpdateDataChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireShowInAppNotificationsChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireUseCustomProxyChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireUseSystemFontChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireUseAnimationsChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireDarkModeChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireTrayIconEnabledChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::firePreferredTechnologyChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (SettingsModel::*)()>(_a, &SettingsModel::fireScreenPrivacyChanged, 25))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 31:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< governikus::AppUpdateDataModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getLanguage(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isAdvancedSettings(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isDeveloperOptions(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isDeveloperMode(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getShowBetaTesting(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getAppendTransportPin(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->useSelfauthenticationTestUri(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->getPinPadMode(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->getShowAccessRights(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getDeviceName(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isVisualPrivacy(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isShuffleScreenKeyboard(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isEnableCanAllowed(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->isSkipRightsOnCanAllowed(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isSimulatorEnabled(); break;
        case 15: *reinterpret_cast<governikus::EnumUiModule::UiModule*>(_v) = _t->getStartupModule(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->getShowOnboarding(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->getOnboardingShown(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isAutoStartAvailable(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isAutoStart(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->autoStartIsSetByAdmin(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->isTrayIconEnabled(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->isAutoUpdateAvailable(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->isAutoCloseWindowAfterAuthentication(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isAutoRedirectAfterAuthentication(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->isAutoUpdateCheck(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->autoUpdateCheckIsSetByAdmin(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->isRemindUserToClose(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->isRemindUserOfAutoRedirect(); break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->isTransportPinReminder(); break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->isShowInAppNotifications(); break;
        case 31: *reinterpret_cast<governikus::AppUpdateDataModel**>(_v) = _t->getAppUpdateData(); break;
        case 32: *reinterpret_cast<QUrl*>(_v) = _t->getCustomProxyUrl(); break;
        case 33: *reinterpret_cast<bool*>(_v) = _t->isCustomProxyAttributesPresent(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->isUseCustomProxy(); break;
        case 35: *reinterpret_cast<bool*>(_v) = _t->isUseSystemFont(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->isUseAnimations(); break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->isScreenPrivacy(); break;
        case 38: *reinterpret_cast<enum ModeOption*>(_v) = _t->getDarkMode(); break;
        case 39: *reinterpret_cast<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType*>(_v) = _t->getPreferredTechnology(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLanguage(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setAdvancedSettings(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setDeveloperOptions(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setDeveloperMode(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setShowBetaTesting(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setAppendTransportPin(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setUseSelfauthenticationTestUri(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setPinPadMode(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setShowAccessRights(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setDeviceName(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setVisualPrivacy(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setShuffleScreenKeyboard(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setEnableCanAllowed(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setSkipRightsOnCanAllowed(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setSimulatorEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setStartupModule(*reinterpret_cast<governikus::EnumUiModule::UiModule*>(_v)); break;
        case 16: _t->setShowOnboarding(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setOnboardingShown(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setAutoStart(*reinterpret_cast<bool*>(_v)); break;
        case 21: _t->setTrayIconEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 23: _t->setAutoCloseWindowAfterAuthentication(*reinterpret_cast<bool*>(_v)); break;
        case 24: _t->setAutoRedirectAfterAuthentication(*reinterpret_cast<bool*>(_v)); break;
        case 25: _t->setAutoUpdateCheck(*reinterpret_cast<bool*>(_v)); break;
        case 27: _t->setRemindUserToClose(*reinterpret_cast<bool*>(_v)); break;
        case 28: _t->setRemindUserOfAutoRedirect(*reinterpret_cast<bool*>(_v)); break;
        case 29: _t->setTransportPinReminder(*reinterpret_cast<bool*>(_v)); break;
        case 30: _t->setShowInAppNotifications(*reinterpret_cast<bool*>(_v)); break;
        case 34: _t->setUseCustomProxy(*reinterpret_cast<bool*>(_v)); break;
        case 35: _t->setUseSystemFont(*reinterpret_cast<bool*>(_v)); break;
        case 36: _t->setUseAnimations(*reinterpret_cast<bool*>(_v)); break;
        case 37: _t->setScreenPrivacy(*reinterpret_cast<bool*>(_v)); break;
        case 38: _t->setDarkMode(*reinterpret_cast<enum ModeOption*>(_v)); break;
        case 39: _t->setPreferredTechnology(*reinterpret_cast<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::SettingsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::SettingsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13SettingsModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<SettingsModel>"))
        return static_cast< SingletonCreator<SettingsModel>*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::SettingsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 33;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void governikus::SettingsModel::fireLanguageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::SettingsModel::fireAdvancedSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::SettingsModel::fireDeveloperOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::SettingsModel::fireSimulatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::SettingsModel::fireDeviceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::SettingsModel::firePinPadModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::SettingsModel::fireShowAccessRightsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::SettingsModel::fireScreenKeyboardChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::SettingsModel::fireCanAllowedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::SettingsModel::fireStartupModuleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::SettingsModel::fireAutoStartChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void governikus::SettingsModel::fireAutoCloseWindowAfterAuthenticationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::SettingsModel::fireAutoRedirectAfterAuthenticationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void governikus::SettingsModel::fireAutoUpdateCheckChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void governikus::SettingsModel::fireRemindUserToCloseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void governikus::SettingsModel::fireRemindUserOfAutoRedirectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void governikus::SettingsModel::fireTransportPinReminderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void governikus::SettingsModel::fireAppUpdateDataChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void governikus::SettingsModel::fireShowInAppNotificationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void governikus::SettingsModel::fireUseCustomProxyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void governikus::SettingsModel::fireUseSystemFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void governikus::SettingsModel::fireUseAnimationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void governikus::SettingsModel::fireDarkModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void governikus::SettingsModel::fireTrayIconEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void governikus::SettingsModel::firePreferredTechnologyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void governikus::SettingsModel::fireScreenPrivacyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}
QT_WARNING_POP
