/****************************************************************************
** Meta object code from reading C++ file 'ChangePinModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/ChangePinModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ChangePinModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus14ChangePinModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::ChangePinModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14ChangePinModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::ChangePinModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireStartWorkflow",
        "",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "startWorkflow",
        "pRequestTransportPin",
        "pActivateUi",
        "pOnlyCheckPin",
        "fireWorkflowStarted",
        "requestTransportPin",
        "onlyCheckPin"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireStartWorkflow'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'startWorkflow'
        QtMocHelpers::MethodData<void(bool, bool, bool)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 }, { QMetaType::Bool, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Method 'startWorkflow'
        QtMocHelpers::MethodData<void(bool, bool)>(9, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Method 'startWorkflow'
        QtMocHelpers::MethodData<void(bool)>(9, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'requestTransportPin'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 0x70000000 | 13),
        // property 'onlyCheckPin'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags, 0x70000000 | 13),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ChangePinModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::ChangePinModel::staticMetaObject = { {
    QMetaObject::SuperData::link<WorkflowModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus14ChangePinModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus14ChangePinModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus14ChangePinModelE_t>.metaTypes,
    nullptr
} };

void governikus::ChangePinModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ChangePinModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireStartWorkflow((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 1: _t->startWorkflow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->startWorkflow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->startWorkflow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ChangePinModel::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &ChangePinModel::fireStartWorkflow, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isRequestTransportPin(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isOnlyCheckPin(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::ChangePinModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::ChangePinModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus14ChangePinModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<ChangePinModel>"))
        return static_cast< SingletonCreator<ChangePinModel>*>(this);
    return WorkflowModel::qt_metacast(_clname);
}

int governikus::ChangePinModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WorkflowModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void governikus::ChangePinModel::fireStartWorkflow(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace CheckNotifySignalValidity_ZN10governikus14ChangePinModelE {
template<typename T> using has_nullary_fireWorkflowStarted = decltype(std::declval<T>().fireWorkflowStarted());
template<typename T> using has_unary_fireWorkflowStarted = decltype(std::declval<T>().fireWorkflowStarted(std::declval<bool>()));
static_assert(qxp::is_detected_v<has_nullary_fireWorkflowStarted, governikus::ChangePinModel> || qxp::is_detected_v<has_unary_fireWorkflowStarted, governikus::ChangePinModel>,
              "NOTIFY signal fireWorkflowStarted does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
