/****************************************************************************
** Meta object code from reading C++ file 'UiPluginWebService.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/webservice/UiPluginWebService.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UiPluginWebService.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::UiPluginWebService::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::UiPluginWebService",
        "doShutdown",
        "",
        "onWorkflowStarted",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "onWorkflowFinished",
        "onWorkflowUnhandled",
        "onNewRequest",
        "QSharedPointer<HttpRequest>"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWorkflowStarted'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onWorkflowFinished'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onWorkflowUnhandled'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onNewRequest'
        QtMocHelpers::SlotData<void(const QSharedPointer<HttpRequest> &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UiPluginWebService, qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::UiPluginWebService::staticMetaObject = { {
    QMetaObject::SuperData::link<UiPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>.metaTypes,
    nullptr
} };

void governikus::UiPluginWebService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UiPluginWebService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->doShutdown(); break;
        case 1: _t->onWorkflowStarted((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 2: _t->onWorkflowFinished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 3: _t->onWorkflowUnhandled((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 4: _t->onNewRequest((*reinterpret_cast<std::add_pointer_t<QSharedPointer<HttpRequest>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<HttpRequest> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *governikus::UiPluginWebService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::UiPluginWebService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18UiPluginWebServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "governikus.UiPlugin"))
        return static_cast< governikus::UiPlugin*>(this);
    return UiPlugin::qt_metacast(_clname);
}

int governikus::UiPluginWebService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UiPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
using namespace governikus;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_UiPluginWebService[] = {
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x72,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'W',  'e',  'b',  'S',  'e',  'r', 
    'v',  'i',  'c',  'e', 
    // "MetaData"
    0x04,  0xa3,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5,  0x67,  'p',  'a',  's',  's', 
    'i',  'v',  'e',  0xf5,  0x6d,  'r',  'e',  'a', 
    'd',  'e',  'r',  'M',  'a',  'n',  'a',  'g', 
    'e',  'r',  0xf4, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(governikus::UiPluginWebService, UiPluginWebService, qt_pluginMetaDataV2_UiPluginWebService)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_UiPluginWebService[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x72,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'W',  'e',  'b',  'S',  'e',  'r', 
    'v',  'i',  'c',  'e', 
    // "MetaData"
    0x04,  0xa3,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5,  0x67,  'p',  'a',  's',  's', 
    'i',  'v',  'e',  0xf5,  0x6d,  'r',  'e',  'a', 
    'd',  'e',  'r',  'M',  'a',  'n',  'a',  'g', 
    'e',  'r',  0xf4, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(governikus::UiPluginWebService, UiPluginWebService)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
