/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<AppUpdateDataModel.h>)
#  include <AppUpdateDataModel.h>
#endif
#if __has_include(<ApplicationModel.h>)
#  include <ApplicationModel.h>
#endif
#if __has_include(<AusweisAppCard_AusweisAppUiQml.h>)
#  include <AusweisAppCard_AusweisAppUiQml.h>
#endif
#if __has_include(<AusweisAppDiagnosis_AusweisAppUiQml.h>)
#  include <AusweisAppDiagnosis_AusweisAppUiQml.h>
#endif
#if __has_include(<AusweisAppGlobal_AusweisAppUiQml.h>)
#  include <AusweisAppGlobal_AusweisAppUiQml.h>
#endif
#if __has_include(<AusweisAppUi_AusweisAppUiQml.h>)
#  include <AusweisAppUi_AusweisAppUiQml.h>
#endif
#if __has_include(<AuthModel.h>)
#  include <AuthModel.h>
#endif
#if __has_include(<CardPosition.h>)
#  include <CardPosition.h>
#endif
#if __has_include(<CardPositionModel.h>)
#  include <CardPositionModel.h>
#endif
#if __has_include(<CertificateDescriptionModel.h>)
#  include <CertificateDescriptionModel.h>
#endif
#if __has_include(<ChangePinModel.h>)
#  include <ChangePinModel.h>
#endif
#if __has_include(<ChatModel.h>)
#  include <ChatModel.h>
#endif
#if __has_include(<CheckIDCardModel.h>)
#  include <CheckIDCardModel.h>
#endif
#if __has_include(<ConnectivityManager.h>)
#  include <ConnectivityManager.h>
#endif
#if __has_include(<FormattedTextModel.h>)
#  include <FormattedTextModel.h>
#endif
#if __has_include(<LogFilesModel.h>)
#  include <LogFilesModel.h>
#endif
#if __has_include(<LogFilterModel.h>)
#  include <LogFilterModel.h>
#endif
#if __has_include(<LogModel.h>)
#  include <LogModel.h>
#endif
#if __has_include(<NotificationModel.h>)
#  include <NotificationModel.h>
#endif
#if __has_include(<NumberModel.h>)
#  include <NumberModel.h>
#endif
#if __has_include(<PinResetInformationModel.h>)
#  include <PinResetInformationModel.h>
#endif
#if __has_include(<ProxyCredentials.h>)
#  include <ProxyCredentials.h>
#endif
#if __has_include(<ReaderModel.h>)
#  include <ReaderModel.h>
#endif
#if __has_include(<ReaderScanEnabler.h>)
#  include <ReaderScanEnabler.h>
#endif
#if __has_include(<ReleaseInformationModel.h>)
#  include <ReleaseInformationModel.h>
#endif
#if __has_include(<RemoteDeviceFilterModel.h>)
#  include <RemoteDeviceFilterModel.h>
#endif
#if __has_include(<RemoteDeviceModel.h>)
#  include <RemoteDeviceModel.h>
#endif
#if __has_include(<RemoteServiceModel.h>)
#  include <RemoteServiceModel.h>
#endif
#if __has_include(<SelfAuthModel.h>)
#  include <SelfAuthModel.h>
#endif
#if __has_include(<SettingsModel.h>)
#  include <SettingsModel.h>
#endif
#if __has_include(<SortedReaderModel.h>)
#  include <SortedReaderModel.h>
#endif
#if __has_include(<SurveyModel.h>)
#  include <SurveyModel.h>
#endif
#if __has_include(<UiPluginModel.h>)
#  include <UiPluginModel.h>
#endif
#if __has_include(<VersionInformationModel.h>)
#  include <VersionInformationModel.h>
#endif
#if __has_include(<WorkflowModel.h>)
#  include <WorkflowModel.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_Governikus_Type()
{
    qmlRegisterModule("Governikus.Type", 2, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<DiagnosisModelForeign>("Governikus.Type", 2);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &governikus::EnumCardReturnCode::staticMetaObject;},
            "governikus::EnumCardReturnCode");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&governikus::EnumCardReturnCode::staticMetaObject, "Governikus.Type", 2, nullptr, &EnumCardReturnCodeForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<governikus::EnumCardReturnCode::CardReturnCode>("governikus::EnumCardReturnCode::CardReturnCode");
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &governikus::EnumGlobalStatusCode::staticMetaObject;},
            "governikus::EnumGlobalStatusCode");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&governikus::EnumGlobalStatusCode::staticMetaObject, "Governikus.Type", 2, nullptr, &EnumGlobalStatusCodeForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<governikus::EnumGlobalStatusCode::GlobalStatusCode>("governikus::EnumGlobalStatusCode::GlobalStatusCode");
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &governikus::EnumReaderManagerPluginType::staticMetaObject;},
            "governikus::EnumReaderManagerPluginType");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&governikus::EnumReaderManagerPluginType::staticMetaObject, "Governikus.Type", 2, nullptr, &EnumReaderManagerPluginTypeForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType>("governikus::EnumReaderManagerPluginType::ReaderManagerPluginType");
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &governikus::EnumUiModule::staticMetaObject;},
            "governikus::EnumUiModule");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&governikus::EnumUiModule::staticMetaObject, "Governikus.Type", 2, nullptr, &EnumUiModuleForeign::staticMetaObject, nullptr);
    qmlRegisterEnum<governikus::EnumUiModule::UiModule>("governikus::EnumUiModule::UiModule");
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    qmlRegisterTypesAndRevisions<SectionModelForeign>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::AppUpdateDataModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ApplicationModel>("Governikus.Type", 2);
    qmlRegisterEnum<governikus::ApplicationModel::Settings>("governikus::ApplicationModel::Settings");
    qmlRegisterEnum<governikus::ApplicationModel::Workflow>("governikus::ApplicationModel::Workflow");
    qmlRegisterEnum<governikus::ApplicationModel::NfcState>("governikus::ApplicationModel::NfcState");
    qmlRegisterTypesAndRevisions<governikus::AuthModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::CardPosition>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::CardPositionModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::CertificateDescriptionModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ChangePinModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ChatModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::CheckIDCardModel>("Governikus.Type", 2);
    qmlRegisterEnum<governikus::CheckIDCardModel::Result>("governikus::CheckIDCardModel::Result");
    qmlRegisterTypesAndRevisions<governikus::ConnectivityManager>("Governikus.Type", 2);
    {
        Q_CONSTINIT static auto metaType = QQmlPrivate::metaTypeForNamespace(
            [](const QtPrivate::QMetaTypeInterface *) {return &governikus::EnumGAnimation::staticMetaObject;},
            "governikus::EnumGAnimation");
        QMetaType(&metaType).id();
    }
    qmlRegisterNamespaceAndRevisions(&governikus::EnumGAnimation::staticMetaObject, "Governikus.Type", 2, nullptr, &governikus::EnumGAnimation::staticMetaObject, nullptr);
    qmlRegisterEnum<governikus::EnumGAnimation::GAnimation>("governikus::EnumGAnimation::GAnimation");
    qmlRegisterTypesAndRevisions<governikus::FormattedTextModel>("Governikus.Type", 2);
    qmlRegisterEnum<governikus::FormattedTextModel::LineType>("governikus::FormattedTextModel::LineType");
    qmlRegisterTypesAndRevisions<governikus::LogFilesModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::LogFilterModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::LogModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::NotificationModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::NumberModel>("Governikus.Type", 2);
    qmlRegisterEnum<governikus::NumberModel::PasswordType>("governikus::NumberModel::PasswordType");
    qmlRegisterTypesAndRevisions<governikus::PinResetInformationModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ProxyCredentials>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ReaderModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ReaderScanEnabler>("Governikus.Type", 2);
    qmlRegisterAnonymousType<QQuickItem, 254>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::ReleaseInformationModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::RemoteDeviceFilterModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::RemoteDeviceModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::RemoteServiceModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::SelfAuthModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::SettingsModel>("Governikus.Type", 2);
    qmlRegisterEnum<governikus::SettingsModel::ModeOption>("governikus::SettingsModel::ModeOption");
    qmlRegisterTypesAndRevisions<governikus::SortedReaderModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::SurveyModel>("Governikus.Type", 2);
    QMetaType::fromType<governikus::UiPlugin *>().id();
    qmlRegisterTypesAndRevisions<governikus::UiPluginModel>("Governikus.Type", 2);
    qmlRegisterTypesAndRevisions<governikus::VersionInformationModel>("Governikus.Type", 2);
    QMetaType::fromType<governikus::WorkflowModel *>().id();
    QT_WARNING_POP
    qmlRegisterModule("Governikus.Type", 2, 5);
}

static const QQmlModuleRegistration governikusTypeRegistration("Governikus.Type", qml_register_types_Governikus_Type);
