/****************************************************************************
** Meta object code from reading C++ file 'AppUpdateDataModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/AppUpdateDataModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AppUpdateDataModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::AppUpdateDataModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::AppUpdateDataModel",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Used by SettingsModel only",
        "QML.Element",
        "auto",
        "fireAppUpdateDataChanged",
        "",
        "fireDownloadProgressChanged",
        "fireAppUpdateAborted",
        "fireAppUpdateFailed",
        "pError",
        "pSupportInfo",
        "fireAppDownloadFinished",
        "fireAppcastUpdateTextChanged",
        "fireAppcastNoUpdateTextChanged",
        "fireAppcastErrorTextChanged",
        "onAppcastFinished",
        "pUpdateAvailable",
        "GlobalStatus",
        "pStatus",
        "onAppcastProgress",
        "pBytesReceived",
        "pBytesTotal",
        "onAppDownloadProgress",
        "onAppDownloadFinished",
        "download",
        "abortDownload",
        "updateAvailable",
        "valid",
        "compatible",
        "appcastRunning",
        "appcastFailed",
        "downloadRunning",
        "appcastProgress",
        "appcastTotal",
        "appcastUpdateText",
        "appcastNoUpdateText",
        "appcastErrorText",
        "downloadProgress",
        "downloadTotal",
        "date",
        "version",
        "url",
        "QUrl",
        "size",
        "checksumUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireAppUpdateDataChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDownloadProgressChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppUpdateAborted'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppUpdateFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'fireAppDownloadFinished'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppcastUpdateTextChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppcastNoUpdateTextChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppcastErrorTextChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAppcastFinished'
        QtMocHelpers::SlotData<void(bool, const GlobalStatus &)>(18, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onAppcastProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(22, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 23 }, { QMetaType::LongLong, 24 },
        }}),
        // Slot 'onAppDownloadProgress'
        QtMocHelpers::SlotData<void(qint64, qint64)>(25, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 23 }, { QMetaType::LongLong, 24 },
        }}),
        // Slot 'onAppDownloadFinished'
        QtMocHelpers::SlotData<void(const GlobalStatus &)>(26, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 12 },
        }}),
        // Method 'download'
        QtMocHelpers::MethodData<bool()>(27, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'abortDownload'
        QtMocHelpers::MethodData<bool()>(28, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'updateAvailable'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'compatible'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'appcastRunning'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'appcastFailed'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'downloadRunning'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'appcastProgress'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'appcastTotal'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'appcastUpdateText'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'appcastNoUpdateText'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'appcastErrorText'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags, 7),
        // property 'downloadProgress'
        QtMocHelpers::PropertyData<int>(40, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'downloadTotal'
        QtMocHelpers::PropertyData<int>(41, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'date'
        QtMocHelpers::PropertyData<QDateTime>(42, QMetaType::QDateTime, QMC::DefaultPropertyFlags, 0),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'size'
        QtMocHelpers::PropertyData<int>(46, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'checksumUrl'
        QtMocHelpers::PropertyData<QUrl>(47, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<AppUpdateDataModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::AppUpdateDataModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t>.metaTypes,
    nullptr
} };

void governikus::AppUpdateDataModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppUpdateDataModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireAppUpdateDataChanged(); break;
        case 1: _t->fireDownloadProgressChanged(); break;
        case 2: _t->fireAppUpdateAborted(); break;
        case 3: _t->fireAppUpdateFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->fireAppDownloadFinished(); break;
        case 5: _t->fireAppcastUpdateTextChanged(); break;
        case 6: _t->fireAppcastNoUpdateTextChanged(); break;
        case 7: _t->fireAppcastErrorTextChanged(); break;
        case 8: _t->onAppcastFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[2]))); break;
        case 9: _t->onAppcastProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 10: _t->onAppDownloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 11: _t->onAppDownloadFinished((*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[1]))); break;
        case 12: { bool _r = _t->download();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->abortDownload();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppUpdateDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireDownloadProgressChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppUpdateAborted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)(const QString & , const QString & )>(_a, &AppUpdateDataModel::fireAppUpdateFailed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppDownloadFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppcastUpdateTextChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppcastNoUpdateTextChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdateDataModel::*)()>(_a, &AppUpdateDataModel::fireAppcastErrorTextChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isUpdateAvailable(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isCompatible(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isAppcastRunning(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->getAppcastFailed(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isDownloadRunning(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->getAppcastProgress(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->getAppcastTotal(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->getAppcastUpdateText(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getAppcastNoUpdateText(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->getAppcastErrorText(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->getDownloadProgress(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->getDownloadTotal(); break;
        case 13: *reinterpret_cast<QDateTime*>(_v) = _t->getDate(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->getVersion(); break;
        case 15: *reinterpret_cast<QUrl*>(_v) = _t->getUrl(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->getSize(); break;
        case 17: *reinterpret_cast<QUrl*>(_v) = _t->getChecksumUrl(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::AppUpdateDataModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::AppUpdateDataModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18AppUpdateDataModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::AppUpdateDataModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void governikus::AppUpdateDataModel::fireAppUpdateDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::AppUpdateDataModel::fireDownloadProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::AppUpdateDataModel::fireAppUpdateAborted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::AppUpdateDataModel::fireAppUpdateFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void governikus::AppUpdateDataModel::fireAppDownloadFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::AppUpdateDataModel::fireAppcastUpdateTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::AppUpdateDataModel::fireAppcastNoUpdateTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::AppUpdateDataModel::fireAppcastErrorTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
