/****************************************************************************
** Meta object code from reading C++ file 'CheckIDCardModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/CheckIDCardModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CheckIDCardModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus16CheckIDCardModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::CheckIDCardModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16CheckIDCardModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::CheckIDCardModel",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "auto",
        "fireResultChanged",
        "",
        "fireScanCompleted",
        "onCardInserted",
        "ReaderInfo",
        "pInfo",
        "onCardRemoved",
        "onReaderAdded",
        "onReaderRemoved",
        "onReaderPropertiesUpdated",
        "onReaderConnectionWaitEnded",
        "stopScanWithResult",
        "Result",
        "result",
        "startScan",
        "ReaderManagerPluginType",
        "pPluginType",
        "startScanIfNecessary",
        "stopScan",
        "UNKNOWN",
        "UNSUPPORTED_PLUGIN_TYPE",
        "NO_READER",
        "CARD_NOT_DETECTED",
        "UNKNOWN_CARD_DETECTED",
        "INSUFFICIENT_APDU_LENGTH",
        "CARD_ACCESS_FAILED",
        "PIN_DEACTIVATED",
        "PIN_SUSPENDED",
        "PIN_BLOCKED",
        "SUCCESS"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireResultChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireScanCompleted'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onCardInserted'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(8, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onCardRemoved'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(11, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onReaderAdded'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(12, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onReaderRemoved'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(13, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onReaderPropertiesUpdated'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(14, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onReaderConnectionWaitEnded'
        QtMocHelpers::SlotData<void()>(15, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'stopScanWithResult'
        QtMocHelpers::SlotData<void(enum Result)>(16, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Method 'startScan'
        QtMocHelpers::MethodData<void(ReaderManagerPluginType)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Method 'startScanIfNecessary'
        QtMocHelpers::MethodData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'stopScan'
        QtMocHelpers::MethodData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'result'
        QtMocHelpers::PropertyData<enum Result>(18, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Result'
        QtMocHelpers::EnumData<enum Result>(17, 17, QMC::EnumIsScoped).add({
            {   24, Result::UNKNOWN },
            {   25, Result::UNSUPPORTED_PLUGIN_TYPE },
            {   26, Result::NO_READER },
            {   27, Result::CARD_NOT_DETECTED },
            {   28, Result::UNKNOWN_CARD_DETECTED },
            {   29, Result::INSUFFICIENT_APDU_LENGTH },
            {   30, Result::CARD_ACCESS_FAILED },
            {   31, Result::PIN_DEACTIVATED },
            {   32, Result::PIN_SUSPENDED },
            {   33, Result::PIN_BLOCKED },
            {   34, Result::SUCCESS },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<CheckIDCardModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::CheckIDCardModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus16CheckIDCardModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus16CheckIDCardModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus16CheckIDCardModelE_t>.metaTypes,
    nullptr
} };

void governikus::CheckIDCardModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CheckIDCardModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireResultChanged(); break;
        case 1: _t->fireScanCompleted(); break;
        case 2: _t->onCardInserted((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 3: _t->onCardRemoved((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 4: _t->onReaderAdded((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 5: _t->onReaderRemoved((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 6: _t->onReaderPropertiesUpdated((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 7: _t->onReaderConnectionWaitEnded(); break;
        case 8: _t->stopScanWithResult((*reinterpret_cast<std::add_pointer_t<enum Result>>(_a[1]))); break;
        case 9: _t->startScan((*reinterpret_cast<std::add_pointer_t<ReaderManagerPluginType>>(_a[1]))); break;
        case 10: _t->startScanIfNecessary(); break;
        case 11: _t->stopScan(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CheckIDCardModel::*)()>(_a, &CheckIDCardModel::fireResultChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CheckIDCardModel::*)()>(_a, &CheckIDCardModel::fireScanCompleted, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Result*>(_v) = _t->getResult(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::CheckIDCardModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::CheckIDCardModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus16CheckIDCardModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::CheckIDCardModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void governikus::CheckIDCardModel::fireResultChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::CheckIDCardModel::fireScanCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
