/****************************************************************************
** Meta object code from reading C++ file 'NumberModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/NumberModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'NumberModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus11NumberModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::NumberModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus11NumberModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::NumberModel",
        "RegisterEnumClassesUnscoped",
        "false",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireCanChanged",
        "",
        "firePinChanged",
        "fireNewPinChanged",
        "fireNewPinConfirmationChanged",
        "firePukChanged",
        "fireInputErrorChanged",
        "fireReaderInfoChanged",
        "fireCanAllowedModeChanged",
        "firePasswordTypeChanged",
        "onCardConnectionChanged",
        "onReaderInfoChanged",
        "ReaderInfo",
        "pInfo",
        "commitNewPin",
        "resetInputError",
        "setInitialInputErrorShown",
        "passwordType",
        "PasswordType",
        "can",
        "pin",
        "newPin",
        "newPinConfirmation",
        "puk",
        "inputError",
        "retryCounter",
        "isCanAllowedMode",
        "initialInputError",
        "TRANSPORT_PIN",
        "PIN",
        "CAN",
        "PUK",
        "NEW_PIN",
        "NEW_PIN_CONFIRMATION",
        "REMOTE_PIN"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireCanChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePinChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireNewPinChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireNewPinConfirmationChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePukChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireInputErrorChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireReaderInfoChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCanAllowedModeChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePasswordTypeChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onCardConnectionChanged'
        QtMocHelpers::SlotData<void()>(17, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onReaderInfoChanged'
        QtMocHelpers::SlotData<void(const ReaderInfo &)>(18, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Method 'commitNewPin'
        QtMocHelpers::MethodData<bool()>(21, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'resetInputError'
        QtMocHelpers::MethodData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'setInitialInputErrorShown'
        QtMocHelpers::MethodData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'passwordType'
        QtMocHelpers::PropertyData<enum PasswordType>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 8),
        // property 'can'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'pin'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'newPin'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'newPinConfirmation'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'puk'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'inputError'
        QtMocHelpers::PropertyData<QString>(31, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'retryCounter'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags, 6),
        // property 'isCanAllowedMode'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'initialInputError'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PasswordType'
        QtMocHelpers::EnumData<enum PasswordType>(25, 25, QMC::EnumIsScoped).add({
            {   35, PasswordType::TRANSPORT_PIN },
            {   36, PasswordType::PIN },
            {   37, PasswordType::CAN },
            {   38, PasswordType::PUK },
            {   39, PasswordType::NEW_PIN },
            {   40, PasswordType::NEW_PIN_CONFIRMATION },
            {   41, PasswordType::REMOTE_PIN },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<NumberModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::NumberModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11NumberModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11NumberModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus11NumberModelE_t>.metaTypes,
    nullptr
} };

void governikus::NumberModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NumberModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireCanChanged(); break;
        case 1: _t->firePinChanged(); break;
        case 2: _t->fireNewPinChanged(); break;
        case 3: _t->fireNewPinConfirmationChanged(); break;
        case 4: _t->firePukChanged(); break;
        case 5: _t->fireInputErrorChanged(); break;
        case 6: _t->fireReaderInfoChanged(); break;
        case 7: _t->fireCanAllowedModeChanged(); break;
        case 8: _t->firePasswordTypeChanged(); break;
        case 9: _t->onCardConnectionChanged(); break;
        case 10: _t->onReaderInfoChanged((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 11: { bool _r = _t->commitNewPin();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->resetInputError(); break;
        case 13: _t->setInitialInputErrorShown(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireCanChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::firePinChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireNewPinChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireNewPinConfirmationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::firePukChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireInputErrorChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireReaderInfoChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::fireCanAllowedModeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberModel::*)()>(_a, &NumberModel::firePasswordTypeChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum PasswordType*>(_v) = _t->getPasswordType(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getCan(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->getPin(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getNewPin(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getNewPinConfirmation(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getPuk(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getInputError(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->getRetryCounter(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isCanAllowedMode(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->getInitialInputError(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCan(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPin(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setNewPin(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setNewPinConfirmation(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setPuk(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::NumberModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::NumberModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11NumberModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<NumberModel>"))
        return static_cast< SingletonCreator<NumberModel>*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::NumberModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void governikus::NumberModel::fireCanChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::NumberModel::firePinChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::NumberModel::fireNewPinChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::NumberModel::fireNewPinConfirmationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::NumberModel::firePukChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::NumberModel::fireInputErrorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::NumberModel::fireReaderInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::NumberModel::fireCanAllowedModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::NumberModel::firePasswordTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
