/**
 * Copyright (c) 2018-2026 Governikus GmbH & Co. KG, Germany
 */

#include "PlatformTools.h"


using namespace governikus;


void PlatformTools::hideFromTaskbar()
{
}


void PlatformTools::restoreToTaskbar()
{
}


void PlatformTools::postNotification(const QString& pTitle, const QString& pMessage)
{
	Q_UNUSED(pTitle)
	Q_UNUSED(pMessage)
}
