/****************************************************************************
** Meta object code from reading C++ file 'Oid.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/asn1/Oid.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Oid.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus12EnumKnownOidE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumKnownOid::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumKnownOidE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumKnownOid",
        "KnownOid",
        "ID_SIGNED_DATA",
        "EC_PS_PUBLICKEY",
        "STANDARDIZED_DOMAINPARAMETERS",
        "BSI_DE_PROTOCOLS_SMARTCARD",
        "ID_PK",
        "ID_PK_DH",
        "ID_PK_ECDH",
        "ID_PS_PK",
        "ID_PS_PK_ECDH_ESCHNORR",
        "ID_TA",
        "ID_TA_RSA",
        "ID_TA_RSA_V1_5_SHA_1",
        "ID_TA_RSA_V1_5_SHA_256",
        "ID_TA_RSA_PSS_SHA_1",
        "ID_TA_RSA_PSS_SHA_256",
        "ID_TA_RSA_V1_5_SHA_512",
        "ID_TA_RSA_PSS_SHA_512",
        "ID_TA_ECDSA",
        "ID_TA_ECDSA_SHA_1",
        "ID_TA_ECDSA_SHA_224",
        "ID_TA_ECDSA_SHA_256",
        "ID_TA_ECDSA_SHA_384",
        "ID_TA_ECDSA_SHA_512",
        "ID_CA",
        "ID_CA_DH",
        "ID_CA_DH_3DES_CBC_CBC",
        "ID_CA_DH_AES_CBC_CMAC_128",
        "ID_CA_DH_AES_CBC_CMAC_192",
        "ID_CA_DH_AES_CBC_CMAC_256",
        "ID_CA_ECDH",
        "ID_CA_ECDH_3DES_CBC_CBC",
        "ID_CA_ECDH_AES_CBC_CMAC_128",
        "ID_CA_ECDH_AES_CBC_CMAC_192",
        "ID_CA_ECDH_AES_CBC_CMAC_256",
        "ID_PACE",
        "ID_PACE_DH_GM",
        "ID_PACE_DH_GM_3DES_CBC_CBC",
        "ID_PACE_DH_GM_AES_CBC_CMAC_128",
        "ID_PACE_DH_GM_AES_CBC_CMAC_192",
        "ID_PACE_DH_GM_AES_CBC_CMAC_256",
        "ID_PACE_ECDH_GM",
        "ID_PACE_ECDH_GM_3DES_CBC_CBC",
        "ID_PACE_ECDH_GM_AES_CBC_CMAC_128",
        "ID_PACE_ECDH_GM_AES_CBC_CMAC_192",
        "ID_PACE_ECDH_GM_AES_CBC_CMAC_256",
        "ID_PACE_DH_IM",
        "ID_PACE_DH_IM_3DES_CBC_CBC",
        "ID_PACE_DH_IM_AES_CBC_CMAC_128",
        "ID_PACE_DH_IM_AES_CBC_CMAC_192",
        "ID_PACE_DH_IM_AES_CBC_CMAC_256",
        "ID_PACE_ECDH_IM",
        "ID_PACE_ECDH_IM_3DES_CBC_CBC",
        "ID_PACE_ECDH_IM_AES_CBC_CMAC_128",
        "ID_PACE_ECDH_IM_AES_CBC_CMAC_192",
        "ID_PACE_ECDH_IM_AES_CBC_CMAC_256",
        "ID_PACE_ECDH_CAM",
        "ID_PACE_ECDH_CAM_AES_CBC_CMAC_128",
        "ID_PACE_ECDH_CAM_AES_CBC_CMAC_192",
        "ID_PACE_ECDH_CAM_AES_CBC_CMAC_256",
        "ID_RI",
        "ID_RI_DH",
        "ID_RI_DH_SHA_1",
        "ID_RI_DH_SHA_224",
        "ID_RI_DH_SHA_256",
        "ID_RI_DH_SHA_384",
        "ID_RI_DH_SHA_512",
        "ID_RI_ECDH",
        "ID_RI_ECDH_SHA_1",
        "ID_RI_ECDH_SHA_224",
        "ID_RI_ECDH_SHA_256",
        "ID_RI_ECDH_SHA_384",
        "ID_RI_ECDH_SHA_512",
        "ID_CI",
        "ID_EID_SECURITY",
        "ID_PT",
        "ID_PS",
        "ID_PSA",
        "ID_PSA_ECDH_ECSCHNORR",
        "ID_PSA_ECDH_ECSCHNORR_SHA_256",
        "ID_PSA_ECDH_ECSCHNORR_SHA_384",
        "ID_PSA_ECDH_ECSCHNORR_SHA_512",
        "ID_PASSWORDTYPE",
        "ID_PASSWORDTYPE_MRZ",
        "ID_PASSWORDTYPE_CAN",
        "ID_PASSWORDTYPE_PIN",
        "ID_PASSWORDTYPE_PUK",
        "ID_AUXILIARY_DATA",
        "ID_DATE_OF_BIRTH",
        "ID_DATE_OF_EXPIRY",
        "ID_MUNICIPALITY_ID",
        "ID_PSM_MESSAGE",
        "ID_SECURITY_OBJECT",
        "ID_ROLES",
        "ID_IS",
        "ID_AT",
        "ID_ST",
        "ID_EXTENSIONS",
        "ID_DESCRIPTION",
        "ID_PLAIN_FORMAT",
        "ID_HTML_FORMAT",
        "ID_PFD_FORMAT",
        "ID_SECTOR_RI",
        "ID_SECTOR_PS",
        "ID_EID_TYPE",
        "ID_CARD_EID_TYPE",
        "ID_MOBILE_EID_TYPE",
        "ID_MOBILE_EID_TYPE_SE_CERTIFIED",
        "ID_MOBILE_EID_TYPE_SE_ENDORSED",
        "ID_MOBILE_EID_TYPE_HW_KEYSTORE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'KnownOid'
        QtMocHelpers::EnumData<KnownOid>(1, 1, QMC::EnumIsScoped).add({
            {    2, KnownOid::ID_SIGNED_DATA },
            {    3, KnownOid::EC_PS_PUBLICKEY },
            {    4, KnownOid::STANDARDIZED_DOMAINPARAMETERS },
            {    5, KnownOid::BSI_DE_PROTOCOLS_SMARTCARD },
            {    6, KnownOid::ID_PK },
            {    7, KnownOid::ID_PK_DH },
            {    8, KnownOid::ID_PK_ECDH },
            {    9, KnownOid::ID_PS_PK },
            {   10, KnownOid::ID_PS_PK_ECDH_ESCHNORR },
            {   11, KnownOid::ID_TA },
            {   12, KnownOid::ID_TA_RSA },
            {   13, KnownOid::ID_TA_RSA_V1_5_SHA_1 },
            {   14, KnownOid::ID_TA_RSA_V1_5_SHA_256 },
            {   15, KnownOid::ID_TA_RSA_PSS_SHA_1 },
            {   16, KnownOid::ID_TA_RSA_PSS_SHA_256 },
            {   17, KnownOid::ID_TA_RSA_V1_5_SHA_512 },
            {   18, KnownOid::ID_TA_RSA_PSS_SHA_512 },
            {   19, KnownOid::ID_TA_ECDSA },
            {   20, KnownOid::ID_TA_ECDSA_SHA_1 },
            {   21, KnownOid::ID_TA_ECDSA_SHA_224 },
            {   22, KnownOid::ID_TA_ECDSA_SHA_256 },
            {   23, KnownOid::ID_TA_ECDSA_SHA_384 },
            {   24, KnownOid::ID_TA_ECDSA_SHA_512 },
            {   25, KnownOid::ID_CA },
            {   26, KnownOid::ID_CA_DH },
            {   27, KnownOid::ID_CA_DH_3DES_CBC_CBC },
            {   28, KnownOid::ID_CA_DH_AES_CBC_CMAC_128 },
            {   29, KnownOid::ID_CA_DH_AES_CBC_CMAC_192 },
            {   30, KnownOid::ID_CA_DH_AES_CBC_CMAC_256 },
            {   31, KnownOid::ID_CA_ECDH },
            {   32, KnownOid::ID_CA_ECDH_3DES_CBC_CBC },
            {   33, KnownOid::ID_CA_ECDH_AES_CBC_CMAC_128 },
            {   34, KnownOid::ID_CA_ECDH_AES_CBC_CMAC_192 },
            {   35, KnownOid::ID_CA_ECDH_AES_CBC_CMAC_256 },
            {   36, KnownOid::ID_PACE },
            {   37, KnownOid::ID_PACE_DH_GM },
            {   38, KnownOid::ID_PACE_DH_GM_3DES_CBC_CBC },
            {   39, KnownOid::ID_PACE_DH_GM_AES_CBC_CMAC_128 },
            {   40, KnownOid::ID_PACE_DH_GM_AES_CBC_CMAC_192 },
            {   41, KnownOid::ID_PACE_DH_GM_AES_CBC_CMAC_256 },
            {   42, KnownOid::ID_PACE_ECDH_GM },
            {   43, KnownOid::ID_PACE_ECDH_GM_3DES_CBC_CBC },
            {   44, KnownOid::ID_PACE_ECDH_GM_AES_CBC_CMAC_128 },
            {   45, KnownOid::ID_PACE_ECDH_GM_AES_CBC_CMAC_192 },
            {   46, KnownOid::ID_PACE_ECDH_GM_AES_CBC_CMAC_256 },
            {   47, KnownOid::ID_PACE_DH_IM },
            {   48, KnownOid::ID_PACE_DH_IM_3DES_CBC_CBC },
            {   49, KnownOid::ID_PACE_DH_IM_AES_CBC_CMAC_128 },
            {   50, KnownOid::ID_PACE_DH_IM_AES_CBC_CMAC_192 },
            {   51, KnownOid::ID_PACE_DH_IM_AES_CBC_CMAC_256 },
            {   52, KnownOid::ID_PACE_ECDH_IM },
            {   53, KnownOid::ID_PACE_ECDH_IM_3DES_CBC_CBC },
            {   54, KnownOid::ID_PACE_ECDH_IM_AES_CBC_CMAC_128 },
            {   55, KnownOid::ID_PACE_ECDH_IM_AES_CBC_CMAC_192 },
            {   56, KnownOid::ID_PACE_ECDH_IM_AES_CBC_CMAC_256 },
            {   57, KnownOid::ID_PACE_ECDH_CAM },
            {   58, KnownOid::ID_PACE_ECDH_CAM_AES_CBC_CMAC_128 },
            {   59, KnownOid::ID_PACE_ECDH_CAM_AES_CBC_CMAC_192 },
            {   60, KnownOid::ID_PACE_ECDH_CAM_AES_CBC_CMAC_256 },
            {   61, KnownOid::ID_RI },
            {   62, KnownOid::ID_RI_DH },
            {   63, KnownOid::ID_RI_DH_SHA_1 },
            {   64, KnownOid::ID_RI_DH_SHA_224 },
            {   65, KnownOid::ID_RI_DH_SHA_256 },
            {   66, KnownOid::ID_RI_DH_SHA_384 },
            {   67, KnownOid::ID_RI_DH_SHA_512 },
            {   68, KnownOid::ID_RI_ECDH },
            {   69, KnownOid::ID_RI_ECDH_SHA_1 },
            {   70, KnownOid::ID_RI_ECDH_SHA_224 },
            {   71, KnownOid::ID_RI_ECDH_SHA_256 },
            {   72, KnownOid::ID_RI_ECDH_SHA_384 },
            {   73, KnownOid::ID_RI_ECDH_SHA_512 },
            {   74, KnownOid::ID_CI },
            {   75, KnownOid::ID_EID_SECURITY },
            {   76, KnownOid::ID_PT },
            {   77, KnownOid::ID_PS },
            {   78, KnownOid::ID_PSA },
            {   79, KnownOid::ID_PSA_ECDH_ECSCHNORR },
            {   80, KnownOid::ID_PSA_ECDH_ECSCHNORR_SHA_256 },
            {   81, KnownOid::ID_PSA_ECDH_ECSCHNORR_SHA_384 },
            {   82, KnownOid::ID_PSA_ECDH_ECSCHNORR_SHA_512 },
            {   83, KnownOid::ID_PASSWORDTYPE },
            {   84, KnownOid::ID_PASSWORDTYPE_MRZ },
            {   85, KnownOid::ID_PASSWORDTYPE_CAN },
            {   86, KnownOid::ID_PASSWORDTYPE_PIN },
            {   87, KnownOid::ID_PASSWORDTYPE_PUK },
            {   88, KnownOid::ID_AUXILIARY_DATA },
            {   89, KnownOid::ID_DATE_OF_BIRTH },
            {   90, KnownOid::ID_DATE_OF_EXPIRY },
            {   91, KnownOid::ID_MUNICIPALITY_ID },
            {   92, KnownOid::ID_PSM_MESSAGE },
            {   93, KnownOid::ID_SECURITY_OBJECT },
            {   94, KnownOid::ID_ROLES },
            {   95, KnownOid::ID_IS },
            {   96, KnownOid::ID_AT },
            {   97, KnownOid::ID_ST },
            {   98, KnownOid::ID_EXTENSIONS },
            {   99, KnownOid::ID_DESCRIPTION },
            {  100, KnownOid::ID_PLAIN_FORMAT },
            {  101, KnownOid::ID_HTML_FORMAT },
            {  102, KnownOid::ID_PFD_FORMAT },
            {  103, KnownOid::ID_SECTOR_RI },
            {  104, KnownOid::ID_SECTOR_PS },
            {  105, KnownOid::ID_EID_TYPE },
            {  106, KnownOid::ID_CARD_EID_TYPE },
            {  107, KnownOid::ID_MOBILE_EID_TYPE },
            {  108, KnownOid::ID_MOBILE_EID_TYPE_SE_CERTIFIED },
            {  109, KnownOid::ID_MOBILE_EID_TYPE_SE_ENDORSED },
            {  110, KnownOid::ID_MOBILE_EID_TYPE_HW_KEYSTORE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus12EnumKnownOidE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus12EnumKnownOidE =
    governikus::EnumKnownOid::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumKnownOidE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus12EnumKnownOidE =
    qt_staticMetaObjectContent_ZN10governikus12EnumKnownOidE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumKnownOidE =
    qt_staticMetaObjectContent_ZN10governikus12EnumKnownOidE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumKnownOid::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumKnownOidE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumKnownOidE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumKnownOidE.metaTypes,
    nullptr
} };

QT_WARNING_POP
