#ifndef AusweisAppCard_AusweisAppUiQml_H
#define AusweisAppCard_AusweisAppUiQml_H
#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <ReaderManagerPluginInfo.h>
namespace EnumReaderManagerPluginTypeForeign{
    Q_NAMESPACE
    QML_FOREIGN_NAMESPACE(governikus::EnumReaderManagerPluginType)
    QML_NAMED_ELEMENT(ReaderManagerPluginType)
}

#endif // AusweisAppCard_AusweisAppUiQml_H
