/****************************************************************************
** Meta object code from reading C++ file 'IfdClient.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/base/IfdClient.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IfdClient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus9IfdClientE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::IfdClient::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus9IfdClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::IfdClient",
        "fireDeviceAppeared",
        "",
        "QSharedPointer<IfdListEntry>",
        "pEntry",
        "fireDeviceUpdated",
        "fireDeviceVanished",
        "fireEstablishConnectionDone",
        "GlobalStatus",
        "pStatus",
        "fireNewDispatcher",
        "QSharedPointer<IfdDispatcherClient>",
        "pDispatcher",
        "fireDispatcherChanged",
        "fireRemoteDevicesInfo",
        "QList<QSharedPointer<IfdListEntry>>",
        "pRemoteDevices",
        "fireDispatcherDestroyed",
        "GlobalStatus::Code",
        "pCloseCode",
        "pId",
        "fireCertificateRemoved",
        "pDeviceName",
        "startDetection",
        "stopDetection",
        "isDetecting",
        "establishConnection",
        "pPsk",
        "requestRemoteDevices"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireDeviceAppeared'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdListEntry> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireDeviceUpdated'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdListEntry> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireDeviceVanished'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdListEntry> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireEstablishConnectionDone'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdListEntry> &, const GlobalStatus &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'fireNewDispatcher'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdDispatcherClient> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'fireDispatcherChanged'
        QtMocHelpers::SignalData<void(const QSharedPointer<IfdDispatcherClient> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'fireRemoteDevicesInfo'
        QtMocHelpers::SignalData<void(const QList<QSharedPointer<IfdListEntry>> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Signal 'fireDispatcherDestroyed'
        QtMocHelpers::SignalData<void(GlobalStatus::Code, const QByteArray &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::QByteArray, 20 },
        }}),
        // Signal 'fireCertificateRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'startDetection'
        QtMocHelpers::MethodData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'stopDetection'
        QtMocHelpers::MethodData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isDetecting'
        QtMocHelpers::MethodData<bool()>(25, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'establishConnection'
        QtMocHelpers::MethodData<void(const QSharedPointer<IfdListEntry> &, const QByteArray &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QByteArray, 27 },
        }}),
        // Method 'requestRemoteDevices'
        QtMocHelpers::MethodData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IfdClient, qt_meta_tag_ZN10governikus9IfdClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::IfdClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9IfdClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9IfdClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus9IfdClientE_t>.metaTypes,
    nullptr
} };

void governikus::IfdClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IfdClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireDeviceAppeared((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1]))); break;
        case 1: _t->fireDeviceUpdated((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1]))); break;
        case 2: _t->fireDeviceVanished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1]))); break;
        case 3: _t->fireEstablishConnectionDone((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[2]))); break;
        case 4: _t->fireNewDispatcher((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdDispatcherClient>>>(_a[1]))); break;
        case 5: _t->fireDispatcherChanged((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdDispatcherClient>>>(_a[1]))); break;
        case 6: _t->fireRemoteDevicesInfo((*reinterpret_cast<std::add_pointer_t<QList<QSharedPointer<IfdListEntry>>>>(_a[1]))); break;
        case 7: _t->fireDispatcherDestroyed((*reinterpret_cast<std::add_pointer_t<GlobalStatus::Code>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 8: _t->fireCertificateRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->startDetection(); break;
        case 10: _t->stopDetection(); break;
        case 11: { bool _r = _t->isDetecting();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->establishConnection((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 13: _t->requestRemoteDevices(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<IfdDispatcherClient> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<IfdDispatcherClient> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdListEntry> & )>(_a, &IfdClient::fireDeviceAppeared, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdListEntry> & )>(_a, &IfdClient::fireDeviceUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdListEntry> & )>(_a, &IfdClient::fireDeviceVanished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdListEntry> & , const GlobalStatus & )>(_a, &IfdClient::fireEstablishConnectionDone, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdDispatcherClient> & )>(_a, &IfdClient::fireNewDispatcher, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QSharedPointer<IfdDispatcherClient> & )>(_a, &IfdClient::fireDispatcherChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QList<QSharedPointer<IfdListEntry>> & )>(_a, &IfdClient::fireRemoteDevicesInfo, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(GlobalStatus::Code , const QByteArray & )>(_a, &IfdClient::fireDispatcherDestroyed, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdClient::*)(const QString & )>(_a, &IfdClient::fireCertificateRemoved, 8))
            return;
    }
}

const QMetaObject *governikus::IfdClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::IfdClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus9IfdClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::IfdClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void governikus::IfdClient::fireDeviceAppeared(const QSharedPointer<IfdListEntry> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::IfdClient::fireDeviceUpdated(const QSharedPointer<IfdListEntry> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::IfdClient::fireDeviceVanished(const QSharedPointer<IfdListEntry> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void governikus::IfdClient::fireEstablishConnectionDone(const QSharedPointer<IfdListEntry> & _t1, const GlobalStatus & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void governikus::IfdClient::fireNewDispatcher(const QSharedPointer<IfdDispatcherClient> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void governikus::IfdClient::fireDispatcherChanged(const QSharedPointer<IfdDispatcherClient> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void governikus::IfdClient::fireRemoteDevicesInfo(const QList<QSharedPointer<IfdListEntry>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void governikus::IfdClient::fireDispatcherDestroyed(GlobalStatus::Code _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}

// SIGNAL 8
void governikus::IfdClient::fireCertificateRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
