/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x1,0xa0,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x7,0x5,0xb5,0xc,0x0,0x26,0xd0,0x31,0x1c,0x40,0x73,0x1b,
  0x37,0x52,0x92,0x4c,0x99,0xb4,0x4b,0xec,0x2a,0x4a,0x13,0x43,0x9d,0xd3,0xe8,0x39,
  0x3,0xef,0x33,0x3c,0x67,0x51,0xac,0xea,0x72,0x29,0x0,0x2a,0x0,0x2a,0x0,0xd1,
  0xdb,0xa6,0x79,0xac,0x9,0xc5,0xb2,0x86,0x46,0x5a,0x1b,0x38,0xcd,0xc9,0xbd,0xc1,
  0xfa,0x8c,0x1c,0x5f,0x62,0x5d,0xe5,0xb8,0xf5,0x91,0x29,0xf,0x1c,0xf7,0x24,0x65,
  0xcf,0x4c,0x1f,0x56,0xee,0xb3,0x34,0xb,0x91,0xbf,0x46,0x30,0x12,0xd4,0x2,0x38,
  0x1,0xb,0x76,0x42,0x3d,0xcf,0xf3,0x3c,0xe6,0x11,0x2c,0x4f,0x28,0xbd,0x29,0xb7,
  0xc,0xe3,0x34,0xc4,0x9a,0x6d,0xf3,0x8c,0x9,0x43,0x76,0xe,0xb7,0xb2,0x16,0x5b,
  0x2b,0xd6,0x9d,0xb8,0xbc,0x4a,0xf2,0xf5,0x51,0x49,0xe6,0x26,0xe0,0xa3,0x2,0x79,
  0x4f,0x9f,0x8b,0x88,0xf1,0x62,0x71,0x24,0xfc,0x3e,0x97,0xdf,0x2c,0x1d,0xd2,0x9b,
  0x9a,0x25,0x8e,0x28,0x63,0xca,0xc5,0xe6,0x75,0xf1,0x35,0xa4,0xaa,0x27,0xe7,0x90,
  0x51,0x58,0x8b,0xec,0xf9,0xac,0x53,0x2f,0x24,0x79,0xa7,0x83,0xe2,0x22,0xde,0x86,
  0x57,0x94,0xbc,0x77,0x73,0x8c,0xe,0xd9,0xa6,0xb0,0xcd,0x7b,0xfc,0x88,0x95,0x59,
  0x5b,0x47,0x1,0x65,0xa8,0x61,0x12,0x92,0x22,0x1a,0xa5,0x30,0xec,0x1d,0x40,0x44,
  0x20,0x12,0x1b,0x75,0x52,0x58,0x24,0x4b,0x10,0x11,0x9,0x46,0xc0,0x61,0x6,0xc9,
  0xa8,0x24,0xb5,0x3f,0x6b,0x50,0x4c,0xbf,0xb0,0x50,0xde,0xfe,0x63,0x55,0xdb,0x4f,
  0x4a,0xf1,0xe4,0x4c,0xcf,0x37,0x4e,0x7e,0x91,0x1a,0xb2,0x7e,0xc9,0xcd,0xed,0x44,
  0x1f,0xfc,0xde,0xe8,0x2,0x17,0xed,0x2d,0x3e,0x74,0xa2,0x2d,0x60,0xff,0xba,0x77,
  0x5b,0x61,0x6c,0x7a,0x4d,0xdc,0x8c,0x9c,0x20,0x2d,0xbf,0xb0,0x9,0x33,0xa6,0xc5,
  0x2,0xfd,0xa1,0xf3,0x7b,0xdb,0x3f,0xf3,0xc0,0x2b,0x42,0xd2,0x1e,0x70,0xfd,0xed,
  0x69,0xf6,0x88,0xe8,0x46,0xdd,0xe7,0xe2,0x63,0x1c,0xd9,0xbd,0x82,0xb,0x3b,0x7d,
  0xda,0xc2,0x93,0x8d,0x4e,0x1b,0x63,0x3f,0x17,0x82,0x5e,0xf0,0xcd,0xbe,0xfc,0xd8,
  0xb0,0x54,0xa8,0x2c,0xed,0xba,0x44,0xb8,0xda,0x99,0xb0,0xf9,0x63,0xd7,0x88,0x8a,
  0xe,0xb,0xd6,0xab,0x6a,0xe9,0x25,0x30,0x3a,0x81,0x1a,0xbc,0x50,0xe,0xf9,0xe9,
  0xcd,0x9f,0x31,0x40,0x6d,0x12,0x45,0x94,0x4a,0xbd,0x58,0x74,0xeb,0x24,0xea,0x1e,
  0x6b,0xa,0x81,0x7c,0x8d,0xd,0xcd,0x3,0xa6,0x59,0x21,0xd4,0xdd,0x5c,0x5,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // Governikus
  0x0,0xa,
  0xc,0x9d,0x4f,0x43,
  0x0,0x47,
  0x0,0x6f,0x0,0x76,0x0,0x65,0x0,0x72,0x0,0x6e,0x0,0x69,0x0,0x6b,0x0,0x75,0x0,0x73,
    // Animations
  0x0,0xa,
  0x3,0x84,0x10,0x53,
  0x0,0x41,
  0x0,0x6e,0x0,0x69,0x0,0x6d,0x0,0x61,0x0,0x74,0x0,0x69,0x0,0x6f,0x0,0x6e,0x0,0x73,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,
  // :/qt/qml/Governikus
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,
  // :/qt/qml/Governikus/Animations
  0x0,0x0,0x0,0x30,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x5,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,
  // :/qt/qml/Governikus/Animations/qmldir
  0x0,0x0,0x0,0x4a,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0xcd,0xf,0xdf,0x30,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Governikus_Animations)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Governikus_Animations)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Governikus_Animations)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Governikus_Animations)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Governikus_Animations)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Governikus_Animations)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
