/****************************************************************************
** Meta object code from reading C++ file 'WorkflowModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/WorkflowModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'WorkflowModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13WorkflowModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::WorkflowModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13WorkflowModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::WorkflowModel",
        "fireWorkflowStarted",
        "",
        "fireCurrentStateChanged",
        "pState",
        "fireStateEntered",
        "fireResultChanged",
        "fireReaderPluginTypeChanged",
        "pExplicitStart",
        "fireSelectedReaderChanged",
        "fireReaderImageChanged",
        "fireNextWorkflowPendingChanged",
        "fireSupportedPluginTypesChanged",
        "fireRemoveCardFeedbackChanged",
        "fireHasCardChanged",
        "fireEidTypeMismatchErrorChanged",
        "fireShowUiRequest",
        "UiModule",
        "pModule",
        "fireWorkflowFinished",
        "pSuccess",
        "fireOnPinUnlocked",
        "fireOnPasswordUsed",
        "fireOnCanSuccess",
        "fireLastReturnCodeChanged",
        "onPaceResultUpdated",
        "insertSimulator",
        "cancelWorkflow",
        "startScanExplicitly",
        "continueWorkflow",
        "setInitialPluginType",
        "isCancellationByUser",
        "getEmailHeader",
        "getEmailBody",
        "pPercentEncoding",
        "pAddLogNotice",
        "sendResultMail",
        "currentState",
        "resultString",
        "error",
        "errorIsMasked",
        "pukInoperative",
        "readerPluginType",
        "governikus::EnumReaderManagerPluginType::ReaderManagerPluginType",
        "supportedPluginTypes",
        "QList<ReaderManagerPluginType>",
        "isBasicReader",
        "isRemoteReader",
        "eidTypeMismatchError",
        "hasNextWorkflowPending",
        "statusCode",
        "governikus::EnumGlobalStatusCode::GlobalStatusCode",
        "statusCodeDisplayString",
        "statusHintText",
        "statusCodeAnimation",
        "governikus::EnumGAnimation::GAnimation",
        "showRemoveCardFeedback",
        "cardInitiallyAppeared",
        "hasCard",
        "lastReturnCode",
        "governikus::EnumCardReturnCode::CardReturnCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireWorkflowStarted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCurrentStateChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'fireStateEntered'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'fireResultChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireReaderPluginTypeChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'fireReaderPluginTypeChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'fireSelectedReaderChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireReaderImageChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireNextWorkflowPendingChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSupportedPluginTypesChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRemoveCardFeedbackChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireHasCardChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireEidTypeMismatchErrorChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireShowUiRequest'
        QtMocHelpers::SignalData<void(UiModule)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'fireWorkflowFinished'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'fireOnPinUnlocked'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireOnPasswordUsed'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireOnCanSuccess'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireLastReturnCodeChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onPaceResultUpdated'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Method 'insertSimulator'
        QtMocHelpers::MethodData<void() const>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'cancelWorkflow'
        QtMocHelpers::MethodData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'startScanExplicitly'
        QtMocHelpers::MethodData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'continueWorkflow'
        QtMocHelpers::MethodData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setInitialPluginType'
        QtMocHelpers::MethodData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'isCancellationByUser'
        QtMocHelpers::MethodData<bool() const>(31, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'getEmailHeader'
        QtMocHelpers::MethodData<QString() const>(32, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getEmailBody'
        QtMocHelpers::MethodData<QString(bool, bool) const>(33, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Bool, 34 }, { QMetaType::Bool, 35 },
        }}),
        // Method 'getEmailBody'
        QtMocHelpers::MethodData<QString(bool) const>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { QMetaType::Bool, 34 },
        }}),
        // Method 'getEmailBody'
        QtMocHelpers::MethodData<QString() const>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString),
        // Method 'sendResultMail'
        QtMocHelpers::MethodData<void() const>(36, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentState'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags, 1),
        // property 'resultString'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'error'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'errorIsMasked'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'pukInoperative'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'readerPluginType'
        QtMocHelpers::PropertyData<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType>(42, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'supportedPluginTypes'
        QtMocHelpers::PropertyData<QList<ReaderManagerPluginType>>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'isBasicReader'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'isRemoteReader'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'eidTypeMismatchError'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags, 12),
        // property 'hasNextWorkflowPending'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags, 8),
        // property 'statusCode'
        QtMocHelpers::PropertyData<governikus::EnumGlobalStatusCode::GlobalStatusCode>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'statusCodeDisplayString'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'statusHintText'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'statusCodeAnimation'
        QtMocHelpers::PropertyData<governikus::EnumGAnimation::GAnimation>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'showRemoveCardFeedback'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'cardInitiallyAppeared'
        QtMocHelpers::PropertyData<bool>(57, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'hasCard'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags, 11),
        // property 'lastReturnCode'
        QtMocHelpers::PropertyData<governikus::EnumCardReturnCode::CardReturnCode>(59, 0x80000000 | 60, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 18),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkflowModel, qt_meta_tag_ZN10governikus13WorkflowModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN10governikus13WorkflowModelE[] = {
    QMetaObject::SuperData::link<governikus::EnumReaderManagerPluginType::staticMetaObject>(),
    QMetaObject::SuperData::link<governikus::EnumGlobalStatusCode::staticMetaObject>(),
    QMetaObject::SuperData::link<governikus::EnumGAnimation::staticMetaObject>(),
    QMetaObject::SuperData::link<governikus::EnumCardReturnCode::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject governikus::WorkflowModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13WorkflowModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13WorkflowModelE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN10governikus13WorkflowModelE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13WorkflowModelE_t>.metaTypes,
    nullptr
} };

void governikus::WorkflowModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkflowModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireWorkflowStarted(); break;
        case 1: _t->fireCurrentStateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->fireStateEntered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fireResultChanged(); break;
        case 4: _t->fireReaderPluginTypeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->fireReaderPluginTypeChanged(); break;
        case 6: _t->fireSelectedReaderChanged(); break;
        case 7: _t->fireReaderImageChanged(); break;
        case 8: _t->fireNextWorkflowPendingChanged(); break;
        case 9: _t->fireSupportedPluginTypesChanged(); break;
        case 10: _t->fireRemoveCardFeedbackChanged(); break;
        case 11: _t->fireHasCardChanged(); break;
        case 12: _t->fireEidTypeMismatchErrorChanged(); break;
        case 13: _t->fireShowUiRequest((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 14: _t->fireWorkflowFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->fireOnPinUnlocked(); break;
        case 16: _t->fireOnPasswordUsed(); break;
        case 17: _t->fireOnCanSuccess(); break;
        case 18: _t->fireLastReturnCodeChanged(); break;
        case 19: _t->onPaceResultUpdated(); break;
        case 20: _t->insertSimulator(); break;
        case 21: _t->cancelWorkflow(); break;
        case 22: _t->startScanExplicitly(); break;
        case 23: _t->continueWorkflow(); break;
        case 24: _t->setInitialPluginType(); break;
        case 25: { bool _r = _t->isCancellationByUser();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { QString _r = _t->getEmailHeader();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->getEmailBody((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->getEmailBody((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { QString _r = _t->getEmailBody();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->sendResultMail(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireWorkflowStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)(const QString & )>(_a, &WorkflowModel::fireCurrentStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)(const QString & )>(_a, &WorkflowModel::fireStateEntered, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireResultChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)(bool )>(_a, &WorkflowModel::fireReaderPluginTypeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireSelectedReaderChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireReaderImageChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireNextWorkflowPendingChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireSupportedPluginTypesChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireRemoveCardFeedbackChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireHasCardChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireEidTypeMismatchErrorChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)(UiModule )>(_a, &WorkflowModel::fireShowUiRequest, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)(bool )>(_a, &WorkflowModel::fireWorkflowFinished, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireOnPinUnlocked, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireOnPasswordUsed, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireOnCanSuccess, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkflowModel::*)()>(_a, &WorkflowModel::fireLastReturnCodeChanged, 18))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getCurrentState(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getResultString(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isError(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isMaskedError(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isPukInoperative(); break;
        case 5: *reinterpret_cast<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType*>(_v) = _t->getReaderPluginType(); break;
        case 6: *reinterpret_cast<QList<ReaderManagerPluginType>*>(_v) = _t->getSupportedReaderPluginTypes(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isBasicReader(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isRemoteReader(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->eidTypeMismatchError(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->getNextWorkflowPending(); break;
        case 11: *reinterpret_cast<governikus::EnumGlobalStatusCode::GlobalStatusCode*>(_v) = _t->getStatusCode(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->getStatusCodeDisplayString(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->getStatusHintText(); break;
        case 14: *reinterpret_cast<governikus::EnumGAnimation::GAnimation*>(_v) = _t->getStatusCodeAnimation(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->showRemoveCardFeedback(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->getCardInitiallyAppeared(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->hasCard(); break;
        case 18: *reinterpret_cast<governikus::EnumCardReturnCode::CardReturnCode*>(_v) = _t->getLastReturnCode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 5: _t->setReaderPluginType(*reinterpret_cast<governikus::EnumReaderManagerPluginType::ReaderManagerPluginType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::WorkflowModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::WorkflowModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13WorkflowModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::WorkflowModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 31;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void governikus::WorkflowModel::fireWorkflowStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::WorkflowModel::fireCurrentStateChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::WorkflowModel::fireStateEntered(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void governikus::WorkflowModel::fireResultChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::WorkflowModel::fireReaderPluginTypeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 6
void governikus::WorkflowModel::fireSelectedReaderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::WorkflowModel::fireReaderImageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::WorkflowModel::fireNextWorkflowPendingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::WorkflowModel::fireSupportedPluginTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::WorkflowModel::fireRemoveCardFeedbackChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void governikus::WorkflowModel::fireHasCardChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::WorkflowModel::fireEidTypeMismatchErrorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void governikus::WorkflowModel::fireShowUiRequest(UiModule _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void governikus::WorkflowModel::fireWorkflowFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void governikus::WorkflowModel::fireOnPinUnlocked()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void governikus::WorkflowModel::fireOnPasswordUsed()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void governikus::WorkflowModel::fireOnCanSuccess()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void governikus::WorkflowModel::fireLastReturnCodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}
namespace {
struct qt_meta_tag_ZN10governikus14EnumGAnimationE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumGAnimation::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumGAnimationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumGAnimation",
        "QML.Element",
        "GAnimation",
        "NONE",
        "PIN_ERROR",
        "CAN_ERROR",
        "PUK_ERROR",
        "PUK_BLOCKED",
        "CARD_ERROR",
        "NETWORK_ERROR",
        "CHANGEPIN_SUCCESS",
        "STATUS_OK",
        "STATUS_ERROR"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GAnimation'
        QtMocHelpers::EnumData<GAnimation>(2, 2, QMC::EnumIsScoped).add({
            {    3, GAnimation::NONE },
            {    4, GAnimation::PIN_ERROR },
            {    5, GAnimation::CAN_ERROR },
            {    6, GAnimation::PUK_ERROR },
            {    7, GAnimation::PUK_BLOCKED },
            {    8, GAnimation::CARD_ERROR },
            {    9, GAnimation::NETWORK_ERROR },
            {   10, GAnimation::CHANGEPIN_SUCCESS },
            {   11, GAnimation::STATUS_OK },
            {   12, GAnimation::STATUS_ERROR },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus14EnumGAnimationE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus14EnumGAnimationE =
    governikus::EnumGAnimation::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumGAnimationE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus14EnumGAnimationE =
    qt_staticMetaObjectContent_ZN10governikus14EnumGAnimationE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumGAnimationE =
    qt_staticMetaObjectContent_ZN10governikus14EnumGAnimationE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumGAnimation::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumGAnimationE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumGAnimationE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumGAnimationE.metaTypes,
    nullptr
} };

QT_WARNING_POP
