#ifndef AusweisAppGlobal_AusweisAppUiQml_H
#define AusweisAppGlobal_AusweisAppUiQml_H
#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <GlobalStatus.h>
#include <CardReturnCode.h>
namespace EnumGlobalStatusCodeForeign{
    Q_NAMESPACE
    QML_FOREIGN_NAMESPACE(governikus::EnumGlobalStatusCode)
    QML_NAMED_ELEMENT(GlobalStatusCode)
}

namespace EnumCardReturnCodeForeign{
    Q_NAMESPACE
    QML_FOREIGN_NAMESPACE(governikus::EnumCardReturnCode)
    QML_NAMED_ELEMENT(CardReturnCode)
}

#endif // AusweisAppGlobal_AusweisAppUiQml_H
