#ifndef AusweisAppUi_AusweisAppUiQml_H
#define AusweisAppUi_AusweisAppUiQml_H
#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <UiPlugin.h>
namespace EnumUiModuleForeign{
    Q_NAMESPACE
    QML_FOREIGN_NAMESPACE(governikus::EnumUiModule)
    QML_NAMED_ELEMENT(UiModule)
}

#endif // AusweisAppUi_AusweisAppUiQml_H
