/****************************************************************************
** Meta object code from reading C++ file 'AccessRoleAndRight.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/asn1/AccessRoleAndRight.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AccessRoleAndRight.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus15EnumAccessRightE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumAccessRight::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15EnumAccessRightE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumAccessRight",
        "AccessRight",
        "WRITE_DG17",
        "WRITE_DG18",
        "WRITE_DG19",
        "WRITE_DG20",
        "WRITE_DG21",
        "WRITE_DG22",
        "RFU",
        "PSA",
        "READ_DG22",
        "READ_DG21",
        "READ_DG20",
        "READ_DG19",
        "READ_DG18",
        "READ_DG17",
        "READ_DG16",
        "READ_DG15",
        "READ_DG14",
        "READ_DG13",
        "READ_DG12",
        "READ_DG11",
        "READ_DG10",
        "READ_DG09",
        "READ_DG08",
        "READ_DG07",
        "READ_DG06",
        "READ_DG05",
        "READ_DG04",
        "READ_DG03",
        "READ_DG02",
        "READ_DG01",
        "INSTALL_QUAL_CERT",
        "INSTALL_CERT",
        "PIN_MANAGEMENT",
        "CAN_ALLOWED",
        "PRIVILEGED_TERMINAL",
        "RESTRICTED_IDENTIFICATION",
        "COMMUNITY_ID_VERIFICATION",
        "AGE_VERIFICATION"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AccessRight'
        QtMocHelpers::EnumData<AccessRight>(1, 1, QMC::EnumIsScoped).add({
            {    2, AccessRight::WRITE_DG17 },
            {    3, AccessRight::WRITE_DG18 },
            {    4, AccessRight::WRITE_DG19 },
            {    5, AccessRight::WRITE_DG20 },
            {    6, AccessRight::WRITE_DG21 },
            {    7, AccessRight::WRITE_DG22 },
            {    8, AccessRight::RFU },
            {    9, AccessRight::PSA },
            {   10, AccessRight::READ_DG22 },
            {   11, AccessRight::READ_DG21 },
            {   12, AccessRight::READ_DG20 },
            {   13, AccessRight::READ_DG19 },
            {   14, AccessRight::READ_DG18 },
            {   15, AccessRight::READ_DG17 },
            {   16, AccessRight::READ_DG16 },
            {   17, AccessRight::READ_DG15 },
            {   18, AccessRight::READ_DG14 },
            {   19, AccessRight::READ_DG13 },
            {   20, AccessRight::READ_DG12 },
            {   21, AccessRight::READ_DG11 },
            {   22, AccessRight::READ_DG10 },
            {   23, AccessRight::READ_DG09 },
            {   24, AccessRight::READ_DG08 },
            {   25, AccessRight::READ_DG07 },
            {   26, AccessRight::READ_DG06 },
            {   27, AccessRight::READ_DG05 },
            {   28, AccessRight::READ_DG04 },
            {   29, AccessRight::READ_DG03 },
            {   30, AccessRight::READ_DG02 },
            {   31, AccessRight::READ_DG01 },
            {   32, AccessRight::INSTALL_QUAL_CERT },
            {   33, AccessRight::INSTALL_CERT },
            {   34, AccessRight::PIN_MANAGEMENT },
            {   35, AccessRight::CAN_ALLOWED },
            {   36, AccessRight::PRIVILEGED_TERMINAL },
            {   37, AccessRight::RESTRICTED_IDENTIFICATION },
            {   38, AccessRight::COMMUNITY_ID_VERIFICATION },
            {   39, AccessRight::AGE_VERIFICATION },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus15EnumAccessRightE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus15EnumAccessRightE =
    governikus::EnumAccessRight::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15EnumAccessRightE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus15EnumAccessRightE =
    qt_staticMetaObjectContent_ZN10governikus15EnumAccessRightE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus15EnumAccessRightE =
    qt_staticMetaObjectContent_ZN10governikus15EnumAccessRightE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumAccessRight::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus15EnumAccessRightE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus15EnumAccessRightE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus15EnumAccessRightE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus20EnumAccessRightNamesE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumAccessRightNames::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumAccessRightNamesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumAccessRightNames",
        "AccessRightNames",
        "ResidencePermitII",
        "ResidencePermitI",
        "CommunityID",
        "Address",
        "BirthName",
        "Nationality",
        "PlaceOfBirth",
        "DateOfBirth",
        "DoctoralDegree",
        "ArtisticName",
        "FamilyName",
        "GivenNames",
        "ValidUntil",
        "IssuingCountry",
        "DocumentType",
        "CanAllowed",
        "Pseudonym",
        "AddressVerification",
        "AgeVerification",
        "WriteAddress",
        "WriteCommunityID",
        "WriteResidencePermitI",
        "WriteResidencePermitII",
        "PinManagement"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AccessRightNames'
        QtMocHelpers::EnumData<AccessRightNames>(1, 1, QMC::EnumIsScoped).add({
            {    2, AccessRightNames::ResidencePermitII },
            {    3, AccessRightNames::ResidencePermitI },
            {    4, AccessRightNames::CommunityID },
            {    5, AccessRightNames::Address },
            {    6, AccessRightNames::BirthName },
            {    7, AccessRightNames::Nationality },
            {    8, AccessRightNames::PlaceOfBirth },
            {    9, AccessRightNames::DateOfBirth },
            {   10, AccessRightNames::DoctoralDegree },
            {   11, AccessRightNames::ArtisticName },
            {   12, AccessRightNames::FamilyName },
            {   13, AccessRightNames::GivenNames },
            {   14, AccessRightNames::ValidUntil },
            {   15, AccessRightNames::IssuingCountry },
            {   16, AccessRightNames::DocumentType },
            {   17, AccessRightNames::CanAllowed },
            {   18, AccessRightNames::Pseudonym },
            {   19, AccessRightNames::AddressVerification },
            {   20, AccessRightNames::AgeVerification },
            {   21, AccessRightNames::WriteAddress },
            {   22, AccessRightNames::WriteCommunityID },
            {   23, AccessRightNames::WriteResidencePermitI },
            {   24, AccessRightNames::WriteResidencePermitII },
            {   25, AccessRightNames::PinManagement },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus20EnumAccessRightNamesE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus20EnumAccessRightNamesE =
    governikus::EnumAccessRightNames::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumAccessRightNamesE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus20EnumAccessRightNamesE =
    qt_staticMetaObjectContent_ZN10governikus20EnumAccessRightNamesE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumAccessRightNamesE =
    qt_staticMetaObjectContent_ZN10governikus20EnumAccessRightNamesE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumAccessRightNames::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumAccessRightNamesE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumAccessRightNamesE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumAccessRightNamesE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus14EnumAccessRoleE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumAccessRole::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumAccessRoleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumAccessRole",
        "AccessRole",
        "CVCA",
        "DV_od",
        "DV_no_f",
        "AT",
        "UNKNOWN"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AccessRole'
        QtMocHelpers::EnumData<AccessRole>(1, 1, QMC::EnumIsScoped).add({
            {    2, AccessRole::CVCA },
            {    3, AccessRole::DV_od },
            {    4, AccessRole::DV_no_f },
            {    5, AccessRole::AT },
            {    6, AccessRole::UNKNOWN },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus14EnumAccessRoleE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus14EnumAccessRoleE =
    governikus::EnumAccessRole::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumAccessRoleE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus14EnumAccessRoleE =
    qt_staticMetaObjectContent_ZN10governikus14EnumAccessRoleE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumAccessRoleE =
    qt_staticMetaObjectContent_ZN10governikus14EnumAccessRoleE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumAccessRole::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumAccessRoleE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumAccessRoleE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumAccessRoleE.metaTypes,
    nullptr
} };

QT_WARNING_POP
