/****************************************************************************
** Meta object code from reading C++ file 'ECardApiResult.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/global/ECardApiResult.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ECardApiResult.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus14ECardApiResultE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::ECardApiResult::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14ECardApiResultE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::ECardApiResult",
        "Major",
        "Unknown",
        "Ok",
        "Warning",
        "Error",
        "Minor",
        "null",
        "AL_Unknown_Error",
        "AL_No_Permission",
        "AL_Internal_Error",
        "AL_Parameter_Error",
        "AL_Unknown_API_Function",
        "AL_Not_Initialized",
        "AL_Warning_Connection_Disconnected",
        "AL_Session_Terminated_Warning",
        "AL_Communication_Error",
        "DP_Timeout_Error",
        "DP_Unknown_Channel_Handle",
        "DP_Communication_Error",
        "DP_Trusted_Channel_Establishment_Failed",
        "DP_Unknown_Protocol",
        "DP_Unknown_Cipher_Suite",
        "DP_Unknown_Webservice_Binding",
        "DP_Node_Not_Reachable",
        "IFDL_Timeout_Error",
        "IFDL_UnknownSlot",
        "IFDL_InvalidSlotHandle",
        "IFDL_CancellationByUser",
        "IFDL_IFD_SharingViolation",
        "IFDL_Terminal_NoCard",
        "IFDL_IO_RepeatedDataMismatch",
        "IFDL_IO_UnknownPINFormat",
        "IL_Signature_InvalidCertificatePath",
        "KEY_KeyGenerationNotPossible",
        "SAL_Cancellation_by_User",
        "SAL_Invalid_Key",
        "SAL_SecurityConditionNotSatisfied",
        "SAL_MEAC_AgeVerificationFailedWarning",
        "SAL_MEAC_CommunityVerificationFailedWarning",
        "SAL_MEAC_DocumentValidityVerificationFailed",
        "Origin",
        "Server",
        "Client"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Major'
        QtMocHelpers::EnumData<enum Major>(1, 1, QMC::EnumIsScoped).add({
            {    2, Major::Unknown },
            {    3, Major::Ok },
            {    4, Major::Warning },
            {    5, Major::Error },
        }),
        // enum 'Minor'
        QtMocHelpers::EnumData<enum Minor>(6, 6, QMC::EnumIsScoped).add({
            {    7, Minor::null },
            {    8, Minor::AL_Unknown_Error },
            {    9, Minor::AL_No_Permission },
            {   10, Minor::AL_Internal_Error },
            {   11, Minor::AL_Parameter_Error },
            {   12, Minor::AL_Unknown_API_Function },
            {   13, Minor::AL_Not_Initialized },
            {   14, Minor::AL_Warning_Connection_Disconnected },
            {   15, Minor::AL_Session_Terminated_Warning },
            {   16, Minor::AL_Communication_Error },
            {   17, Minor::DP_Timeout_Error },
            {   18, Minor::DP_Unknown_Channel_Handle },
            {   19, Minor::DP_Communication_Error },
            {   20, Minor::DP_Trusted_Channel_Establishment_Failed },
            {   21, Minor::DP_Unknown_Protocol },
            {   22, Minor::DP_Unknown_Cipher_Suite },
            {   23, Minor::DP_Unknown_Webservice_Binding },
            {   24, Minor::DP_Node_Not_Reachable },
            {   25, Minor::IFDL_Timeout_Error },
            {   26, Minor::IFDL_UnknownSlot },
            {   27, Minor::IFDL_InvalidSlotHandle },
            {   28, Minor::IFDL_CancellationByUser },
            {   29, Minor::IFDL_IFD_SharingViolation },
            {   30, Minor::IFDL_Terminal_NoCard },
            {   31, Minor::IFDL_IO_RepeatedDataMismatch },
            {   32, Minor::IFDL_IO_UnknownPINFormat },
            {   33, Minor::IL_Signature_InvalidCertificatePath },
            {   34, Minor::KEY_KeyGenerationNotPossible },
            {   35, Minor::SAL_Cancellation_by_User },
            {   36, Minor::SAL_Invalid_Key },
            {   37, Minor::SAL_SecurityConditionNotSatisfied },
            {   38, Minor::SAL_MEAC_AgeVerificationFailedWarning },
            {   39, Minor::SAL_MEAC_CommunityVerificationFailedWarning },
            {   40, Minor::SAL_MEAC_DocumentValidityVerificationFailed },
        }),
        // enum 'Origin'
        QtMocHelpers::EnumData<enum Origin>(41, 41, QMC::EnumIsScoped).add({
            {   42, Origin::Server },
            {   43, Origin::Client },
        }),
    };
    return QtMocHelpers::metaObjectData<ECardApiResult, qt_meta_tag_ZN10governikus14ECardApiResultE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::ECardApiResult::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus14ECardApiResultE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus14ECardApiResultE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus14ECardApiResultE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
