/****************************************************************************
** Meta object code from reading C++ file 'PcscReaderFeature.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/pcsc/PcscReaderFeature.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PcscReaderFeature.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13EnumFeatureIDE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumFeatureID::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13EnumFeatureIDE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumFeatureID",
        "FeatureID",
        "VERIFY_PIN_START",
        "VERIFY_PIN_FINISH",
        "MODIFY_PIN_START",
        "MODIFY_PIN_FINISH",
        "GET_KEY_PRESSED",
        "VERIFY_PIN_DIRECT",
        "MODIFY_PIN_DIRECT",
        "MCT_READERDIRECT",
        "MCT_UNIVERSAL",
        "IFD_PIN_PROP",
        "ABORT",
        "SET_SPE_MESSAGE",
        "VERIFY_PIN_DIRECT_APP_ID",
        "MODIFY_PIN_DIRECT_APP_ID",
        "WRITE_DISPLAY",
        "GET_KEY",
        "IFD_DISPLAY_PROPERTIES",
        "TLV_PROPERTIES",
        "CCID_ESC_COMMAND",
        "EXECUTE_PACE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FeatureID'
        QtMocHelpers::EnumData<FeatureID>(1, 1, QMC::EnumIsScoped).add({
            {    2, FeatureID::VERIFY_PIN_START },
            {    3, FeatureID::VERIFY_PIN_FINISH },
            {    4, FeatureID::MODIFY_PIN_START },
            {    5, FeatureID::MODIFY_PIN_FINISH },
            {    6, FeatureID::GET_KEY_PRESSED },
            {    7, FeatureID::VERIFY_PIN_DIRECT },
            {    8, FeatureID::MODIFY_PIN_DIRECT },
            {    9, FeatureID::MCT_READERDIRECT },
            {   10, FeatureID::MCT_UNIVERSAL },
            {   11, FeatureID::IFD_PIN_PROP },
            {   12, FeatureID::ABORT },
            {   13, FeatureID::SET_SPE_MESSAGE },
            {   14, FeatureID::VERIFY_PIN_DIRECT_APP_ID },
            {   15, FeatureID::MODIFY_PIN_DIRECT_APP_ID },
            {   16, FeatureID::WRITE_DISPLAY },
            {   17, FeatureID::GET_KEY },
            {   18, FeatureID::IFD_DISPLAY_PROPERTIES },
            {   19, FeatureID::TLV_PROPERTIES },
            {   20, FeatureID::CCID_ESC_COMMAND },
            {   21, FeatureID::EXECUTE_PACE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus13EnumFeatureIDE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus13EnumFeatureIDE =
    governikus::EnumFeatureID::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13EnumFeatureIDE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus13EnumFeatureIDE =
    qt_staticMetaObjectContent_ZN10governikus13EnumFeatureIDE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus13EnumFeatureIDE =
    qt_staticMetaObjectContent_ZN10governikus13EnumFeatureIDE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumFeatureID::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus13EnumFeatureIDE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus13EnumFeatureIDE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus13EnumFeatureIDE.metaTypes,
    nullptr
} };

QT_WARNING_POP
