/****************************************************************************
** Meta object code from reading C++ file 'GeneralSettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/settings/GeneralSettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GeneralSettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus15GeneralSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::GeneralSettings::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15GeneralSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::GeneralSettings",
        "fireLanguageChanged",
        "",
        "firePreferredTechnologyChanged",
        "fireDeveloperOptionsChanged",
        "fireShowInAppNotificationsChanged",
        "fireProxyChanged",
        "fireUseSystemFontChanged",
        "fireUseAnimationsChanged",
        "fireDarkModeChanged",
        "fireScreenPrivacyChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireLanguageChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePreferredTechnologyChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDeveloperOptionsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireShowInAppNotificationsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireProxyChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUseSystemFontChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUseAnimationsChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireDarkModeChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireScreenPrivacyChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GeneralSettings, qt_meta_tag_ZN10governikus15GeneralSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::GeneralSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractSettings::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15GeneralSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15GeneralSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus15GeneralSettingsE_t>.metaTypes,
    nullptr
} };

void governikus::GeneralSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GeneralSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireLanguageChanged(); break;
        case 1: _t->firePreferredTechnologyChanged(); break;
        case 2: _t->fireDeveloperOptionsChanged(); break;
        case 3: _t->fireShowInAppNotificationsChanged(); break;
        case 4: _t->fireProxyChanged(); break;
        case 5: _t->fireUseSystemFontChanged(); break;
        case 6: _t->fireUseAnimationsChanged(); break;
        case 7: _t->fireDarkModeChanged(); break;
        case 8: _t->fireScreenPrivacyChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireLanguageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::firePreferredTechnologyChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireDeveloperOptionsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireShowInAppNotificationsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireProxyChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireUseSystemFontChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireUseAnimationsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireDarkModeChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GeneralSettings::*)()>(_a, &GeneralSettings::fireScreenPrivacyChanged, 8))
            return;
    }
}

const QMetaObject *governikus::GeneralSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::GeneralSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus15GeneralSettingsE_t>.strings))
        return static_cast<void*>(this);
    return AbstractSettings::qt_metacast(_clname);
}

int governikus::GeneralSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractSettings::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void governikus::GeneralSettings::fireLanguageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::GeneralSettings::firePreferredTechnologyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::GeneralSettings::fireDeveloperOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::GeneralSettings::fireShowInAppNotificationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::GeneralSettings::fireProxyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::GeneralSettings::fireUseSystemFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::GeneralSettings::fireUseAnimationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::GeneralSettings::fireDarkModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::GeneralSettings::fireScreenPrivacyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
