/****************************************************************************
** Meta object code from reading C++ file 'IfdDispatcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/base/IfdDispatcher.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IfdDispatcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus13IfdDispatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::IfdDispatcher::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus13IfdDispatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::IfdDispatcher",
        "fireReceived",
        "",
        "IfdMessageType",
        "pMessageType",
        "QJsonObject",
        "pJsonObject",
        "pId",
        "fireClosed",
        "GlobalStatus::Code",
        "pCloseCode",
        "onReceived",
        "pDataBlock",
        "onClosed",
        "send",
        "QSharedPointer<const IfdMessage>",
        "pMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireReceived'
        QtMocHelpers::SignalData<void(IfdMessageType, const QJsonObject &, const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QByteArray, 7 },
        }}),
        // Signal 'fireClosed'
        QtMocHelpers::SignalData<void(GlobalStatus::Code, const QByteArray &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QByteArray, 7 },
        }}),
        // Slot 'onReceived'
        QtMocHelpers::SlotData<void(const QByteArray &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'onClosed'
        QtMocHelpers::SlotData<void(GlobalStatus::Code)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'send'
        QtMocHelpers::MethodData<void(const QSharedPointer<const IfdMessage> &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IfdDispatcher, qt_meta_tag_ZN10governikus13IfdDispatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::IfdDispatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13IfdDispatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13IfdDispatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus13IfdDispatcherE_t>.metaTypes,
    nullptr
} };

void governikus::IfdDispatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IfdDispatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireReceived((*reinterpret_cast<std::add_pointer_t<IfdMessageType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJsonObject>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 1: _t->fireClosed((*reinterpret_cast<std::add_pointer_t<GlobalStatus::Code>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        case 2: _t->onReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->onClosed((*reinterpret_cast<std::add_pointer_t<GlobalStatus::Code>>(_a[1]))); break;
        case 4: _t->send((*reinterpret_cast<std::add_pointer_t<QSharedPointer<const IfdMessage>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (IfdDispatcher::*)(IfdMessageType , const QJsonObject & , const QByteArray & )>(_a, &IfdDispatcher::fireReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdDispatcher::*)(GlobalStatus::Code , const QByteArray & )>(_a, &IfdDispatcher::fireClosed, 1))
            return;
    }
}

const QMetaObject *governikus::IfdDispatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::IfdDispatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus13IfdDispatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::IfdDispatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void governikus::IfdDispatcher::fireReceived(IfdMessageType _t1, const QJsonObject & _t2, const QByteArray & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void governikus::IfdDispatcher::fireClosed(GlobalStatus::Code _t1, const QByteArray & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
