/****************************************************************************
** Meta object code from reading C++ file 'SecurityProtocol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/SecurityProtocol.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SecurityProtocol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus16EnumProtocolTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumProtocolType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumProtocolTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumProtocolType",
        "ProtocolType",
        "UNDEFINED",
        "TA",
        "CA",
        "PACE",
        "RI"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ProtocolType'
        QtMocHelpers::EnumData<ProtocolType>(1, 1, QMC::EnumIsScoped).add({
            {    2, ProtocolType::UNDEFINED },
            {    3, ProtocolType::TA },
            {    4, ProtocolType::CA },
            {    5, ProtocolType::PACE },
            {    6, ProtocolType::RI },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus16EnumProtocolTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus16EnumProtocolTypeE =
    governikus::EnumProtocolType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumProtocolTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus16EnumProtocolTypeE =
    qt_staticMetaObjectContent_ZN10governikus16EnumProtocolTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumProtocolTypeE =
    qt_staticMetaObjectContent_ZN10governikus16EnumProtocolTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumProtocolType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumProtocolTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumProtocolTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumProtocolTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus20EnumKeyAgreementTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumKeyAgreementType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumKeyAgreementTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumKeyAgreementType",
        "KeyAgreementType",
        "UNDEFINED",
        "DH",
        "ECDH"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'KeyAgreementType'
        QtMocHelpers::EnumData<KeyAgreementType>(1, 1, QMC::EnumIsScoped).add({
            {    2, KeyAgreementType::UNDEFINED },
            {    3, KeyAgreementType::DH },
            {    4, KeyAgreementType::ECDH },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus20EnumKeyAgreementTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus20EnumKeyAgreementTypeE =
    governikus::EnumKeyAgreementType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20EnumKeyAgreementTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus20EnumKeyAgreementTypeE =
    qt_staticMetaObjectContent_ZN10governikus20EnumKeyAgreementTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumKeyAgreementTypeE =
    qt_staticMetaObjectContent_ZN10governikus20EnumKeyAgreementTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumKeyAgreementType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumKeyAgreementTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus20EnumKeyAgreementTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus20EnumKeyAgreementTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus15EnumMappingTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumMappingType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15EnumMappingTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumMappingType",
        "MappingType",
        "UNDEFINED",
        "GM",
        "IM",
        "CAM"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MappingType'
        QtMocHelpers::EnumData<MappingType>(1, 1, QMC::EnumIsScoped).add({
            {    2, MappingType::UNDEFINED },
            {    3, MappingType::GM },
            {    4, MappingType::IM },
            {    5, MappingType::CAM },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus15EnumMappingTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus15EnumMappingTypeE =
    governikus::EnumMappingType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus15EnumMappingTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus15EnumMappingTypeE =
    qt_staticMetaObjectContent_ZN10governikus15EnumMappingTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus15EnumMappingTypeE =
    qt_staticMetaObjectContent_ZN10governikus15EnumMappingTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumMappingType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus15EnumMappingTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus15EnumMappingTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus15EnumMappingTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus14EnumCipherTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumCipherType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumCipherTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumCipherType",
        "CipherType",
        "UNDEFINED",
        "DES3_CBC",
        "AES_128_CBC",
        "AES_192_CBC",
        "AES_256_CBC"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CipherType'
        QtMocHelpers::EnumData<CipherType>(1, 1, QMC::EnumIsScoped).add({
            {    2, CipherType::UNDEFINED },
            {    3, CipherType::DES3_CBC },
            {    4, CipherType::AES_128_CBC },
            {    5, CipherType::AES_192_CBC },
            {    6, CipherType::AES_256_CBC },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus14EnumCipherTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus14EnumCipherTypeE =
    governikus::EnumCipherType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumCipherTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus14EnumCipherTypeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumCipherTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumCipherTypeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumCipherTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumCipherType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumCipherTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumCipherTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumCipherTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus17EnumSignatureTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumSignatureType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus17EnumSignatureTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumSignatureType",
        "SignatureType",
        "UNDEFINED",
        "RSA",
        "ECDSA"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SignatureType'
        QtMocHelpers::EnumData<SignatureType>(1, 1, QMC::EnumIsScoped).add({
            {    2, SignatureType::UNDEFINED },
            {    3, SignatureType::RSA },
            {    4, SignatureType::ECDSA },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus17EnumSignatureTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus17EnumSignatureTypeE =
    governikus::EnumSignatureType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus17EnumSignatureTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus17EnumSignatureTypeE =
    qt_staticMetaObjectContent_ZN10governikus17EnumSignatureTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus17EnumSignatureTypeE =
    qt_staticMetaObjectContent_ZN10governikus17EnumSignatureTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumSignatureType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus17EnumSignatureTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus17EnumSignatureTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus17EnumSignatureTypeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
