/****************************************************************************
** Meta object code from reading C++ file 'CommandApdu.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/apdu/CommandApdu.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CommandApdu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus7EnumInsE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumIns::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus7EnumInsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumIns",
        "Ins",
        "UNKNOWN",
        "DEACTIVATE",
        "VERIFY",
        "MSE_SET",
        "ACTIVATE",
        "EXTERNAL_AUTHENTICATE",
        "GET_CHALLENGE",
        "GENERAL_AUTHENTICATE",
        "PSO_VERIFY",
        "PSO_COMPUTE",
        "RESET_RETRY_COUNTER",
        "SELECT",
        "READ_BINARY",
        "UPDATE_BINARY"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Ins'
        QtMocHelpers::EnumData<Ins>(1, 1, QMC::EnumIsScoped).add({
            {    2, Ins::UNKNOWN },
            {    3, Ins::DEACTIVATE },
            {    4, Ins::VERIFY },
            {    5, Ins::MSE_SET },
            {    6, Ins::ACTIVATE },
            {    7, Ins::EXTERNAL_AUTHENTICATE },
            {    8, Ins::GET_CHALLENGE },
            {    9, Ins::GENERAL_AUTHENTICATE },
            {   10, Ins::PSO_VERIFY },
            {   11, Ins::PSO_COMPUTE },
            {   12, Ins::RESET_RETRY_COUNTER },
            {   13, Ins::SELECT },
            {   14, Ins::READ_BINARY },
            {   15, Ins::UPDATE_BINARY },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus7EnumInsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus7EnumInsE =
    governikus::EnumIns::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus7EnumInsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus7EnumInsE =
    qt_staticMetaObjectContent_ZN10governikus7EnumInsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus7EnumInsE =
    qt_staticMetaObjectContent_ZN10governikus7EnumInsE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumIns::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus7EnumInsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus7EnumInsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus7EnumInsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
