/****************************************************************************
** Meta object code from reading C++ file 'SelfAuthenticationData.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/workflows/selfauth/SelfAuthenticationData.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SelfAuthenticationData.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus26EnumSelfAuthDataPermissionE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumSelfAuthDataPermission::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus26EnumSelfAuthDataPermissionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumSelfAuthDataPermission",
        "SelfAuthDataPermission",
        "UNKNOWN",
        "ALLOWED",
        "PROHIBITED",
        "NOTONCHIP"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelfAuthDataPermission'
        QtMocHelpers::EnumData<SelfAuthDataPermission>(1, 1, QMC::EnumIsScoped).add({
            {    2, SelfAuthDataPermission::UNKNOWN },
            {    3, SelfAuthDataPermission::ALLOWED },
            {    4, SelfAuthDataPermission::PROHIBITED },
            {    5, SelfAuthDataPermission::NOTONCHIP },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus26EnumSelfAuthDataPermissionE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus26EnumSelfAuthDataPermissionE =
    governikus::EnumSelfAuthDataPermission::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus26EnumSelfAuthDataPermissionE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus26EnumSelfAuthDataPermissionE =
    qt_staticMetaObjectContent_ZN10governikus26EnumSelfAuthDataPermissionE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus26EnumSelfAuthDataPermissionE =
    qt_staticMetaObjectContent_ZN10governikus26EnumSelfAuthDataPermissionE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumSelfAuthDataPermission::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus26EnumSelfAuthDataPermissionE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus26EnumSelfAuthDataPermissionE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus26EnumSelfAuthDataPermissionE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus16EnumSelfAuthDataE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumSelfAuthData::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumSelfAuthDataE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumSelfAuthData",
        "SelfAuthData",
        "UNKNOWN",
        "DocumentType",
        "IssuingState",
        "DateOfExpiry",
        "GivenNames",
        "FamilyNames",
        "ArtisticName",
        "AcademicTitle",
        "BirthName",
        "DateOfBirth",
        "PlaceOfBirth",
        "PlaceOfResidence",
        "Nationality",
        "ResidencePermitI",
        "RestrictedID",
        "AgeVerification",
        "PlaceVerification",
        "PlaceOfResidenceStreet",
        "PlaceOfResidenceCity",
        "PlaceOfResidenceCountry",
        "PlaceOfResidenceZipCode",
        "PlaceOfResidenceNoPlaceInfo"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SelfAuthData'
        QtMocHelpers::EnumData<SelfAuthData>(1, 1, QMC::EnumIsScoped).add({
            {    2, SelfAuthData::UNKNOWN },
            {    3, SelfAuthData::DocumentType },
            {    4, SelfAuthData::IssuingState },
            {    5, SelfAuthData::DateOfExpiry },
            {    6, SelfAuthData::GivenNames },
            {    7, SelfAuthData::FamilyNames },
            {    8, SelfAuthData::ArtisticName },
            {    9, SelfAuthData::AcademicTitle },
            {   10, SelfAuthData::BirthName },
            {   11, SelfAuthData::DateOfBirth },
            {   12, SelfAuthData::PlaceOfBirth },
            {   13, SelfAuthData::PlaceOfResidence },
            {   14, SelfAuthData::Nationality },
            {   15, SelfAuthData::ResidencePermitI },
            {   16, SelfAuthData::RestrictedID },
            {   17, SelfAuthData::AgeVerification },
            {   18, SelfAuthData::PlaceVerification },
            {   19, SelfAuthData::PlaceOfResidenceStreet },
            {   20, SelfAuthData::PlaceOfResidenceCity },
            {   21, SelfAuthData::PlaceOfResidenceCountry },
            {   22, SelfAuthData::PlaceOfResidenceZipCode },
            {   23, SelfAuthData::PlaceOfResidenceNoPlaceInfo },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus16EnumSelfAuthDataE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus16EnumSelfAuthDataE =
    governikus::EnumSelfAuthData::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumSelfAuthDataE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus16EnumSelfAuthDataE =
    qt_staticMetaObjectContent_ZN10governikus16EnumSelfAuthDataE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumSelfAuthDataE =
    qt_staticMetaObjectContent_ZN10governikus16EnumSelfAuthDataE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumSelfAuthData::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumSelfAuthDataE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumSelfAuthDataE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumSelfAuthDataE.metaTypes,
    nullptr
} };

QT_WARNING_POP
