/****************************************************************************
** Meta object code from reading C++ file 'RemoteIfdReaderManagerPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/remote/RemoteIfdReaderManagerPlugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RemoteIfdReaderManagerPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::RemoteIfdReaderManagerPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::RemoteIfdReaderManagerPlugin",
        "connectToPairedReaders",
        "",
        "continueConnectToPairedReaders",
        "QList<QSharedPointer<IfdListEntry>>",
        "pRemoteDevices",
        "onDeviceVanished",
        "QSharedPointer<IfdListEntry>",
        "pEntry",
        "onEstablishConnectionDone",
        "GlobalStatus",
        "pStatus"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'connectToPairedReaders'
        QtMocHelpers::SlotData<void() const>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'continueConnectToPairedReaders'
        QtMocHelpers::SlotData<void(const QList<QSharedPointer<IfdListEntry>> &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onDeviceVanished'
        QtMocHelpers::SlotData<void(const QSharedPointer<IfdListEntry> &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onEstablishConnectionDone'
        QtMocHelpers::SlotData<void(const QSharedPointer<IfdListEntry> &, const GlobalStatus &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RemoteIfdReaderManagerPlugin, qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::RemoteIfdReaderManagerPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<IfdReaderManagerPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>.metaTypes,
    nullptr
} };

void governikus::RemoteIfdReaderManagerPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteIfdReaderManagerPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectToPairedReaders(); break;
        case 1: _t->continueConnectToPairedReaders((*reinterpret_cast<std::add_pointer_t<QList<QSharedPointer<IfdListEntry>>>>(_a[1]))); break;
        case 2: _t->onDeviceVanished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1]))); break;
        case 3: _t->onEstablishConnectionDone((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *governikus::RemoteIfdReaderManagerPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::RemoteIfdReaderManagerPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus28RemoteIfdReaderManagerPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "governikus.ReaderManagerPlugin"))
        return static_cast< governikus::ReaderManagerPlugin*>(this);
    return IfdReaderManagerPlugin::qt_metacast(_clname);
}

int governikus::RemoteIfdReaderManagerPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IfdReaderManagerPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
using namespace governikus;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_RemoteIfdReaderManagerPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1e,  'g',  'o',  'v',  'e',  'r', 
    'n',  'i',  'k',  'u',  's',  '.',  'R',  'e', 
    'a',  'd',  'e',  'r',  'M',  'a',  'n',  'a', 
    'g',  'e',  'r',  'P',  'l',  'u',  'g',  'i', 
    'n', 
    // "className"
    0x03,  0x78,  0x1c,  'R',  'e',  'm',  'o',  't', 
    'e',  'I',  'f',  'd',  'R',  'e',  'a',  'd', 
    'e',  'r',  'M',  'a',  'n',  'a',  'g',  'e', 
    'r',  'P',  'l',  'u',  'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa2,  0x6c,  'd',  'e',  'p',  'e',  'n', 
    'd',  'e',  'n',  'c',  'i',  'e',  's',  0x80, 
    0x64,  'n',  'a',  'm',  'e',  0x78,  0x1c,  'R', 
    'e',  'm',  'o',  't',  'e',  'I',  'f',  'd', 
    'R',  'e',  'a',  'd',  'e',  'r',  'M',  'a', 
    'n',  'a',  'g',  'e',  'r',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(governikus::RemoteIfdReaderManagerPlugin, RemoteIfdReaderManagerPlugin, qt_pluginMetaDataV2_RemoteIfdReaderManagerPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_RemoteIfdReaderManagerPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1e,  'g',  'o',  'v',  'e',  'r', 
    'n',  'i',  'k',  'u',  's',  '.',  'R',  'e', 
    'a',  'd',  'e',  'r',  'M',  'a',  'n',  'a', 
    'g',  'e',  'r',  'P',  'l',  'u',  'g',  'i', 
    'n', 
    // "className"
    0x03,  0x78,  0x1c,  'R',  'e',  'm',  'o',  't', 
    'e',  'I',  'f',  'd',  'R',  'e',  'a',  'd', 
    'e',  'r',  'M',  'a',  'n',  'a',  'g',  'e', 
    'r',  'P',  'l',  'u',  'g',  'i',  'n', 
    // "MetaData"
    0x04,  0xa2,  0x6c,  'd',  'e',  'p',  'e',  'n', 
    'd',  'e',  'n',  'c',  'i',  'e',  's',  0x80, 
    0x64,  'n',  'a',  'm',  'e',  0x78,  0x1c,  'R', 
    'e',  'm',  'o',  't',  'e',  'I',  'f',  'd', 
    'R',  'e',  'a',  'd',  'e',  'r',  'M',  'a', 
    'n',  'a',  'g',  'e',  'r',  'P',  'l',  'u', 
    'g',  'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(governikus::RemoteIfdReaderManagerPlugin, RemoteIfdReaderManagerPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
