/****************************************************************************
** Meta object code from reading C++ file 'IfdConnector.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/base/IfdConnector.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IfdConnector.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus12IfdConnectorE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::IfdConnector::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12IfdConnectorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::IfdConnector",
        "fireDispatcherCreated",
        "",
        "pIfdId",
        "QSharedPointer<IfdDispatcherClient>",
        "pClientDispatcher",
        "fireDispatcherError",
        "IfdErrorCode",
        "pErrorCode",
        "onConnectRequest",
        "Discovery",
        "pDiscovery",
        "pPsk"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireDispatcherCreated'
        QtMocHelpers::SignalData<void(const QByteArray &, const QSharedPointer<IfdDispatcherClient> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'fireDispatcherError'
        QtMocHelpers::SignalData<void(const QByteArray &, IfdErrorCode)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onConnectRequest'
        QtMocHelpers::SlotData<void(const Discovery &, const QByteArray &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::QByteArray, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<IfdConnector, qt_meta_tag_ZN10governikus12IfdConnectorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::IfdConnector::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus12IfdConnectorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus12IfdConnectorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus12IfdConnectorE_t>.metaTypes,
    nullptr
} };

void governikus::IfdConnector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<IfdConnector *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireDispatcherCreated((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdDispatcherClient>>>(_a[2]))); break;
        case 1: _t->fireDispatcherError((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<IfdErrorCode>>(_a[2]))); break;
        case 2: _t->onConnectRequest((*reinterpret_cast<std::add_pointer_t<Discovery>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<IfdDispatcherClient> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (IfdConnector::*)(const QByteArray & , const QSharedPointer<IfdDispatcherClient> & )>(_a, &IfdConnector::fireDispatcherCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (IfdConnector::*)(const QByteArray & , IfdErrorCode )>(_a, &IfdConnector::fireDispatcherError, 1))
            return;
    }
}

const QMetaObject *governikus::IfdConnector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::IfdConnector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus12IfdConnectorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::IfdConnector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void governikus::IfdConnector::fireDispatcherCreated(const QByteArray & _t1, const QSharedPointer<IfdDispatcherClient> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void governikus::IfdConnector::fireDispatcherError(const QByteArray & _t1, IfdErrorCode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN10governikus16EnumIfdErrorCodeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumIfdErrorCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumIfdErrorCodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumIfdErrorCode",
        "IfdErrorCode",
        "INVALID_REQUEST",
        "NO_SUPPORTED_API_LEVEL",
        "CONNECTION_ERROR",
        "REMOTE_HOST_REFUSED_CONNECTION"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'IfdErrorCode'
        QtMocHelpers::EnumData<IfdErrorCode>(1, 1, QMC::EnumIsScoped).add({
            {    2, IfdErrorCode::INVALID_REQUEST },
            {    3, IfdErrorCode::NO_SUPPORTED_API_LEVEL },
            {    4, IfdErrorCode::CONNECTION_ERROR },
            {    5, IfdErrorCode::REMOTE_HOST_REFUSED_CONNECTION },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus16EnumIfdErrorCodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus16EnumIfdErrorCodeE =
    governikus::EnumIfdErrorCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus16EnumIfdErrorCodeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus16EnumIfdErrorCodeE =
    qt_staticMetaObjectContent_ZN10governikus16EnumIfdErrorCodeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumIfdErrorCodeE =
    qt_staticMetaObjectContent_ZN10governikus16EnumIfdErrorCodeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumIfdErrorCode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumIfdErrorCodeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus16EnumIfdErrorCodeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus16EnumIfdErrorCodeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
