/****************************************************************************
** Meta object code from reading C++ file 'StateEstablishPaceChannel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/workflows/base/states/StateEstablishPaceChannel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StateEstablishPaceChannel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::StateEstablishPaceChannel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::StateEstablishPaceChannel",
        "fireNoCardConnection",
        "",
        "fireWrongPassword",
        "fireThirdPinAttemptFailed",
        "fireAbortAndUnfortunateCardPosition",
        "firePaceChannelFailed",
        "firePaceCanEstablished",
        "firePacePukEstablished",
        "onEstablishConnectionDone",
        "QSharedPointer<BaseCardCommand>",
        "pCommand"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireNoCardConnection'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireWrongPassword'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireThirdPinAttemptFailed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAbortAndUnfortunateCardPosition'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePaceChannelFailed'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePaceCanEstablished'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePacePukEstablished'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onEstablishConnectionDone'
        QtMocHelpers::SlotData<void(QSharedPointer<BaseCardCommand>)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateEstablishPaceChannel, qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::StateEstablishPaceChannel::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractState::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>.metaTypes,
    nullptr
} };

void governikus::StateEstablishPaceChannel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateEstablishPaceChannel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireNoCardConnection(); break;
        case 1: _t->fireWrongPassword(); break;
        case 2: _t->fireThirdPinAttemptFailed(); break;
        case 3: _t->fireAbortAndUnfortunateCardPosition(); break;
        case 4: _t->firePaceChannelFailed(); break;
        case 5: _t->firePaceCanEstablished(); break;
        case 6: _t->firePacePukEstablished(); break;
        case 7: _t->onEstablishConnectionDone((*reinterpret_cast<std::add_pointer_t<QSharedPointer<BaseCardCommand>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<BaseCardCommand> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::fireNoCardConnection, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::fireWrongPassword, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::fireThirdPinAttemptFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::fireAbortAndUnfortunateCardPosition, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::firePaceChannelFailed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::firePaceCanEstablished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateEstablishPaceChannel::*)()>(_a, &StateEstablishPaceChannel::firePacePukEstablished, 6))
            return;
    }
}

const QMetaObject *governikus::StateEstablishPaceChannel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::StateEstablishPaceChannel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus25StateEstablishPaceChannelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "GenericContextContainer<WorkflowContext>"))
        return static_cast< GenericContextContainer<WorkflowContext>*>(this);
    return AbstractState::qt_metacast(_clname);
}

int governikus::StateEstablishPaceChannel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void governikus::StateEstablishPaceChannel::fireNoCardConnection()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::StateEstablishPaceChannel::fireWrongPassword()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::StateEstablishPaceChannel::fireThirdPinAttemptFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::StateEstablishPaceChannel::fireAbortAndUnfortunateCardPosition()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::StateEstablishPaceChannel::firePaceChannelFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void governikus::StateEstablishPaceChannel::firePaceCanEstablished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::StateEstablishPaceChannel::firePacePukEstablished()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
