/****************************************************************************
** Meta object code from reading C++ file 'AppUpdater.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/services/AppUpdater.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'AppUpdater.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus10AppUpdaterE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::AppUpdater::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus10AppUpdaterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::AppUpdater",
        "fireAppcastCheckFinished",
        "",
        "pUpdateAvailable",
        "GlobalStatus",
        "pError",
        "fireAppcastProgress",
        "pBytesReceived",
        "pBytesTotal",
        "fireAppDownloadFinished",
        "fireAppDownloadProgress",
        "fireDownloadTypeChanged",
        "onDownloadFinished",
        "QUrl",
        "pUpdateUrl",
        "pNewTimestamp",
        "pData",
        "onDownloadFailed",
        "GlobalStatus::Code",
        "pErrorCode",
        "onDownloadUnnecessary",
        "onDownloadProgress"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireAppcastCheckFinished'
        QtMocHelpers::SignalData<void(bool, const GlobalStatus &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'fireAppcastProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 }, { QMetaType::LongLong, 8 },
        }}),
        // Signal 'fireAppDownloadFinished'
        QtMocHelpers::SignalData<void(const GlobalStatus &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'fireAppDownloadProgress'
        QtMocHelpers::SignalData<void(qint64, qint64)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 7 }, { QMetaType::LongLong, 8 },
        }}),
        // Signal 'fireDownloadTypeChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDownloadFinished'
        QtMocHelpers::SlotData<void(const QUrl &, const QDateTime &, const QByteArray &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QDateTime, 15 }, { QMetaType::QByteArray, 16 },
        }}),
        // Slot 'onDownloadFailed'
        QtMocHelpers::SlotData<void(const QUrl &, GlobalStatus::Code)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'onDownloadUnnecessary'
        QtMocHelpers::SlotData<void(const QUrl &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'onDownloadProgress'
        QtMocHelpers::SlotData<void(const QUrl &, qint64, qint64)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::LongLong, 7 }, { QMetaType::LongLong, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AppUpdater, qt_meta_tag_ZN10governikus10AppUpdaterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::AppUpdater::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus10AppUpdaterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus10AppUpdaterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus10AppUpdaterE_t>.metaTypes,
    nullptr
} };

void governikus::AppUpdater::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppUpdater *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireAppcastCheckFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[2]))); break;
        case 1: _t->fireAppcastProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 2: _t->fireAppDownloadFinished((*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[1]))); break;
        case 3: _t->fireAppDownloadProgress((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 4: _t->fireDownloadTypeChanged(); break;
        case 5: _t->onDownloadFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3]))); break;
        case 6: _t->onDownloadFailed((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus::Code>>(_a[2]))); break;
        case 7: _t->onDownloadUnnecessary((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->onDownloadProgress((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AppUpdater::*)(bool , const GlobalStatus & )>(_a, &AppUpdater::fireAppcastCheckFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdater::*)(qint64 , qint64 )>(_a, &AppUpdater::fireAppcastProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdater::*)(const GlobalStatus & )>(_a, &AppUpdater::fireAppDownloadFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdater::*)(qint64 , qint64 )>(_a, &AppUpdater::fireAppDownloadProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppUpdater::*)()>(_a, &AppUpdater::fireDownloadTypeChanged, 4))
            return;
    }
}

const QMetaObject *governikus::AppUpdater::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::AppUpdater::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus10AppUpdaterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::AppUpdater::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void governikus::AppUpdater::fireAppcastCheckFinished(bool _t1, const GlobalStatus & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void governikus::AppUpdater::fireAppcastProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void governikus::AppUpdater::fireAppDownloadFinished(const GlobalStatus & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void governikus::AppUpdater::fireAppDownloadProgress(qint64 _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void governikus::AppUpdater::fireDownloadTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
