/****************************************************************************
** Meta object code from reading C++ file 'UiPluginQml.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/UiPluginQml.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UiPluginQml.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus11UiPluginQmlE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::UiPluginQml::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus11UiPluginQmlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::UiPluginQml",
        "fireTranslationChanged",
        "",
        "fireAppConfigChanged",
        "show",
        "doShutdown",
        "onWorkflowStarted",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "onWorkflowFinished",
        "onApplicationInitialized",
        "onApplicationStarted",
        "onShowUi",
        "UiModule",
        "pModule",
        "onHideUi",
        "onTranslationChanged",
        "onProxyAuthenticationRequired",
        "QNetworkProxy",
        "pProxy",
        "QAuthenticator*",
        "pAuthenticator",
        "onUiDomination",
        "const UiPlugin*",
        "pUi",
        "pInformation",
        "pAccepted",
        "onUiDominationReleased",
        "onShowUserInformation",
        "pMessage",
        "onUpdateScheduled",
        "onQmlWarnings",
        "QList<QQmlError>",
        "pWarnings",
        "onQmlObjectCreated",
        "const QObject*",
        "pObject",
        "onSceneGraphError",
        "QQuickWindow::SceneGraphError",
        "pError",
        "onRawLog",
        "pCategoryName",
        "onWindowPaletteChanged",
        "onUserDarkModeChanged",
        "onUseSystemFontChanged",
        "onTrayIconEnabledChanged",
        "onAppConfigChanged",
        "onReaderStatusChanged",
        "ReaderManagerPluginInfo",
        "pInfo",
        "hideFromTaskbar",
        "doRefresh"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireTranslationChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireAppConfigChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'show'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWorkflowStarted'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onWorkflowFinished'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onApplicationInitialized'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onApplicationStarted'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onShowUi'
        QtMocHelpers::SlotData<void(UiModule)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'onHideUi'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onProxyAuthenticationRequired'
        QtMocHelpers::SlotData<void(const QNetworkProxy &, QAuthenticator *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onUiDomination'
        QtMocHelpers::SlotData<void(const UiPlugin *, const QString &, bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 }, { QMetaType::QString, 25 }, { QMetaType::Bool, 26 },
        }}),
        // Slot 'onUiDominationReleased'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onShowUserInformation'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'onUpdateScheduled'
        QtMocHelpers::SlotData<void() const>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onQmlWarnings'
        QtMocHelpers::SlotData<void(const QList<QQmlError> &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'onQmlObjectCreated'
        QtMocHelpers::SlotData<void(const QObject *)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'onSceneGraphError'
        QtMocHelpers::SlotData<void(QQuickWindow::SceneGraphError, const QString &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'onRawLog'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 }, { QMetaType::QString, 41 },
        }}),
        // Slot 'onWindowPaletteChanged'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onUserDarkModeChanged'
        QtMocHelpers::SlotData<void() const>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onUseSystemFontChanged'
        QtMocHelpers::SlotData<void() const>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTrayIconEnabledChanged'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onAppConfigChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onReaderStatusChanged'
        QtMocHelpers::SlotData<void(const ReaderManagerPluginInfo &) const>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Method 'hideFromTaskbar'
        QtMocHelpers::MethodData<void() const>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'doRefresh'
        QtMocHelpers::MethodData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UiPluginQml, qt_meta_tag_ZN10governikus11UiPluginQmlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::UiPluginQml::staticMetaObject = { {
    QMetaObject::SuperData::link<UiPluginModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11UiPluginQmlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11UiPluginQmlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus11UiPluginQmlE_t>.metaTypes,
    nullptr
} };

void governikus::UiPluginQml::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UiPluginQml *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireTranslationChanged(); break;
        case 1: _t->fireAppConfigChanged(); break;
        case 2: _t->show(); break;
        case 3: _t->doShutdown(); break;
        case 4: _t->onWorkflowStarted((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 5: _t->onWorkflowFinished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 6: _t->onApplicationInitialized(); break;
        case 7: _t->onApplicationStarted(); break;
        case 8: _t->onShowUi((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 9: _t->onHideUi(); break;
        case 10: _t->onTranslationChanged(); break;
        case 11: _t->onProxyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        case 12: _t->onUiDomination((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 13: _t->onUiDominationReleased(); break;
        case 14: _t->onShowUserInformation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->onUpdateScheduled(); break;
        case 16: _t->onQmlWarnings((*reinterpret_cast<std::add_pointer_t<QList<QQmlError>>>(_a[1]))); break;
        case 17: _t->onQmlObjectCreated((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1]))); break;
        case 18: _t->onSceneGraphError((*reinterpret_cast<std::add_pointer_t<QQuickWindow::SceneGraphError>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 19: _t->onRawLog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->onWindowPaletteChanged(); break;
        case 21: _t->onUserDarkModeChanged(); break;
        case 22: _t->onUseSystemFontChanged(); break;
        case 23: _t->onTrayIconEnabledChanged(); break;
        case 24: _t->onAppConfigChanged(); break;
        case 25: _t->onReaderStatusChanged((*reinterpret_cast<std::add_pointer_t<ReaderManagerPluginInfo>>(_a[1]))); break;
        case 26: _t->hideFromTaskbar(); break;
        case 27: _t->doRefresh(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkProxy >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UiPluginQml::*)()>(_a, &UiPluginQml::fireTranslationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPluginQml::*)()>(_a, &UiPluginQml::fireAppConfigChanged, 1))
            return;
    }
}

const QMetaObject *governikus::UiPluginQml::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::UiPluginQml::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11UiPluginQmlE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "governikus.UiPlugin"))
        return static_cast< governikus::UiPlugin*>(this);
    return UiPluginModel::qt_metacast(_clname);
}

int governikus::UiPluginQml::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UiPluginModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void governikus::UiPluginQml::fireTranslationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::UiPluginQml::fireAppConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
using namespace governikus;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_UiPluginQml[] = {
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x6b,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'Q',  'm',  'l', 
    // "MetaData"
    0x04,  0xa3,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5,  0x63,  'e',  'n',  'v',  0xa1, 
    0x68,  'Q',  'S',  'G',  '_',  'I',  'N',  'F', 
    'O',  0x61,  '1',  0x6f,  'u',  's',  'e',  'r', 
    'I',  'n',  't',  'e',  'r',  'a',  'c',  't', 
    'i',  'v',  'e',  0xf5, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(governikus::UiPluginQml, UiPluginQml, qt_pluginMetaDataV2_UiPluginQml)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_UiPluginQml[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x6b,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'Q',  'm',  'l', 
    // "MetaData"
    0x04,  0xa3,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5,  0x63,  'e',  'n',  'v',  0xa1, 
    0x68,  'Q',  'S',  'G',  '_',  'I',  'N',  'F', 
    'O',  0x61,  '1',  0x6f,  'u',  's',  'e',  'r', 
    'I',  'n',  't',  'e',  'r',  'a',  'c',  't', 
    'i',  'v',  'e',  0xf5, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(governikus::UiPluginQml, UiPluginQml)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
