/****************************************************************************
** Meta object code from reading C++ file 'IfdMessage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/base/messages/IfdMessage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IfdMessage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus18EnumIfdMessageTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumIfdMessageType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumIfdMessageTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumIfdMessageType",
        "IfdMessageType",
        "IFDEstablishContext",
        "IFDEstablishContextResponse",
        "IFDGetStatus",
        "IFDStatus",
        "IFDConnect",
        "IFDConnectResponse",
        "IFDDisconnect",
        "IFDDisconnectResponse",
        "IFDError",
        "IFDTransmit",
        "IFDTransmitResponse",
        "IFDEstablishPACEChannel",
        "IFDEstablishPACEChannelResponse",
        "IFDDestroyPACEChannel",
        "IFDDestroyPACEChannelResponse",
        "IFDModifyPIN",
        "IFDModifyPINResponse",
        "UNDEFINED"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'IfdMessageType'
        QtMocHelpers::EnumData<IfdMessageType>(1, 1, QMC::EnumIsScoped).add({
            {    2, IfdMessageType::IFDEstablishContext },
            {    3, IfdMessageType::IFDEstablishContextResponse },
            {    4, IfdMessageType::IFDGetStatus },
            {    5, IfdMessageType::IFDStatus },
            {    6, IfdMessageType::IFDConnect },
            {    7, IfdMessageType::IFDConnectResponse },
            {    8, IfdMessageType::IFDDisconnect },
            {    9, IfdMessageType::IFDDisconnectResponse },
            {   10, IfdMessageType::IFDError },
            {   11, IfdMessageType::IFDTransmit },
            {   12, IfdMessageType::IFDTransmitResponse },
            {   13, IfdMessageType::IFDEstablishPACEChannel },
            {   14, IfdMessageType::IFDEstablishPACEChannelResponse },
            {   15, IfdMessageType::IFDDestroyPACEChannel },
            {   16, IfdMessageType::IFDDestroyPACEChannelResponse },
            {   17, IfdMessageType::IFDModifyPIN },
            {   18, IfdMessageType::IFDModifyPINResponse },
            {   19, IfdMessageType::UNDEFINED },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus18EnumIfdMessageTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus18EnumIfdMessageTypeE =
    governikus::EnumIfdMessageType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumIfdMessageTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus18EnumIfdMessageTypeE =
    qt_staticMetaObjectContent_ZN10governikus18EnumIfdMessageTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumIfdMessageTypeE =
    qt_staticMetaObjectContent_ZN10governikus18EnumIfdMessageTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumIfdMessageType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumIfdMessageTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumIfdMessageTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumIfdMessageTypeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
