/****************************************************************************
** Meta object code from reading C++ file 'CardReturnCode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/global/CardReturnCode.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CardReturnCode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus18EnumCardReturnCodeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumCardReturnCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumCardReturnCodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumCardReturnCode",
        "QML.Element",
        "CardReturnCode",
        "UNDEFINED",
        "OK",
        "OK_PUK",
        "OK_CAN",
        "RESPONSE_EMPTY",
        "CARD_NOT_FOUND",
        "UNKNOWN",
        "INPUT_TIME_OUT",
        "INVALID_CAN",
        "INVALID_PASSWORD",
        "INVALID_PIN",
        "INVALID_PIN_2",
        "INVALID_PIN_3",
        "INVALID_PUK",
        "COMMAND_FAILED",
        "CANCELLATION_BY_USER",
        "PIN_NOT_BLOCKED",
        "PROTOCOL_ERROR",
        "WRONG_LENGTH",
        "UNEXPECTED_TRANSMIT_STATUS"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CardReturnCode'
        QtMocHelpers::EnumData<CardReturnCode>(2, 2, QMC::EnumIsScoped).add({
            {    3, CardReturnCode::UNDEFINED },
            {    4, CardReturnCode::OK },
            {    5, CardReturnCode::OK_PUK },
            {    6, CardReturnCode::OK_CAN },
            {    7, CardReturnCode::RESPONSE_EMPTY },
            {    8, CardReturnCode::CARD_NOT_FOUND },
            {    9, CardReturnCode::UNKNOWN },
            {   10, CardReturnCode::INPUT_TIME_OUT },
            {   11, CardReturnCode::INVALID_CAN },
            {   12, CardReturnCode::INVALID_PASSWORD },
            {   13, CardReturnCode::INVALID_PIN },
            {   14, CardReturnCode::INVALID_PIN_2 },
            {   15, CardReturnCode::INVALID_PIN_3 },
            {   16, CardReturnCode::INVALID_PUK },
            {   17, CardReturnCode::COMMAND_FAILED },
            {   18, CardReturnCode::CANCELLATION_BY_USER },
            {   19, CardReturnCode::PIN_NOT_BLOCKED },
            {   20, CardReturnCode::PROTOCOL_ERROR },
            {   21, CardReturnCode::WRONG_LENGTH },
            {   22, CardReturnCode::UNEXPECTED_TRANSMIT_STATUS },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus18EnumCardReturnCodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus18EnumCardReturnCodeE =
    governikus::EnumCardReturnCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumCardReturnCodeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus18EnumCardReturnCodeE =
    qt_staticMetaObjectContent_ZN10governikus18EnumCardReturnCodeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumCardReturnCodeE =
    qt_staticMetaObjectContent_ZN10governikus18EnumCardReturnCodeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumCardReturnCode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumCardReturnCodeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumCardReturnCodeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumCardReturnCodeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
