/****************************************************************************
** Meta object code from reading C++ file 'FailureCode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../AusweisApp-2.5.1/src/global/FailureCode.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FailureCode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus11FailureCodeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::FailureCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus11FailureCodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::FailureCode",
        "Reason",
        "User_Cancelled",
        "Card_Removed",
        "Parse_TcToken_Invalid_Url",
        "Parse_TcToken_Missing_Url",
        "Get_TcToken_Invalid_Url",
        "Get_TcToken_Invalid_Redirect_Url",
        "Get_TcToken_Invalid_Certificate_Key_Length",
        "Get_TcToken_Invalid_Ephemeral_Key_Length",
        "Get_TcToken_Invalid_Server_Reply",
        "Get_TcToken_ServiceUnavailable",
        "Get_TcToken_Server_Error",
        "Get_TcToken_Client_Error",
        "Get_TcToken_Empty_Data",
        "Get_TcToken_Invalid_Data",
        "Get_TcToken_Network_Error",
        "Certificate_Check_Failed_No_Description",
        "Certificate_Check_Failed_No_SubjectUrl_In_Description",
        "Certificate_Check_Failed_Hash_Mismatch",
        "Certificate_Check_Failed_Same_Origin_Policy_Violation",
        "Certificate_Check_Failed_Hash_Missing_In_Description",
        "Pre_Verification_No_Test_Environment",
        "Pre_Verification_Invalid_Certificate_Chain",
        "Pre_Verification_Invalid_Certificate_Signature",
        "Pre_Verification_Certificate_Expired",
        "Extract_Cvcs_From_Eac1_No_Unique_At",
        "Extract_Cvcs_From_Eac1_No_Unique_Dv",
        "Extract_Cvcs_From_Eac1_At_Missing",
        "Extract_Cvcs_From_Eac1_Dv_Missing",
        "Connect_Card_Connection_Failed",
        "Connect_Card_Eid_Inactive",
        "Establish_Pace_Channel_Basic_Reader_No_Pin",
        "Establish_Pace_Channel_Puk_Inoperative",
        "Establish_Pace_Channel_Unknown_Password_Id",
        "Establish_Pace_Channel_User_Cancelled",
        "Maintain_Card_Connection_Pace_Unrecoverable",
        "Did_Authenticate_Eac1_Card_Command_Failed",
        "Process_Certificates_From_Eac2_Cvc_Chain_Missing",
        "Did_Authenticate_Eac2_Invalid_Cvc_Chain",
        "Did_Authenticate_Eac2_Card_Command_Failed",
        "Generic_Send_Receive_Paos_Unhandled",
        "Generic_Send_Receive_Network_Error",
        "Generic_Send_Receive_Tls_Error",
        "Generic_Send_Receive_Server_Error",
        "Generic_Send_Receive_Service_Unavailable",
        "Generic_Send_Receive_Client_Error",
        "Generic_Send_Receive_Paos_Unknown",
        "Generic_Send_Receive_Paos_Unexpected",
        "Generic_Send_Receive_Invalid_Ephemeral_Key_Length",
        "Generic_Send_Receive_Certificate_Error",
        "Generic_Send_Receive_Session_Resumption_Failed",
        "Transmit_Card_Command_Failed",
        "Start_Paos_Response_Missing",
        "Start_Paos_Response_Error",
        "Check_Refresh_Address_Fatal_Tls_Error_Before_Reply",
        "Check_Refresh_Address_Invalid_Ephemeral_Key_Length",
        "Check_Refresh_Address_Service_Unavailable",
        "Check_Refresh_Address_Server_Error",
        "Check_Refresh_Address_Client_Error",
        "Check_Refresh_Address_Service_Timeout",
        "Check_Refresh_Address_Proxy_Error",
        "Check_Refresh_Address_Fatal_Tls_Error_After_Reply",
        "Check_Refresh_Address_Unknown_Network_Error",
        "Check_Refresh_Address_Invalid_Http_Response",
        "Check_Refresh_Address_Empty",
        "Check_Refresh_Address_Invalid_Url",
        "Check_Refresh_Address_No_Https_Scheme",
        "Check_Refresh_Address_Fetch_Certificate_Error",
        "Check_Refresh_Address_Unsupported_Certificate",
        "Check_Refresh_Address_Hash_Missing_In_Certificate",
        "Browser_Send_Failed",
        "Generic_Provider_Communication_Network_Error",
        "Generic_Provider_Communication_Invalid_Ephemeral_Key_Length",
        "Generic_Provider_Communication_Certificate_Error",
        "Generic_Provider_Communication_Tls_Error",
        "Generic_Provider_Communication_ServiceUnavailable",
        "Generic_Provider_Communication_Server_Error",
        "Generic_Provider_Communication_Client_Error",
        "Get_SelfAuthData_Invalid_Or_Empty",
        "Change_Pin_No_SetEidPinCommand_Response",
        "Change_Pin_Input_Timeout",
        "Change_Pin_User_Cancelled",
        "Change_Pin_New_Pin_Mismatch",
        "Change_Pin_New_Pin_Invalid_Length",
        "Change_Pin_Unexpected_Transmit_Status",
        "Change_Pin_Card_User_Cancelled",
        "Change_Pin_Unrecoverable",
        "Start_Ifd_Service_Failed",
        "Prepare_Pace_Ifd_Unknown",
        "Establish_Pace_Ifd_Unknown",
        "Enter_Pace_Password_Ifd_User_Cancelled",
        "Enter_New_Pace_Pin_Ifd_User_Cancelled",
        "Info",
        "State_Name",
        "Card_Return_Code",
        "Network_Error",
        "Http_Status_Code",
        "Certificate_Status",
        "Ssl_Errors",
        "Paos_Type",
        "Ephemeral_Server_Key_Algorithm",
        "Ephemeral_Server_Key_Length"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Reason'
        QtMocHelpers::EnumData<enum Reason>(1, 1, QMC::EnumIsScoped).add({
            {    2, Reason::User_Cancelled },
            {    3, Reason::Card_Removed },
            {    4, Reason::Parse_TcToken_Invalid_Url },
            {    5, Reason::Parse_TcToken_Missing_Url },
            {    6, Reason::Get_TcToken_Invalid_Url },
            {    7, Reason::Get_TcToken_Invalid_Redirect_Url },
            {    8, Reason::Get_TcToken_Invalid_Certificate_Key_Length },
            {    9, Reason::Get_TcToken_Invalid_Ephemeral_Key_Length },
            {   10, Reason::Get_TcToken_Invalid_Server_Reply },
            {   11, Reason::Get_TcToken_ServiceUnavailable },
            {   12, Reason::Get_TcToken_Server_Error },
            {   13, Reason::Get_TcToken_Client_Error },
            {   14, Reason::Get_TcToken_Empty_Data },
            {   15, Reason::Get_TcToken_Invalid_Data },
            {   16, Reason::Get_TcToken_Network_Error },
            {   17, Reason::Certificate_Check_Failed_No_Description },
            {   18, Reason::Certificate_Check_Failed_No_SubjectUrl_In_Description },
            {   19, Reason::Certificate_Check_Failed_Hash_Mismatch },
            {   20, Reason::Certificate_Check_Failed_Same_Origin_Policy_Violation },
            {   21, Reason::Certificate_Check_Failed_Hash_Missing_In_Description },
            {   22, Reason::Pre_Verification_No_Test_Environment },
            {   23, Reason::Pre_Verification_Invalid_Certificate_Chain },
            {   24, Reason::Pre_Verification_Invalid_Certificate_Signature },
            {   25, Reason::Pre_Verification_Certificate_Expired },
            {   26, Reason::Extract_Cvcs_From_Eac1_No_Unique_At },
            {   27, Reason::Extract_Cvcs_From_Eac1_No_Unique_Dv },
            {   28, Reason::Extract_Cvcs_From_Eac1_At_Missing },
            {   29, Reason::Extract_Cvcs_From_Eac1_Dv_Missing },
            {   30, Reason::Connect_Card_Connection_Failed },
            {   31, Reason::Connect_Card_Eid_Inactive },
            {   32, Reason::Establish_Pace_Channel_Basic_Reader_No_Pin },
            {   33, Reason::Establish_Pace_Channel_Puk_Inoperative },
            {   34, Reason::Establish_Pace_Channel_Unknown_Password_Id },
            {   35, Reason::Establish_Pace_Channel_User_Cancelled },
            {   36, Reason::Maintain_Card_Connection_Pace_Unrecoverable },
            {   37, Reason::Did_Authenticate_Eac1_Card_Command_Failed },
            {   38, Reason::Process_Certificates_From_Eac2_Cvc_Chain_Missing },
            {   39, Reason::Did_Authenticate_Eac2_Invalid_Cvc_Chain },
            {   40, Reason::Did_Authenticate_Eac2_Card_Command_Failed },
            {   41, Reason::Generic_Send_Receive_Paos_Unhandled },
            {   42, Reason::Generic_Send_Receive_Network_Error },
            {   43, Reason::Generic_Send_Receive_Tls_Error },
            {   44, Reason::Generic_Send_Receive_Server_Error },
            {   45, Reason::Generic_Send_Receive_Service_Unavailable },
            {   46, Reason::Generic_Send_Receive_Client_Error },
            {   47, Reason::Generic_Send_Receive_Paos_Unknown },
            {   48, Reason::Generic_Send_Receive_Paos_Unexpected },
            {   49, Reason::Generic_Send_Receive_Invalid_Ephemeral_Key_Length },
            {   50, Reason::Generic_Send_Receive_Certificate_Error },
            {   51, Reason::Generic_Send_Receive_Session_Resumption_Failed },
            {   52, Reason::Transmit_Card_Command_Failed },
            {   53, Reason::Start_Paos_Response_Missing },
            {   54, Reason::Start_Paos_Response_Error },
            {   55, Reason::Check_Refresh_Address_Fatal_Tls_Error_Before_Reply },
            {   56, Reason::Check_Refresh_Address_Invalid_Ephemeral_Key_Length },
            {   57, Reason::Check_Refresh_Address_Service_Unavailable },
            {   58, Reason::Check_Refresh_Address_Server_Error },
            {   59, Reason::Check_Refresh_Address_Client_Error },
            {   60, Reason::Check_Refresh_Address_Service_Timeout },
            {   61, Reason::Check_Refresh_Address_Proxy_Error },
            {   62, Reason::Check_Refresh_Address_Fatal_Tls_Error_After_Reply },
            {   63, Reason::Check_Refresh_Address_Unknown_Network_Error },
            {   64, Reason::Check_Refresh_Address_Invalid_Http_Response },
            {   65, Reason::Check_Refresh_Address_Empty },
            {   66, Reason::Check_Refresh_Address_Invalid_Url },
            {   67, Reason::Check_Refresh_Address_No_Https_Scheme },
            {   68, Reason::Check_Refresh_Address_Fetch_Certificate_Error },
            {   69, Reason::Check_Refresh_Address_Unsupported_Certificate },
            {   70, Reason::Check_Refresh_Address_Hash_Missing_In_Certificate },
            {   71, Reason::Browser_Send_Failed },
            {   72, Reason::Generic_Provider_Communication_Network_Error },
            {   73, Reason::Generic_Provider_Communication_Invalid_Ephemeral_Key_Length },
            {   74, Reason::Generic_Provider_Communication_Certificate_Error },
            {   75, Reason::Generic_Provider_Communication_Tls_Error },
            {   76, Reason::Generic_Provider_Communication_ServiceUnavailable },
            {   77, Reason::Generic_Provider_Communication_Server_Error },
            {   78, Reason::Generic_Provider_Communication_Client_Error },
            {   79, Reason::Get_SelfAuthData_Invalid_Or_Empty },
            {   80, Reason::Change_Pin_No_SetEidPinCommand_Response },
            {   81, Reason::Change_Pin_Input_Timeout },
            {   82, Reason::Change_Pin_User_Cancelled },
            {   83, Reason::Change_Pin_New_Pin_Mismatch },
            {   84, Reason::Change_Pin_New_Pin_Invalid_Length },
            {   85, Reason::Change_Pin_Unexpected_Transmit_Status },
            {   86, Reason::Change_Pin_Card_User_Cancelled },
            {   87, Reason::Change_Pin_Unrecoverable },
            {   88, Reason::Start_Ifd_Service_Failed },
            {   89, Reason::Prepare_Pace_Ifd_Unknown },
            {   90, Reason::Establish_Pace_Ifd_Unknown },
            {   91, Reason::Enter_Pace_Password_Ifd_User_Cancelled },
            {   92, Reason::Enter_New_Pace_Pin_Ifd_User_Cancelled },
        }),
        // enum 'Info'
        QtMocHelpers::EnumData<enum Info>(93, 93, QMC::EnumIsScoped).add({
            {   94, Info::State_Name },
            {   95, Info::Card_Return_Code },
            {   96, Info::Network_Error },
            {   97, Info::Http_Status_Code },
            {   98, Info::Certificate_Status },
            {   99, Info::Ssl_Errors },
            {  100, Info::Paos_Type },
            {  101, Info::Ephemeral_Server_Key_Algorithm },
            {  102, Info::Ephemeral_Server_Key_Length },
        }),
    };
    return QtMocHelpers::metaObjectData<FailureCode, qt_meta_tag_ZN10governikus11FailureCodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::FailureCode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11FailureCodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus11FailureCodeE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus11FailureCodeE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
