/****************************************************************************
** Meta object code from reading C++ file 'SmartCardDefinitions.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/SmartCardDefinitions.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SmartCardDefinitions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus12EnumCardTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumCardType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumCardTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumCardType",
        "CardType",
        "NONE",
        "UNKNOWN",
        "EID_CARD"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CardType'
        QtMocHelpers::EnumData<CardType>(1, 1, QMC::EnumIsScoped).add({
            {    2, CardType::NONE },
            {    3, CardType::UNKNOWN },
            {    4, CardType::EID_CARD },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus12EnumCardTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus12EnumCardTypeE =
    governikus::EnumCardType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumCardTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus12EnumCardTypeE =
    qt_staticMetaObjectContent_ZN10governikus12EnumCardTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumCardTypeE =
    qt_staticMetaObjectContent_ZN10governikus12EnumCardTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumCardType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumCardTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumCardTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumCardTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus18EnumPacePasswordIdE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumPacePasswordId::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumPacePasswordIdE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumPacePasswordId",
        "PacePasswordId",
        "UNKNOWN",
        "PACE_MRZ",
        "PACE_CAN",
        "PACE_PIN",
        "PACE_PUK"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PacePasswordId'
        QtMocHelpers::EnumData<PacePasswordId>(1, 1, QMC::EnumIsScoped).add({
            {    2, PacePasswordId::UNKNOWN },
            {    3, PacePasswordId::PACE_MRZ },
            {    4, PacePasswordId::PACE_CAN },
            {    5, PacePasswordId::PACE_PIN },
            {    6, PacePasswordId::PACE_PUK },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus18EnumPacePasswordIdE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus18EnumPacePasswordIdE =
    governikus::EnumPacePasswordId::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18EnumPacePasswordIdE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus18EnumPacePasswordIdE =
    qt_staticMetaObjectContent_ZN10governikus18EnumPacePasswordIdE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumPacePasswordIdE =
    qt_staticMetaObjectContent_ZN10governikus18EnumPacePasswordIdE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumPacePasswordId::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumPacePasswordIdE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus18EnumPacePasswordIdE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus18EnumPacePasswordIdE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus19EnumAcceptedEidTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumAcceptedEidType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus19EnumAcceptedEidTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumAcceptedEidType",
        "AcceptedEidType",
        "CARD_CERTIFIED",
        "SE_CERTIFIED",
        "SE_ENDORSED",
        "HW_KEYSTORE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AcceptedEidType'
        QtMocHelpers::EnumData<AcceptedEidType>(1, 1, QMC::EnumIsScoped).add({
            {    2, AcceptedEidType::CARD_CERTIFIED },
            {    3, AcceptedEidType::SE_CERTIFIED },
            {    4, AcceptedEidType::SE_ENDORSED },
            {    5, AcceptedEidType::HW_KEYSTORE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus19EnumAcceptedEidTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus19EnumAcceptedEidTypeE =
    governikus::EnumAcceptedEidType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus19EnumAcceptedEidTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus19EnumAcceptedEidTypeE =
    qt_staticMetaObjectContent_ZN10governikus19EnumAcceptedEidTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus19EnumAcceptedEidTypeE =
    qt_staticMetaObjectContent_ZN10governikus19EnumAcceptedEidTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumAcceptedEidType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus19EnumAcceptedEidTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus19EnumAcceptedEidTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus19EnumAcceptedEidTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus17EnumMobileEidTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumMobileEidType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus17EnumMobileEidTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumMobileEidType",
        "MobileEidType",
        "UNKNOWN",
        "SE_CERTIFIED",
        "SE_ENDORSED",
        "HW_KEYSTORE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MobileEidType'
        QtMocHelpers::EnumData<MobileEidType>(1, 1, QMC::EnumIsScoped).add({
            {    2, MobileEidType::UNKNOWN },
            {    3, MobileEidType::SE_CERTIFIED },
            {    4, MobileEidType::SE_ENDORSED },
            {    5, MobileEidType::HW_KEYSTORE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus17EnumMobileEidTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus17EnumMobileEidTypeE =
    governikus::EnumMobileEidType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus17EnumMobileEidTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus17EnumMobileEidTypeE =
    qt_staticMetaObjectContent_ZN10governikus17EnumMobileEidTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus17EnumMobileEidTypeE =
    qt_staticMetaObjectContent_ZN10governikus17EnumMobileEidTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumMobileEidType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus17EnumMobileEidTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus17EnumMobileEidTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus17EnumMobileEidTypeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
