/****************************************************************************
** Meta object code from reading C++ file 'UiPlugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/base/UiPlugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UiPlugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus8UiPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::UiPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus8UiPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::UiPlugin",
        "fireWorkflowRequested",
        "",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "fireShowUiRequested",
        "UiModule",
        "pModule",
        "fireShowUserInformationRequested",
        "pMsg",
        "fireQuitApplicationRequest",
        "pExitCode",
        "fireCloseReminderFinished",
        "pDontRemindAgain",
        "fireApplicationActivated",
        "fireSystemSettingsChanged",
        "fireUiDominationRequest",
        "const UiPlugin*",
        "pUi",
        "pInformation",
        "fireUiDominationRelease",
        "fireRestartApplicationRequested",
        "doShutdown",
        "onWorkflowStarted",
        "onWorkflowFinished",
        "onWorkflowUnhandled",
        "onApplicationInitialized",
        "onApplicationStarted",
        "onShowUi",
        "onHideUi",
        "onTranslationChanged",
        "onProxyAuthenticationRequired",
        "QNetworkProxy",
        "pProxy",
        "QAuthenticator*",
        "pAuthenticator",
        "onUiDomination",
        "pAccepted",
        "onUiDominationReleased",
        "onShowUserInformation",
        "pInformationMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireWorkflowRequested'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireShowUiRequested'
        QtMocHelpers::SignalData<void(UiModule)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'fireShowUserInformationRequested'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'fireQuitApplicationRequest'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'fireQuitApplicationRequest'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Signal 'fireCloseReminderFinished'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'fireApplicationActivated'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSystemSettingsChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireUiDominationRequest'
        QtMocHelpers::SignalData<void(const UiPlugin *, const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 },
        }}),
        // Signal 'fireUiDominationRelease'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRestartApplicationRequested'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onWorkflowStarted'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onWorkflowFinished'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onWorkflowUnhandled'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onApplicationInitialized'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onApplicationStarted'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onShowUi'
        QtMocHelpers::SlotData<void(UiModule)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'onHideUi'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onProxyAuthenticationRequired'
        QtMocHelpers::SlotData<void(const QNetworkProxy &, QAuthenticator *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 }, { 0x80000000 | 34, 35 },
        }}),
        // Slot 'onUiDomination'
        QtMocHelpers::SlotData<void(const UiPlugin *, const QString &, bool)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 37 },
        }}),
        // Slot 'onUiDominationReleased'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onShowUserInformation'
        QtMocHelpers::SlotData<void(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 40 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UiPlugin, qt_meta_tag_ZN10governikus8UiPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::UiPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus8UiPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus8UiPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus8UiPluginE_t>.metaTypes,
    nullptr
} };

void governikus::UiPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UiPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireWorkflowRequested((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 1: _t->fireShowUiRequested((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 2: _t->fireShowUserInformationRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fireQuitApplicationRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->fireQuitApplicationRequest(); break;
        case 5: _t->fireCloseReminderFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fireApplicationActivated(); break;
        case 7: _t->fireSystemSettingsChanged(); break;
        case 8: _t->fireUiDominationRequest((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->fireUiDominationRelease(); break;
        case 10: _t->fireRestartApplicationRequested(); break;
        case 11: _t->doShutdown(); break;
        case 12: _t->onWorkflowStarted((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 13: _t->onWorkflowFinished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 14: _t->onWorkflowUnhandled((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 15: _t->onApplicationInitialized(); break;
        case 16: _t->onApplicationStarted(); break;
        case 17: _t->onShowUi((*reinterpret_cast<std::add_pointer_t<UiModule>>(_a[1]))); break;
        case 18: _t->onHideUi(); break;
        case 19: _t->onTranslationChanged(); break;
        case 20: _t->onProxyAuthenticationRequired((*reinterpret_cast<std::add_pointer_t<QNetworkProxy>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAuthenticator*>>(_a[2]))); break;
        case 21: _t->onUiDomination((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 22: _t->onUiDominationReleased(); break;
        case 23: _t->onShowUserInformation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkProxy >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &UiPlugin::fireWorkflowRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(UiModule )>(_a, &UiPlugin::fireShowUiRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(const QString & )>(_a, &UiPlugin::fireShowUserInformationRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(int )>(_a, &UiPlugin::fireQuitApplicationRequest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(bool )>(_a, &UiPlugin::fireCloseReminderFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)()>(_a, &UiPlugin::fireApplicationActivated, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)()>(_a, &UiPlugin::fireSystemSettingsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)(const UiPlugin * , const QString & )>(_a, &UiPlugin::fireUiDominationRequest, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)()>(_a, &UiPlugin::fireUiDominationRelease, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (UiPlugin::*)()>(_a, &UiPlugin::fireRestartApplicationRequested, 10))
            return;
    }
}

const QMetaObject *governikus::UiPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::UiPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus8UiPluginE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::UiPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void governikus::UiPlugin::fireWorkflowRequested(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::UiPlugin::fireShowUiRequested(UiModule _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::UiPlugin::fireShowUserInformationRequested(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void governikus::UiPlugin::fireQuitApplicationRequest(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 5
void governikus::UiPlugin::fireCloseReminderFinished(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void governikus::UiPlugin::fireApplicationActivated()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::UiPlugin::fireSystemSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::UiPlugin::fireUiDominationRequest(const UiPlugin * _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void governikus::UiPlugin::fireUiDominationRelease()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void governikus::UiPlugin::fireRestartApplicationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
namespace {
struct qt_meta_tag_ZN10governikus12EnumUiModuleE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumUiModule::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumUiModuleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumUiModule",
        "QML.Element",
        "UiModule",
        "CURRENT",
        "DEFAULT",
        "IDENTIFY",
        "SETTINGS",
        "PINMANAGEMENT",
        "HELP",
        "SELF_AUTHENTICATION",
        "ONBOARDING",
        "UPDATEINFORMATION",
        "REMOTE_SERVICE",
        "CHECK_ID_CARD"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'UiModule'
        QtMocHelpers::EnumData<UiModule>(2, 2, QMC::EnumIsScoped).add({
            {    3, UiModule::CURRENT },
            {    4, UiModule::DEFAULT },
            {    5, UiModule::IDENTIFY },
            {    6, UiModule::SETTINGS },
            {    7, UiModule::PINMANAGEMENT },
            {    8, UiModule::HELP },
            {    9, UiModule::SELF_AUTHENTICATION },
            {   10, UiModule::ONBOARDING },
            {   11, UiModule::UPDATEINFORMATION },
            {   12, UiModule::REMOTE_SERVICE },
            {   13, UiModule::CHECK_ID_CARD },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus12EnumUiModuleE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus12EnumUiModuleE =
    governikus::EnumUiModule::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumUiModuleE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus12EnumUiModuleE =
    qt_staticMetaObjectContent_ZN10governikus12EnumUiModuleE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumUiModuleE =
    qt_staticMetaObjectContent_ZN10governikus12EnumUiModuleE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumUiModule::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumUiModuleE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumUiModuleE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumUiModuleE.metaTypes,
    nullptr
} };

QT_WARNING_POP
