/****************************************************************************
** Meta object code from reading C++ file 'RemoteServiceModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/RemoteServiceModel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'RemoteServiceModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus18RemoteServiceModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::RemoteServiceModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus18RemoteServiceModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::RemoteServiceModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireStartWorkflow",
        "",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "fireIsStartingChanged",
        "fireIsRunningChanged",
        "fireEnvironmentChanged",
        "firePskChanged",
        "pPsk",
        "fireDisplayTextChanged",
        "fireConnectedChanged",
        "fireConnectedServerDisconnected",
        "fireServerPskChanged",
        "firePairingFailed",
        "pDeviceName",
        "pErrorMessage",
        "firePairingSuccess",
        "firePairingCompleted",
        "fireConnectionInfoChanged",
        "fireConnectedServerDeviceNamesChanged",
        "fireRemoteReaderVisibleChanged",
        "fireEstablishPaceChannelUpdated",
        "fireCertificateRemoved",
        "fireTransactionInfoChanged",
        "onEstablishConnectionDone",
        "QSharedPointer<IfdListEntry>",
        "pEntry",
        "GlobalStatus",
        "pStatus",
        "onConnectionInfoChanged",
        "pConnected",
        "onCardConnected",
        "QSharedPointer<CardConnection>",
        "pConnection",
        "onCardDisconnected",
        "onConnectedDevicesChanged",
        "onEnvironmentChanged",
        "onApplicationStateChanged",
        "pIsAppInForeground",
        "onPairingCompleted",
        "QSslCertificate",
        "pCertificate",
        "onNameChanged",
        "onTranslationChanged",
        "onReaderPluginTypesChanged",
        "pExplicitStart",
        "setRunning",
        "pState",
        "pEnablePairing",
        "startDetection",
        "stopDetection",
        "pStopScan",
        "rememberServer",
        "QVariant",
        "pDeviceId",
        "connectToRememberedServer",
        "pServerPsk",
        "pinPadModeOn",
        "forgetDevice",
        "pId",
        "cancelPasswordRequest",
        "passwordsDiffer",
        "changePinLength",
        "isPinAuthentication",
        "running",
        "isStarting",
        "runnable",
        "canEnableNfc",
        "errorMessage",
        "isPairing",
        "psk",
        "displayText",
        "percentage",
        "connectedToPairedDevice",
        "connectionInfo",
        "connectedServerDeviceNames",
        "allDevices",
        "governikus::RemoteDeviceModel*",
        "availableDevicesInPairingMode",
        "governikus::RemoteDeviceFilterModel*",
        "availablePairedDevices",
        "unavailablePairedDevices",
        "enableTransportPinLink",
        "remoteReaderVisible",
        "requiresLocalNetworkPermission",
        "transactionInfo",
        "connectedClientName"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireStartWorkflow'
        QtMocHelpers::SignalData<void(const QSharedPointer<WorkflowRequest> &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'fireIsStartingChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireIsRunningChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireEnvironmentChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePskChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 },
        }}),
        // Signal 'fireDisplayTextChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireConnectedChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireConnectedServerDisconnected'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireServerPskChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'firePairingFailed'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 }, { QMetaType::QString, 20 },
        }}),
        // Signal 'firePairingSuccess'
        QtMocHelpers::SignalData<void(const QString &)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'firePairingCompleted'
        QtMocHelpers::SignalData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireConnectionInfoChanged'
        QtMocHelpers::SignalData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireConnectedServerDeviceNamesChanged'
        QtMocHelpers::SignalData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireRemoteReaderVisibleChanged'
        QtMocHelpers::SignalData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireEstablishPaceChannelUpdated'
        QtMocHelpers::SignalData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireCertificateRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(27, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'fireTransactionInfoChanged'
        QtMocHelpers::SignalData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onEstablishConnectionDone'
        QtMocHelpers::SlotData<void(const QSharedPointer<IfdListEntry> &, const GlobalStatus &)>(29, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { 0x80000000 | 32, 33 },
        }}),
        // Slot 'onConnectionInfoChanged'
        QtMocHelpers::SlotData<void(bool)>(34, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'onCardConnected'
        QtMocHelpers::SlotData<void(const QSharedPointer<CardConnection> &) const>(36, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'onCardDisconnected'
        QtMocHelpers::SlotData<void(const QSharedPointer<CardConnection> &) const>(39, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'onConnectedDevicesChanged'
        QtMocHelpers::SlotData<void()>(40, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onEnvironmentChanged'
        QtMocHelpers::SlotData<void()>(41, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onApplicationStateChanged'
        QtMocHelpers::SlotData<void(const bool) const>(42, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'onPairingCompleted'
        QtMocHelpers::SlotData<void(const QSslCertificate &)>(44, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'onNameChanged'
        QtMocHelpers::SlotData<void()>(47, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(48, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onReaderPluginTypesChanged'
        QtMocHelpers::SlotData<void(bool)>(49, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 50 },
        }}),
        // Method 'setRunning'
        QtMocHelpers::MethodData<void(bool, bool)>(51, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 52 }, { QMetaType::Bool, 53 },
        }}),
        // Method 'setRunning'
        QtMocHelpers::MethodData<void(bool)>(51, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 52 },
        }}),
        // Method 'startDetection'
        QtMocHelpers::MethodData<void()>(54, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'stopDetection'
        QtMocHelpers::MethodData<void(bool)>(55, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Method 'rememberServer'
        QtMocHelpers::MethodData<bool(const QVariant &)>(57, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 58, 59 },
        }}),
        // Method 'connectToRememberedServer'
        QtMocHelpers::MethodData<void(const QByteArray &)>(60, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 61 },
        }}),
        // Method 'pinPadModeOn'
        QtMocHelpers::MethodData<bool() const>(62, 6, QMC::AccessPublic, QMetaType::Bool),
        // Method 'forgetDevice'
        QtMocHelpers::MethodData<void(const QVariant &)>(63, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 58, 64 },
        }}),
        // Method 'cancelPasswordRequest'
        QtMocHelpers::MethodData<void()>(65, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'passwordsDiffer'
        QtMocHelpers::MethodData<void()>(66, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'changePinLength'
        QtMocHelpers::MethodData<void()>(67, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'isPinAuthentication'
        QtMocHelpers::MethodData<bool() const>(68, 6, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'running'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'isStarting'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'runnable'
        QtMocHelpers::PropertyData<bool>(71, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'canEnableNfc'
        QtMocHelpers::PropertyData<bool>(72, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'errorMessage'
        QtMocHelpers::PropertyData<QString>(73, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'isPairing'
        QtMocHelpers::PropertyData<bool>(74, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'psk'
        QtMocHelpers::PropertyData<QByteArray>(75, QMetaType::QByteArray, QMC::DefaultPropertyFlags, 4),
        // property 'displayText'
        QtMocHelpers::PropertyData<QString>(76, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'percentage'
        QtMocHelpers::PropertyData<int>(77, QMetaType::Int, QMC::DefaultPropertyFlags, 5),
        // property 'connectedToPairedDevice'
        QtMocHelpers::PropertyData<bool>(78, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'connectionInfo'
        QtMocHelpers::PropertyData<QString>(79, QMetaType::QString, QMC::DefaultPropertyFlags, 12),
        // property 'connectedServerDeviceNames'
        QtMocHelpers::PropertyData<QString>(80, QMetaType::QString, QMC::DefaultPropertyFlags, 13),
        // property 'allDevices'
        QtMocHelpers::PropertyData<governikus::RemoteDeviceModel*>(81, 0x80000000 | 82, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'availableDevicesInPairingMode'
        QtMocHelpers::PropertyData<governikus::RemoteDeviceFilterModel*>(83, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'availablePairedDevices'
        QtMocHelpers::PropertyData<governikus::RemoteDeviceFilterModel*>(85, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'unavailablePairedDevices'
        QtMocHelpers::PropertyData<governikus::RemoteDeviceFilterModel*>(86, 0x80000000 | 84, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'enableTransportPinLink'
        QtMocHelpers::PropertyData<bool>(87, QMetaType::Bool, QMC::DefaultPropertyFlags, 15),
        // property 'remoteReaderVisible'
        QtMocHelpers::PropertyData<bool>(88, QMetaType::Bool, QMC::DefaultPropertyFlags, 14),
        // property 'requiresLocalNetworkPermission'
        QtMocHelpers::PropertyData<bool>(89, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'transactionInfo'
        QtMocHelpers::PropertyData<QString>(90, QMetaType::QString, QMC::DefaultPropertyFlags, 17),
        // property 'connectedClientName'
        QtMocHelpers::PropertyData<QString>(91, QMetaType::QString, QMC::DefaultPropertyFlags, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<RemoteServiceModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::RemoteServiceModel::staticMetaObject = { {
    QMetaObject::SuperData::link<WorkflowModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18RemoteServiceModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18RemoteServiceModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus18RemoteServiceModelE_t>.metaTypes,
    nullptr
} };

void governikus::RemoteServiceModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RemoteServiceModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireStartWorkflow((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 1: _t->fireIsStartingChanged(); break;
        case 2: _t->fireIsRunningChanged(); break;
        case 3: _t->fireEnvironmentChanged(); break;
        case 4: _t->firePskChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 5: _t->fireDisplayTextChanged(); break;
        case 6: _t->fireConnectedChanged(); break;
        case 7: _t->fireConnectedServerDisconnected(); break;
        case 8: _t->fireServerPskChanged(); break;
        case 9: _t->firePairingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 10: _t->firePairingSuccess((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->firePairingCompleted(); break;
        case 12: _t->fireConnectionInfoChanged(); break;
        case 13: _t->fireConnectedServerDeviceNamesChanged(); break;
        case 14: _t->fireRemoteReaderVisibleChanged(); break;
        case 15: _t->fireEstablishPaceChannelUpdated(); break;
        case 16: _t->fireCertificateRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->fireTransactionInfoChanged(); break;
        case 18: _t->onEstablishConnectionDone((*reinterpret_cast<std::add_pointer_t<QSharedPointer<IfdListEntry>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GlobalStatus>>(_a[2]))); break;
        case 19: _t->onConnectionInfoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->onCardConnected((*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[1]))); break;
        case 21: _t->onCardDisconnected((*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[1]))); break;
        case 22: _t->onConnectedDevicesChanged(); break;
        case 23: _t->onEnvironmentChanged(); break;
        case 24: _t->onApplicationStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->onPairingCompleted((*reinterpret_cast<std::add_pointer_t<QSslCertificate>>(_a[1]))); break;
        case 26: _t->onNameChanged(); break;
        case 27: _t->onTranslationChanged(); break;
        case 28: _t->onReaderPluginTypesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->setRunning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 30: _t->setRunning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->startDetection(); break;
        case 32: _t->stopDetection((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: { bool _r = _t->rememberServer((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->connectToRememberedServer((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 35: { bool _r = _t->pinPadModeOn();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->forgetDevice((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 37: _t->cancelPasswordRequest(); break;
        case 38: _t->passwordsDiffer(); break;
        case 39: _t->changePinLength(); break;
        case 40: { bool _r = _t->isPinAuthentication();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslCertificate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)(const QSharedPointer<WorkflowRequest> & )>(_a, &RemoteServiceModel::fireStartWorkflow, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireIsStartingChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireIsRunningChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireEnvironmentChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)(const QByteArray & )>(_a, &RemoteServiceModel::firePskChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireDisplayTextChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireConnectedChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireConnectedServerDisconnected, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireServerPskChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)(const QString & , const QString & )>(_a, &RemoteServiceModel::firePairingFailed, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)(const QString & )>(_a, &RemoteServiceModel::firePairingSuccess, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::firePairingCompleted, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireConnectionInfoChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireConnectedServerDeviceNamesChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireRemoteReaderVisibleChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireEstablishPaceChannelUpdated, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)(const QString & )>(_a, &RemoteServiceModel::fireCertificateRemoved, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (RemoteServiceModel::*)()>(_a, &RemoteServiceModel::fireTransactionInfoChanged, 17))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
        case 14:
        case 13:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< governikus::RemoteDeviceFilterModel* >(); break;
        case 12:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< governikus::RemoteDeviceModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isRunning(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isStarting(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isRunnable(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isCanEnableNfc(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getErrorMessage(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isPairing(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->getPsk(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getDisplayText(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->getPercentage(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isConnectedToPairedDevice(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->getConnectionInfo(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->getConnectedServerDeviceNames(); break;
        case 12: *reinterpret_cast<governikus::RemoteDeviceModel**>(_v) = _t->getAllDevices(); break;
        case 13: *reinterpret_cast<governikus::RemoteDeviceFilterModel**>(_v) = _t->getAvailableDevicesInPairingMode(); break;
        case 14: *reinterpret_cast<governikus::RemoteDeviceFilterModel**>(_v) = _t->getAvailablePairedDevices(); break;
        case 15: *reinterpret_cast<governikus::RemoteDeviceFilterModel**>(_v) = _t->getUnavailablePairedDevices(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->enableTransportPinLink(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->getRemoteReaderVisible(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->mRequiresLocalNetworkPermission; break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->getTransactionInfo(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->getConnectedClientName(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::RemoteServiceModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::RemoteServiceModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus18RemoteServiceModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<RemoteServiceModel>"))
        return static_cast< SingletonCreator<RemoteServiceModel>*>(this);
    return WorkflowModel::qt_metacast(_clname);
}

int governikus::RemoteServiceModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WorkflowModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 41)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void governikus::RemoteServiceModel::fireStartWorkflow(const QSharedPointer<WorkflowRequest> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::RemoteServiceModel::fireIsStartingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::RemoteServiceModel::fireIsRunningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::RemoteServiceModel::fireEnvironmentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::RemoteServiceModel::firePskChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void governikus::RemoteServiceModel::fireDisplayTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::RemoteServiceModel::fireConnectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::RemoteServiceModel::fireConnectedServerDisconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void governikus::RemoteServiceModel::fireServerPskChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void governikus::RemoteServiceModel::firePairingFailed(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void governikus::RemoteServiceModel::firePairingSuccess(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void governikus::RemoteServiceModel::firePairingCompleted()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void governikus::RemoteServiceModel::fireConnectionInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void governikus::RemoteServiceModel::fireConnectedServerDeviceNamesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void governikus::RemoteServiceModel::fireRemoteReaderVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void governikus::RemoteServiceModel::fireEstablishPaceChannelUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void governikus::RemoteServiceModel::fireCertificateRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void governikus::RemoteServiceModel::fireTransactionInfoChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
