/**
 * Copyright (c) 2022-2026 Governikus GmbH & Co. KG, Germany
 */

#include "Backup.h"


using namespace governikus;


void Backup::disable(const QSharedPointer<QSettings>& pSettings)
{
	Q_UNUSED(pSettings)
}
