/****************************************************************************
** Meta object code from reading C++ file 'PcscUtils.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/pcsc/PcscUtils.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PcscUtils.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus4pcscE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::pcsc::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus4pcscE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::pcsc",
        "PcscReturnCode",
        "Scard_S_Success",
        "Scard_F_Internal_Error",
        "Scard_E_Cancelled",
        "Scard_E_Invalid_Handle",
        "Scard_E_Invalid_Parameter",
        "Scard_E_Invalid_Target",
        "Scard_E_No_Memory",
        "Scard_F_Waited_Too_Long",
        "Scard_E_Insufficient_Buffer",
        "Scard_E_Unknown_Reader",
        "Scard_E_Timeout",
        "Scard_E_Sharing_Violation",
        "Scard_E_No_Smartcard",
        "Scard_E_Unknown_Card",
        "Scard_E_Cant_Dispose",
        "Scard_E_Proto_Mismatch",
        "Scard_E_Not_Ready",
        "Scard_E_Invalid_Value",
        "Scard_E_System_Cancelled",
        "Scard_F_Comm_Error",
        "Scard_F_Unknown_Error",
        "Scard_E_Invalid_Atr",
        "Scard_E_Not_Transacted",
        "Scard_E_Reader_Unavailable",
        "Scard_P_Shutdown",
        "Scard_E_Pci_Too_Small",
        "Scard_E_Reader_Unsupported",
        "Scard_E_Duplicate_Reader",
        "Scard_E_Card_Unsupported",
        "Scard_E_No_Service",
        "Scard_E_Service_Stopped",
        "Scard_E_Unsupported_Feature",
        "Scard_E_Unexpected",
        "Scard_E_Icc_Installation",
        "Scard_E_Icc_Createorder",
        "Scard_E_Dir_Not_Found",
        "Scard_E_File_Not_Found",
        "Scard_E_No_Dir",
        "Scard_E_No_File",
        "Scard_E_No_Access",
        "Scard_E_Write_Too_Many",
        "Scard_E_Bad_Seek",
        "Scard_E_Invalid_Chv",
        "Scard_E_Unknown_Res_Mng",
        "Scard_E_No_Such_Certificate",
        "Scard_E_Certificate_Unavailable",
        "Scard_E_No_Readers_Available",
        "Scard_E_Comm_Data_Lost",
        "Scard_E_No_Key_Container",
        "Scard_E_Server_Too_Busy",
        "Scard_W_Unsupported_Card",
        "Scard_W_Unresponsive_Card",
        "Scard_W_Unpowered_Card",
        "Scard_W_Reset_Card",
        "Scard_W_Removed_Card",
        "Scard_W_Security_Violation",
        "Scard_W_Wrong_Chv",
        "Scard_W_Chv_Blocked",
        "Scard_W_Eof",
        "Scard_W_Cancelled_By_User",
        "Scard_W_Card_Not_Authenticated"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PcscReturnCode'
        QtMocHelpers::EnumData<PcscReturnCode>(1, 1, QMC::EnumFlags{}).add({
            {    2, PcscReturnCode::Scard_S_Success },
            {    3, PcscReturnCode::Scard_F_Internal_Error },
            {    4, PcscReturnCode::Scard_E_Cancelled },
            {    5, PcscReturnCode::Scard_E_Invalid_Handle },
            {    6, PcscReturnCode::Scard_E_Invalid_Parameter },
            {    7, PcscReturnCode::Scard_E_Invalid_Target },
            {    8, PcscReturnCode::Scard_E_No_Memory },
            {    9, PcscReturnCode::Scard_F_Waited_Too_Long },
            {   10, PcscReturnCode::Scard_E_Insufficient_Buffer },
            {   11, PcscReturnCode::Scard_E_Unknown_Reader },
            {   12, PcscReturnCode::Scard_E_Timeout },
            {   13, PcscReturnCode::Scard_E_Sharing_Violation },
            {   14, PcscReturnCode::Scard_E_No_Smartcard },
            {   15, PcscReturnCode::Scard_E_Unknown_Card },
            {   16, PcscReturnCode::Scard_E_Cant_Dispose },
            {   17, PcscReturnCode::Scard_E_Proto_Mismatch },
            {   18, PcscReturnCode::Scard_E_Not_Ready },
            {   19, PcscReturnCode::Scard_E_Invalid_Value },
            {   20, PcscReturnCode::Scard_E_System_Cancelled },
            {   21, PcscReturnCode::Scard_F_Comm_Error },
            {   22, PcscReturnCode::Scard_F_Unknown_Error },
            {   23, PcscReturnCode::Scard_E_Invalid_Atr },
            {   24, PcscReturnCode::Scard_E_Not_Transacted },
            {   25, PcscReturnCode::Scard_E_Reader_Unavailable },
            {   26, PcscReturnCode::Scard_P_Shutdown },
            {   27, PcscReturnCode::Scard_E_Pci_Too_Small },
            {   28, PcscReturnCode::Scard_E_Reader_Unsupported },
            {   29, PcscReturnCode::Scard_E_Duplicate_Reader },
            {   30, PcscReturnCode::Scard_E_Card_Unsupported },
            {   31, PcscReturnCode::Scard_E_No_Service },
            {   32, PcscReturnCode::Scard_E_Service_Stopped },
            {   33, PcscReturnCode::Scard_E_Unsupported_Feature },
            {   34, PcscReturnCode::Scard_E_Unexpected },
            {   35, PcscReturnCode::Scard_E_Icc_Installation },
            {   36, PcscReturnCode::Scard_E_Icc_Createorder },
            {   37, PcscReturnCode::Scard_E_Dir_Not_Found },
            {   38, PcscReturnCode::Scard_E_File_Not_Found },
            {   39, PcscReturnCode::Scard_E_No_Dir },
            {   40, PcscReturnCode::Scard_E_No_File },
            {   41, PcscReturnCode::Scard_E_No_Access },
            {   42, PcscReturnCode::Scard_E_Write_Too_Many },
            {   43, PcscReturnCode::Scard_E_Bad_Seek },
            {   44, PcscReturnCode::Scard_E_Invalid_Chv },
            {   45, PcscReturnCode::Scard_E_Unknown_Res_Mng },
            {   46, PcscReturnCode::Scard_E_No_Such_Certificate },
            {   47, PcscReturnCode::Scard_E_Certificate_Unavailable },
            {   48, PcscReturnCode::Scard_E_No_Readers_Available },
            {   49, PcscReturnCode::Scard_E_Comm_Data_Lost },
            {   50, PcscReturnCode::Scard_E_No_Key_Container },
            {   51, PcscReturnCode::Scard_E_Server_Too_Busy },
            {   52, PcscReturnCode::Scard_W_Unsupported_Card },
            {   53, PcscReturnCode::Scard_W_Unresponsive_Card },
            {   54, PcscReturnCode::Scard_W_Unpowered_Card },
            {   55, PcscReturnCode::Scard_W_Reset_Card },
            {   56, PcscReturnCode::Scard_W_Removed_Card },
            {   57, PcscReturnCode::Scard_W_Security_Violation },
            {   58, PcscReturnCode::Scard_W_Wrong_Chv },
            {   59, PcscReturnCode::Scard_W_Chv_Blocked },
            {   60, PcscReturnCode::Scard_W_Eof },
            {   61, PcscReturnCode::Scard_W_Cancelled_By_User },
            {   62, PcscReturnCode::Scard_W_Card_Not_Authenticated },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus4pcscE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus4pcscE =
    governikus::pcsc::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus4pcscE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus4pcscE =
    qt_staticMetaObjectContent_ZN10governikus4pcscE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus4pcscE =
    qt_staticMetaObjectContent_ZN10governikus4pcscE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::pcsc::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus4pcscE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus4pcscE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus4pcscE.metaTypes,
    nullptr
} };

QT_WARNING_POP
