/****************************************************************************
** Meta object code from reading C++ file 'CertificateDescriptionModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/qml/CertificateDescriptionModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'CertificateDescriptionModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::CertificateDescriptionModel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::CertificateDescriptionModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "fireChanged",
        "",
        "onDidAuthenticateEac1Changed",
        "onTranslationChanged",
        "subjectName",
        "purpose"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDidAuthenticateEac1Changed'
        QtMocHelpers::SlotData<void()>(7, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTranslationChanged'
        QtMocHelpers::SlotData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'subjectName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'purpose'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<CertificateDescriptionModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject governikus::CertificateDescriptionModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t>.metaTypes,
    nullptr
} };

void governikus::CertificateDescriptionModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CertificateDescriptionModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireChanged(); break;
        case 1: _t->onDidAuthenticateEac1Changed(); break;
        case 2: _t->onTranslationChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CertificateDescriptionModel::*)()>(_a, &CertificateDescriptionModel::fireChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->getSubjectName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->getPurpose(); break;
        default: break;
        }
    }
}

const QMetaObject *governikus::CertificateDescriptionModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::CertificateDescriptionModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus27CertificateDescriptionModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SingletonCreator<CertificateDescriptionModel>"))
        return static_cast< SingletonCreator<CertificateDescriptionModel>*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int governikus::CertificateDescriptionModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void governikus::CertificateDescriptionModel::fireChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
