/****************************************************************************
** Meta object code from reading C++ file 'UiPluginWebSocket.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/websocket/UiPluginWebSocket.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UiPluginWebSocket.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::UiPluginWebSocket::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::UiPluginWebSocket",
        "doShutdown",
        "",
        "onWorkflowStarted",
        "QSharedPointer<WorkflowRequest>",
        "pRequest",
        "onWorkflowFinished",
        "onUiDomination",
        "const UiPlugin*",
        "pUi",
        "pInformation",
        "pAccepted",
        "onUiDominationReleased",
        "onNewWebSocketRequest",
        "QSharedPointer<HttpRequest>",
        "onNewConnection",
        "onClientDisconnected",
        "onTextMessageReceived",
        "pMessage",
        "onJsonMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'doShutdown'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onWorkflowStarted'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onWorkflowFinished'
        QtMocHelpers::SlotData<void(const QSharedPointer<WorkflowRequest> &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'onUiDomination'
        QtMocHelpers::SlotData<void(const UiPlugin *, const QString &, bool)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'onUiDominationReleased'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onNewWebSocketRequest'
        QtMocHelpers::SlotData<void(const QSharedPointer<HttpRequest> &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 5 },
        }}),
        // Slot 'onNewConnection'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onClientDisconnected'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTextMessageReceived'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'onJsonMessage'
        QtMocHelpers::SlotData<void(const QByteArray &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UiPluginWebSocket, qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::UiPluginWebSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<UiPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>.metaTypes,
    nullptr
} };

void governikus::UiPluginWebSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UiPluginWebSocket *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->doShutdown(); break;
        case 1: _t->onWorkflowStarted((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 2: _t->onWorkflowFinished((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WorkflowRequest>>>(_a[1]))); break;
        case 3: _t->onUiDomination((*reinterpret_cast<std::add_pointer_t<const UiPlugin*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 4: _t->onUiDominationReleased(); break;
        case 5: _t->onNewWebSocketRequest((*reinterpret_cast<std::add_pointer_t<QSharedPointer<HttpRequest>>>(_a[1]))); break;
        case 6: _t->onNewConnection(); break;
        case 7: _t->onClientDisconnected(); break;
        case 8: _t->onTextMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->onJsonMessage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<HttpRequest> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *governikus::UiPluginWebSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::UiPluginWebSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus17UiPluginWebSocketE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "governikus.UiPlugin"))
        return static_cast< governikus::UiPlugin*>(this);
    return UiPlugin::qt_metacast(_clname);
}

int governikus::UiPluginWebSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UiPlugin::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
using namespace governikus;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_UiPluginWebSocket[] = {
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x71,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'W',  'e',  'b',  'S',  'o',  'c', 
    'k',  'e',  't', 
    // "MetaData"
    0x04,  0xa1,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(governikus::UiPluginWebSocket, UiPluginWebSocket, qt_pluginMetaDataV2_UiPluginWebSocket)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_UiPluginWebSocket[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x73,  'g',  'o',  'v',  'e',  'r',  'n', 
    'i',  'k',  'u',  's',  '.',  'U',  'i',  'P', 
    'l',  'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x71,  'U',  'i',  'P',  'l',  'u',  'g', 
    'i',  'n',  'W',  'e',  'b',  'S',  'o',  'c', 
    'k',  'e',  't', 
    // "MetaData"
    0x04,  0xa1,  0x67,  'd',  'e',  'f',  'a',  'u', 
    'l',  't',  0xf5, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(governikus::UiPluginWebSocket, UiPluginWebSocket)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
