/****************************************************************************
** Meta object code from reading C++ file 'ReaderManagerPluginInfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/ReaderManagerPluginInfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ReaderManagerPluginInfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus27EnumReaderManagerPluginTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumReaderManagerPluginType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus27EnumReaderManagerPluginTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumReaderManagerPluginType",
        "QML.Element",
        "ReaderManagerPluginType",
        "UNKNOWN",
        "MOCK",
        "PCSC",
        "NFC",
        "REMOTE_IFD",
        "LOCAL_IFD",
        "SIMULATOR"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ReaderManagerPluginType'
        QtMocHelpers::EnumData<ReaderManagerPluginType>(2, 2, QMC::EnumIsScoped).add({
            {    3, ReaderManagerPluginType::UNKNOWN },
            {    4, ReaderManagerPluginType::MOCK },
            {    5, ReaderManagerPluginType::PCSC },
            {    6, ReaderManagerPluginType::NFC },
            {    7, ReaderManagerPluginType::REMOTE_IFD },
            {    8, ReaderManagerPluginType::LOCAL_IFD },
            {    9, ReaderManagerPluginType::SIMULATOR },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus27EnumReaderManagerPluginTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus27EnumReaderManagerPluginTypeE =
    governikus::EnumReaderManagerPluginType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus27EnumReaderManagerPluginTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus27EnumReaderManagerPluginTypeE =
    qt_staticMetaObjectContent_ZN10governikus27EnumReaderManagerPluginTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus27EnumReaderManagerPluginTypeE =
    qt_staticMetaObjectContent_ZN10governikus27EnumReaderManagerPluginTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumReaderManagerPluginType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus27EnumReaderManagerPluginTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus27EnumReaderManagerPluginTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus27EnumReaderManagerPluginTypeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
