/****************************************************************************
** Meta object code from reading C++ file 'MsgTypes.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ui/json/messages/MsgTypes.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MsgTypes.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus12EnumMsgLevelE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumMsgLevel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumMsgLevelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumMsgLevel",
        "MsgLevel",
        "v1",
        "v2",
        "v3"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MsgLevel'
        QtMocHelpers::EnumData<MsgLevel>(1, 1, QMC::EnumIsScoped).add({
            {    2, MsgLevel::v1 },
            {    3, MsgLevel::v2 },
            {    4, MsgLevel::v3 },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus12EnumMsgLevelE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus12EnumMsgLevelE =
    governikus::EnumMsgLevel::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus12EnumMsgLevelE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus12EnumMsgLevelE =
    qt_staticMetaObjectContent_ZN10governikus12EnumMsgLevelE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumMsgLevelE =
    qt_staticMetaObjectContent_ZN10governikus12EnumMsgLevelE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumMsgLevel::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumMsgLevelE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus12EnumMsgLevelE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus12EnumMsgLevelE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus11EnumMsgTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumMsgType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus11EnumMsgTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumMsgType",
        "MsgType",
        "VOID",
        "INVALID",
        "UNKNOWN_COMMAND",
        "INTERNAL_ERROR",
        "STATUS",
        "LOG",
        "INFO",
        "API_LEVEL",
        "READER",
        "READER_LIST",
        "BAD_STATE",
        "AUTH",
        "CHANGE_PIN",
        "CERTIFICATE",
        "ACCESS_RIGHTS",
        "INSERT_CARD",
        "ENTER_PIN",
        "ENTER_NEW_PIN",
        "ENTER_CAN",
        "ENTER_PUK",
        "PAUSE"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MsgType'
        QtMocHelpers::EnumData<MsgType>(1, 1, QMC::EnumIsScoped).add({
            {    2, MsgType::VOID },
            {    3, MsgType::INVALID },
            {    4, MsgType::UNKNOWN_COMMAND },
            {    5, MsgType::INTERNAL_ERROR },
            {    6, MsgType::STATUS },
            {    7, MsgType::LOG },
            {    8, MsgType::INFO },
            {    9, MsgType::API_LEVEL },
            {   10, MsgType::READER },
            {   11, MsgType::READER_LIST },
            {   12, MsgType::BAD_STATE },
            {   13, MsgType::AUTH },
            {   14, MsgType::CHANGE_PIN },
            {   15, MsgType::CERTIFICATE },
            {   16, MsgType::ACCESS_RIGHTS },
            {   17, MsgType::INSERT_CARD },
            {   18, MsgType::ENTER_PIN },
            {   19, MsgType::ENTER_NEW_PIN },
            {   20, MsgType::ENTER_CAN },
            {   21, MsgType::ENTER_PUK },
            {   22, MsgType::PAUSE },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus11EnumMsgTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus11EnumMsgTypeE =
    governikus::EnumMsgType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus11EnumMsgTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus11EnumMsgTypeE =
    qt_staticMetaObjectContent_ZN10governikus11EnumMsgTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus11EnumMsgTypeE =
    qt_staticMetaObjectContent_ZN10governikus11EnumMsgTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumMsgType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus11EnumMsgTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus11EnumMsgTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus11EnumMsgTypeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus14EnumMsgCmdTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumMsgCmdType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumMsgCmdTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumMsgCmdType",
        "MsgCmdType",
        "UNDEFINED",
        "ACCEPT",
        "CANCEL",
        "CONTINUE",
        "INTERRUPT",
        "GET_STATUS",
        "GET_LOG",
        "GET_INFO",
        "GET_API_LEVEL",
        "SET_API_LEVEL",
        "GET_READER",
        "GET_READER_LIST",
        "RUN_AUTH",
        "RUN_CHANGE_PIN",
        "GET_CERTIFICATE",
        "GET_ACCESS_RIGHTS",
        "SET_ACCESS_RIGHTS",
        "SET_CARD",
        "SET_PIN",
        "SET_NEW_PIN",
        "SET_CAN",
        "SET_PUK"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MsgCmdType'
        QtMocHelpers::EnumData<MsgCmdType>(1, 1, QMC::EnumIsScoped).add({
            {    2, MsgCmdType::UNDEFINED },
            {    3, MsgCmdType::ACCEPT },
            {    4, MsgCmdType::CANCEL },
            {    5, MsgCmdType::CONTINUE },
            {    6, MsgCmdType::INTERRUPT },
            {    7, MsgCmdType::GET_STATUS },
            {    8, MsgCmdType::GET_LOG },
            {    9, MsgCmdType::GET_INFO },
            {   10, MsgCmdType::GET_API_LEVEL },
            {   11, MsgCmdType::SET_API_LEVEL },
            {   12, MsgCmdType::GET_READER },
            {   13, MsgCmdType::GET_READER_LIST },
            {   14, MsgCmdType::RUN_AUTH },
            {   15, MsgCmdType::RUN_CHANGE_PIN },
            {   16, MsgCmdType::GET_CERTIFICATE },
            {   17, MsgCmdType::GET_ACCESS_RIGHTS },
            {   18, MsgCmdType::SET_ACCESS_RIGHTS },
            {   19, MsgCmdType::SET_CARD },
            {   20, MsgCmdType::SET_PIN },
            {   21, MsgCmdType::SET_NEW_PIN },
            {   22, MsgCmdType::SET_CAN },
            {   23, MsgCmdType::SET_PUK },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus14EnumMsgCmdTypeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus14EnumMsgCmdTypeE =
    governikus::EnumMsgCmdType::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumMsgCmdTypeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus14EnumMsgCmdTypeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumMsgCmdTypeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumMsgCmdTypeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumMsgCmdTypeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumMsgCmdType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumMsgCmdTypeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumMsgCmdTypeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumMsgCmdTypeE.metaTypes,
    nullptr
} };

QT_WARNING_POP
