/****************************************************************************
** Meta object code from reading C++ file 'ResponseApdu.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/card/base/apdu/ResponseApdu.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ResponseApdu.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus14EnumStatusCodeE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumStatusCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumStatusCodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumStatusCode",
        "StatusCode",
        "UNKNOWN",
        "SUCCESS",
        "NO_PKCS15_APP",
        "END_OF_FILE",
        "PIN_DEACTIVATED",
        "FCI_NO_ISO7816_4",
        "VERIFICATION_FAILED",
        "INPUT_TIMEOUT",
        "INPUT_CANCELLED",
        "PASSWORDS_DIFFER",
        "PASSWORD_OUTOF_RANGE",
        "CARD_EJECTED_AND_REINSERTED",
        "EEPROM_CELL_DEFECT",
        "SECURITY_ENVIRONMENT",
        "WRONG_LENGTH",
        "NO_BINARY_FILE",
        "LAST_CHAIN_CMD_EXPECTED",
        "ACCESS_DENIED",
        "PASSWORD_COUNTER_EXPIRED",
        "DIRECTORY_OR_PASSWORD_LOCKED_OR_NOT_ALLOWED",
        "NO_PARENT_FILE",
        "NOT_YET_INITIALIZED",
        "NO_CURRENT_DIRECTORY_SELECTED",
        "DATAFIELD_EXPECTED",
        "INVALID_SM_OBJECTS",
        "SW_APPLET_SELECT_FAILED",
        "COMMAND_NOT_ALLOWED",
        "INVALID_DATAFIELD",
        "ALGORITHM_ID",
        "FILE_NOT_FOUND",
        "RECORD_NOT_FOUND",
        "INVALID_PARAMETER",
        "LC_INCONSISTENT",
        "REFERENCED_DATA_NOT_FOUND",
        "ILLEGAL_OFFSET",
        "UNSUPPORTED_CLA",
        "CANT_DISPLAY",
        "INVALID_P1P2",
        "UNSUPPORTED_INS",
        "PIN_BLOCKED",
        "PIN_SUSPENDED",
        "PIN_RETRY_COUNT_2",
        "NO_PRECISE_DIAGNOSIS"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'StatusCode'
        QtMocHelpers::EnumData<StatusCode>(1, 1, QMC::EnumIsScoped).add({
            {    2, StatusCode::UNKNOWN },
            {    3, StatusCode::SUCCESS },
            {    4, StatusCode::NO_PKCS15_APP },
            {    5, StatusCode::END_OF_FILE },
            {    6, StatusCode::PIN_DEACTIVATED },
            {    7, StatusCode::FCI_NO_ISO7816_4 },
            {    8, StatusCode::VERIFICATION_FAILED },
            {    9, StatusCode::INPUT_TIMEOUT },
            {   10, StatusCode::INPUT_CANCELLED },
            {   11, StatusCode::PASSWORDS_DIFFER },
            {   12, StatusCode::PASSWORD_OUTOF_RANGE },
            {   13, StatusCode::CARD_EJECTED_AND_REINSERTED },
            {   14, StatusCode::EEPROM_CELL_DEFECT },
            {   15, StatusCode::SECURITY_ENVIRONMENT },
            {   16, StatusCode::WRONG_LENGTH },
            {   17, StatusCode::NO_BINARY_FILE },
            {   18, StatusCode::LAST_CHAIN_CMD_EXPECTED },
            {   19, StatusCode::ACCESS_DENIED },
            {   20, StatusCode::PASSWORD_COUNTER_EXPIRED },
            {   21, StatusCode::DIRECTORY_OR_PASSWORD_LOCKED_OR_NOT_ALLOWED },
            {   22, StatusCode::NO_PARENT_FILE },
            {   23, StatusCode::NOT_YET_INITIALIZED },
            {   24, StatusCode::NO_CURRENT_DIRECTORY_SELECTED },
            {   25, StatusCode::DATAFIELD_EXPECTED },
            {   26, StatusCode::INVALID_SM_OBJECTS },
            {   27, StatusCode::SW_APPLET_SELECT_FAILED },
            {   28, StatusCode::COMMAND_NOT_ALLOWED },
            {   29, StatusCode::INVALID_DATAFIELD },
            {   30, StatusCode::ALGORITHM_ID },
            {   31, StatusCode::FILE_NOT_FOUND },
            {   32, StatusCode::RECORD_NOT_FOUND },
            {   33, StatusCode::INVALID_PARAMETER },
            {   34, StatusCode::LC_INCONSISTENT },
            {   35, StatusCode::REFERENCED_DATA_NOT_FOUND },
            {   36, StatusCode::ILLEGAL_OFFSET },
            {   37, StatusCode::UNSUPPORTED_CLA },
            {   38, StatusCode::CANT_DISPLAY },
            {   39, StatusCode::INVALID_P1P2 },
            {   40, StatusCode::UNSUPPORTED_INS },
            {   41, StatusCode::PIN_BLOCKED },
            {   42, StatusCode::PIN_SUSPENDED },
            {   43, StatusCode::PIN_RETRY_COUNT_2 },
            {   44, StatusCode::NO_PRECISE_DIAGNOSIS },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus14EnumStatusCodeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus14EnumStatusCodeE =
    governikus::EnumStatusCode::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus14EnumStatusCodeE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus14EnumStatusCodeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumStatusCodeE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumStatusCodeE =
    qt_staticMetaObjectContent_ZN10governikus14EnumStatusCodeE.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumStatusCode::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumStatusCodeE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus14EnumStatusCodeE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus14EnumStatusCodeE.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN10governikus7EnumSW1E_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::EnumSW1::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus7EnumSW1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::EnumSW1",
        "SW1",
        "UNKNOWN",
        "MORE_DATA_AVAILABLE",
        "NONVOLATILE_MEMORY_UNCHANGED_1",
        "NONVOLATILE_MEMORY_CHANGED_1",
        "NONVOLATILE_MEMORY_UNCHANGED_2",
        "NONVOLATILE_MEMORY_CHANGED_2",
        "SECURITY_ISSUE",
        "WRONG_LENGTH",
        "FUNCTIONS_IN_CLASS_NOT_SUPPORTED",
        "ERROR_COMMAND_NOT_ALLOWED",
        "WRONG_PARAMETERS_P1_P2",
        "WRONG_PARAMETERS_P1_P2_NO_INFO",
        "WRONG_LE_FIELD",
        "INSTRUCTION_CODE_INVALID",
        "CLASS_NOT_SUPPORTED",
        "NO_PRECISE_DIAGNOSIS",
        "SUCCESS"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SW1'
        QtMocHelpers::EnumData<SW1>(1, 1, QMC::EnumIsScoped).add({
            {    2, SW1::UNKNOWN },
            {    3, SW1::MORE_DATA_AVAILABLE },
            {    4, SW1::NONVOLATILE_MEMORY_UNCHANGED_1 },
            {    5, SW1::NONVOLATILE_MEMORY_CHANGED_1 },
            {    6, SW1::NONVOLATILE_MEMORY_UNCHANGED_2 },
            {    7, SW1::NONVOLATILE_MEMORY_CHANGED_2 },
            {    8, SW1::SECURITY_ISSUE },
            {    9, SW1::WRONG_LENGTH },
            {   10, SW1::FUNCTIONS_IN_CLASS_NOT_SUPPORTED },
            {   11, SW1::ERROR_COMMAND_NOT_ALLOWED },
            {   12, SW1::WRONG_PARAMETERS_P1_P2 },
            {   13, SW1::WRONG_PARAMETERS_P1_P2_NO_INFO },
            {   14, SW1::WRONG_LE_FIELD },
            {   15, SW1::INSTRUCTION_CODE_INVALID },
            {   16, SW1::CLASS_NOT_SUPPORTED },
            {   17, SW1::NO_PRECISE_DIAGNOSIS },
            {   18, SW1::SUCCESS },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10governikus7EnumSW1E_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10governikus7EnumSW1E =
    governikus::EnumSW1::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus7EnumSW1E_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10governikus7EnumSW1E =
    qt_staticMetaObjectContent_ZN10governikus7EnumSW1E.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10governikus7EnumSW1E =
    qt_staticMetaObjectContent_ZN10governikus7EnumSW1E.relocatingData;

Q_CONSTINIT const QMetaObject governikus::EnumSW1::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10governikus7EnumSW1E.stringdata,
    qt_staticMetaObjectStaticContent_ZN10governikus7EnumSW1E.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10governikus7EnumSW1E.metaTypes,
    nullptr
} };

QT_WARNING_POP
