/**
 * Copyright (c) 2016-2026 Governikus GmbH & Co. KG, Germany
 */

#include "DiagnosisController.h"


using namespace governikus;


void DiagnosisController::getPcscInfo(QList<DiagnosisContext::ComponentInfo>& pComponents,
		QList<DiagnosisContext::ComponentInfo>& pDrivers)
{
	Q_UNUSED(pComponents)
	Q_UNUSED(pDrivers)
}
