/****************************************************************************
** Meta object code from reading C++ file 'StateProcessIfdMessages.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/workflows/ifd/states/StateProcessIfdMessages.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'StateProcessIfdMessages.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::StateProcessIfdMessages::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::StateProcessIfdMessages",
        "fireEstablishPaceChannel",
        "",
        "fireEstablishPaceChannelPinPad",
        "fireModifyPin",
        "fireModifyPinPinPad",
        "fireSecureMessagingStopped",
        "onMessageHandlerAdded",
        "QSharedPointer<ServerMessageHandler>",
        "pHandler",
        "onClosed",
        "onConnectedChanged",
        "pConnected",
        "onReaderPropertiesUpdated",
        "ReaderInfo",
        "pInfo",
        "onCardConnected",
        "onDisplayTextChanged",
        "pDisplayText",
        "onModifyPin",
        "QSharedPointer<const IfdModifyPin>",
        "pMessage",
        "QSharedPointer<CardConnection>",
        "pConnection",
        "onEstablishPaceChannel",
        "QSharedPointer<const IfdEstablishPaceChannel>",
        "onCardDisconnected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireEstablishPaceChannel'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireEstablishPaceChannelPinPad'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireModifyPin'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireModifyPinPinPad'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSecureMessagingStopped'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onMessageHandlerAdded'
        QtMocHelpers::SlotData<void(const QSharedPointer<ServerMessageHandler> &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'onClosed'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onConnectedChanged'
        QtMocHelpers::SlotData<void(bool) const>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'onReaderPropertiesUpdated'
        QtMocHelpers::SlotData<void(const ReaderInfo &) const>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'onCardConnected'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDisplayTextChanged'
        QtMocHelpers::SlotData<void(const QString &) const>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'onModifyPin'
        QtMocHelpers::SlotData<void(const QSharedPointer<const IfdModifyPin> &, const QSharedPointer<CardConnection> &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'onEstablishPaceChannel'
        QtMocHelpers::SlotData<void(const QSharedPointer<const IfdEstablishPaceChannel> &, const QSharedPointer<CardConnection> &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'onCardDisconnected'
        QtMocHelpers::SlotData<void() const>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<StateProcessIfdMessages, qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::StateProcessIfdMessages::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractState::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>.metaTypes,
    nullptr
} };

void governikus::StateProcessIfdMessages::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StateProcessIfdMessages *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireEstablishPaceChannel(); break;
        case 1: _t->fireEstablishPaceChannelPinPad(); break;
        case 2: _t->fireModifyPin(); break;
        case 3: _t->fireModifyPinPinPad(); break;
        case 4: _t->fireSecureMessagingStopped(); break;
        case 5: _t->onMessageHandlerAdded((*reinterpret_cast<std::add_pointer_t<QSharedPointer<ServerMessageHandler>>>(_a[1]))); break;
        case 6: _t->onClosed(); break;
        case 7: _t->onConnectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->onReaderPropertiesUpdated((*reinterpret_cast<std::add_pointer_t<ReaderInfo>>(_a[1]))); break;
        case 9: _t->onCardConnected(); break;
        case 10: _t->onDisplayTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->onModifyPin((*reinterpret_cast<std::add_pointer_t<QSharedPointer<const IfdModifyPin>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[2]))); break;
        case 12: _t->onEstablishPaceChannel((*reinterpret_cast<std::add_pointer_t<QSharedPointer<const IfdEstablishPaceChannel>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[2]))); break;
        case 13: _t->onCardDisconnected(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<ServerMessageHandler> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StateProcessIfdMessages::*)()>(_a, &StateProcessIfdMessages::fireEstablishPaceChannel, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateProcessIfdMessages::*)()>(_a, &StateProcessIfdMessages::fireEstablishPaceChannelPinPad, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateProcessIfdMessages::*)()>(_a, &StateProcessIfdMessages::fireModifyPin, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateProcessIfdMessages::*)()>(_a, &StateProcessIfdMessages::fireModifyPinPinPad, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (StateProcessIfdMessages::*)()>(_a, &StateProcessIfdMessages::fireSecureMessagingStopped, 4))
            return;
    }
}

const QMetaObject *governikus::StateProcessIfdMessages::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::StateProcessIfdMessages::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus23StateProcessIfdMessagesE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "GenericContextContainer<IfdServiceContext>"))
        return static_cast< GenericContextContainer<IfdServiceContext>*>(this);
    return AbstractState::qt_metacast(_clname);
}

int governikus::StateProcessIfdMessages::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractState::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void governikus::StateProcessIfdMessages::fireEstablishPaceChannel()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void governikus::StateProcessIfdMessages::fireEstablishPaceChannelPinPad()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void governikus::StateProcessIfdMessages::fireModifyPin()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void governikus::StateProcessIfdMessages::fireModifyPinPinPad()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void governikus::StateProcessIfdMessages::fireSecureMessagingStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
