/**
 * Copyright (c) 2018-2026 Governikus GmbH & Co. KG, Germany
 */

#pragma once

#include "MsgHandler.h"

namespace governikus
{

class MsgHandlerLog
	: public MsgHandler
{
	public:
		MsgHandlerLog();
};


} // namespace governikus
