/****************************************************************************
** Meta object code from reading C++ file 'ServerMessageHandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../AusweisApp-2.5.1/src/ifd/base/ServerMessageHandler.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ServerMessageHandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto governikus::ServerMessageHandler::qt_create_metaobjectdata<qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "governikus::ServerMessageHandler",
        "fireCardConnected",
        "",
        "QSharedPointer<CardConnection>",
        "pConnection",
        "fireDisplayTextChanged",
        "pDisplayText",
        "fireEstablishPaceChannel",
        "QSharedPointer<const IfdEstablishPaceChannel>",
        "pMessage",
        "fireModifyPin",
        "QSharedPointer<const IfdModifyPin>",
        "fireCardDisconnected",
        "fireClosed",
        "fireSecureMessagingStopped",
        "fireNameChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fireCardConnected'
        QtMocHelpers::SignalData<void(const QSharedPointer<CardConnection> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireDisplayTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'fireEstablishPaceChannel'
        QtMocHelpers::SignalData<void(const QSharedPointer<const IfdEstablishPaceChannel> &, const QSharedPointer<CardConnection> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireModifyPin'
        QtMocHelpers::SignalData<void(const QSharedPointer<const IfdModifyPin> &, const QSharedPointer<CardConnection> &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 9 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireCardDisconnected'
        QtMocHelpers::SignalData<void(const QSharedPointer<CardConnection> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'fireClosed'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireSecureMessagingStopped'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fireNameChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ServerMessageHandler, qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject governikus::ServerMessageHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>.metaTypes,
    nullptr
} };

void governikus::ServerMessageHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ServerMessageHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fireCardConnected((*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[1]))); break;
        case 1: _t->fireDisplayTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->fireEstablishPaceChannel((*reinterpret_cast<std::add_pointer_t<QSharedPointer<const IfdEstablishPaceChannel>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[2]))); break;
        case 3: _t->fireModifyPin((*reinterpret_cast<std::add_pointer_t<QSharedPointer<const IfdModifyPin>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[2]))); break;
        case 4: _t->fireCardDisconnected((*reinterpret_cast<std::add_pointer_t<QSharedPointer<CardConnection>>>(_a[1]))); break;
        case 5: _t->fireClosed(); break;
        case 6: _t->fireSecureMessagingStopped(); break;
        case 7: _t->fireNameChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSharedPointer<CardConnection> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)(const QSharedPointer<CardConnection> & )>(_a, &ServerMessageHandler::fireCardConnected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)(const QString & )>(_a, &ServerMessageHandler::fireDisplayTextChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)(const QSharedPointer<const IfdEstablishPaceChannel> & , const QSharedPointer<CardConnection> & )>(_a, &ServerMessageHandler::fireEstablishPaceChannel, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)(const QSharedPointer<const IfdModifyPin> & , const QSharedPointer<CardConnection> & )>(_a, &ServerMessageHandler::fireModifyPin, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)(const QSharedPointer<CardConnection> & )>(_a, &ServerMessageHandler::fireCardDisconnected, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)()>(_a, &ServerMessageHandler::fireClosed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)()>(_a, &ServerMessageHandler::fireSecureMessagingStopped, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerMessageHandler::*)()>(_a, &ServerMessageHandler::fireNameChanged, 7))
            return;
    }
}

const QMetaObject *governikus::ServerMessageHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *governikus::ServerMessageHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10governikus20ServerMessageHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int governikus::ServerMessageHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void governikus::ServerMessageHandler::fireCardConnected(const QSharedPointer<CardConnection> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void governikus::ServerMessageHandler::fireDisplayTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void governikus::ServerMessageHandler::fireEstablishPaceChannel(const QSharedPointer<const IfdEstablishPaceChannel> & _t1, const QSharedPointer<CardConnection> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void governikus::ServerMessageHandler::fireModifyPin(const QSharedPointer<const IfdModifyPin> & _t1, const QSharedPointer<CardConnection> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void governikus::ServerMessageHandler::fireCardDisconnected(const QSharedPointer<CardConnection> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void governikus::ServerMessageHandler::fireClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void governikus::ServerMessageHandler::fireSecureMessagingStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void governikus::ServerMessageHandler::fireNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
