#ifndef AusweisAppDiagnosis_AusweisAppUiQml_H
#define AusweisAppDiagnosis_AusweisAppUiQml_H
#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#include <DiagnosisModel.h>
#include <SectionModel.h>
struct DiagnosisModelForeign{
    Q_GADGET
    QML_FOREIGN(governikus::DiagnosisModel)
    QML_NAMED_ELEMENT(DiagnosisModel)
};

struct SectionModelForeign{
    Q_GADGET
    QML_FOREIGN(governikus::SectionModel)
    QML_NAMED_ELEMENT(SectionModel)
    QML_UNCREATABLE("Used by Diagnosis(Model|View) only")
};

#endif // AusweisAppDiagnosis_AusweisAppUiQml_H
